<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 7d142f7c469d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 11:26 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_folder_document extends we_folder_base{
	/** @var string database table in which the object is stored */
	public $Table = FILE_TABLE;

	/** @var string ContentType of the Object  */
	public $ContentType = we_base_ContentTypes::FOLDER_DOCUMENT;

	/** @var ?string */
	protected $urlMap = null;

	public function __construct($id = 0){
		parent::__construct($id);
	}

	protected function hasCopyPerm(): bool{
		return we_base_permission::hasPerm([we_base_perm::WORK_FOLDERS_COPY]);
	}

	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		if($this->checkMovePathExists()){
			return false;
		}
		return parent::we_save($resave, $skipHook);
	}

	/**
	 * Create the folders on the server in the public and the site directory.
	 * Note: for internal use
	 *
	 * @return boolean true on success, false in case of failure
	 */
	protected function saveToServer(): bool{
		$path = $this->getPath();
		// creates the folder on the local machine in the root-dir
		if(!we_base_file::createLocalFolderByPath(WEBEDITION_PATH . '..' . $path ?: '')){
			return false;
		}
		$oldText = basename($this->OldPath ?: $this->Path);
		if($this->urlMap){
			we_base_file::checkAndMakeFolder(WE_DOCUMENT_ROOT . WE_THUMBNAIL_DIRECTORY);
			foreach([
			WEBEDITION_PATH => WE_DOCUMENT_ROOT . $path . rtrim(WEBEDITION_DIR, '/'),
			WE_DOCUMENT_ROOT . $path => WE_DOCUMENT_ROOT . $path . '/' . $this->Filename,
			WE_DOCUMENT_ROOT . WE_THUMBNAIL_DIRECTORY => WE_DOCUMENT_ROOT . $path . WE_THUMBNAIL_DIRECTORY
			] as $dest => $link){
				we_base_file::makeSymbolicLink((string) $dest, $link);
			}
			//make sure we have a symbolic dir
		} else {
			//cleanup
			foreach([
			WE_DOCUMENT_ROOT . $path . rtrim(WEBEDITION_DIR, '/'),
			WE_DOCUMENT_ROOT . $path . WE_THUMBNAIL_DIRECTORY,
			WE_DOCUMENT_ROOT . $path . '/' . $oldText
			] as $p){
				if(is_link($p)){
					we_base_file::delete($p);
				}
			}
		}

		// creates the folder on the local machine in the root-dir+site-dir
		if(!we_base_file::createLocalFolderByPath(WE_DOCUMENT_ROOT . WE_SITE_DIR . $path)){
			return false;
		}
		return true;
	}

	protected function initStatic(): void{
		parent::initStatic();
		$this->persistent_slots = array_merge($this->persistent_slots, [
			'urlMap' => we_base_request::STRING,
		]);
	}

	function formPath(bool $disablePath = false, bool $notSetHot = false, string $extra = ''): string{
		return parent::formPath($disablePath, $notSetHot, $extra) .
			($this->userCanChangePath() && $this->ID && we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) ?
			$this->formInput('', 'urlMap', g_l('weClass', '[urlMap]'), 0, 255, ['onchange' => "_EditorFrame.setEditorIsHot(true);"]) :
			'')
		;
	}

	protected function moveAtServer(): bool{
		// renames the folder on the local machine in the root-dir
		$path = $this->getRealPath();
		$oldPath = $this->getRealPath(true);
		if(!file_exists($path) && !file_exists($oldPath)){
			t_e('old path doesn\'t exist', $oldPath);
			return false;
		}
		// renames the folder on the local machine in the root-dir+site-dir
		$sitepath = WE_DOCUMENT_ROOT . WE_SITE_DIR . substr($this->Path, 1);
		$siteoldPath = WE_DOCUMENT_ROOT . WE_SITE_DIR . substr($this->OldPath, 1);
		if(file_exists($sitepath) && file_exists($siteoldPath)){
			t_e('old and new dir exists!', $sitepath, $siteoldPath);
			return false;
		}
		if(!file_exists($sitepath) && !file_exists($siteoldPath)){
			t_e('old directory doesn\'t exist!', $oldPath);
			return false;
		}

		if(!file_exists($path) && file_exists($oldPath)){
			if(!rename($oldPath, $path)){
				return false;
			}
		}

		//we are responsible for site dir!
		if(!file_exists($sitepath) && file_exists($siteoldPath)){
			if(!rename($siteoldPath, $sitepath)){
				//move back other dir!
				rename($path, $oldPath);
				return false;
			}
		}

		if($this->urlMap){
			$oldText = basename($oldPath);
			if(is_link(($p = $path . '/' . $oldText))){
				we_base_file::delete($p);
			}
			we_base_file::makeSymbolicLink($path, $path . '/' . $this->Filename);
		}

		return true;
	}

}
