<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e9f231e9d4ea $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-04-08 23:14 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_fragment_del extends we_fragment_base{
	/** @var we_database_base */
	private $db;

	/** @var string */
	private $table;

	function __construct(string $name, string $table){
		$this->db = new DB_WE();
		$this->table = $table;
		parent::__construct($name, 10);
	}

	protected function init(): void{
		if(empty($_SESSION['weS']['fragDel']['todel'])){
			return;
		}
		$filesToDel = we_base_request::filterIntList($_SESSION['weS']['fragDel']['todel']);
		$this->alldata = $this->db->getAllq('SELECT ID FROM ' . FILE_TABLE . ' WHERE ID IN (' . implode(',', $filesToDel) . ') ORDER BY IsFolder, LENGTH(Path) DESC', true);
	}

	protected function doTask(): void{
		we_base_delete::$foldersNotDel = [];
		$currentID = we_base_request::int('currentID', 0);
		$currentParents = [];
		we_base_util::readParents($currentID, $currentParents, $this->table);

		we_base_delete::deleteEntry($this->data, $this->table, false);
		if(we_base_delete::$foldersNotDel){
			$_SESSION['weS']['fragDel']['we_not_deleted_entries'][] = we_base_delete::$foldersNotDel[0];
		}
		if($this->data == $currentID){
			$_SESSION['weS']['fragDel']['we_go_seem_start'] = true;
		}
	}

	protected function updateProgressBar(): void{
		$this->jsCmd->addCmd('setProgress', [
			'progress' => ((int) ((100 / count($this->alldata)) * (1 + $this->currentTask))),
			'name' => 'pb1',
			'text' => sprintf(g_l('backup', '[delete_entry]'), we_base_util::shortenPath(id2path($this->data, $this->table), 70)),
			'win' => 'delmain'
		]);
	}

	protected function finish(): void{
		if($_SESSION['weS']['fragDel']['we_not_deleted_entries']){
			$this->jsCmd->addMsg(sprintf(g_l('alert', '[folder_not_empty]'), implode("\n", $_SESSION['weS']['fragDel']['we_not_deleted_entries']) . "\n"), we_base_util::WE_MESSAGE_ERROR);
		} else {
			$this->jsCmd->addMsg(g_l('alert', '[delete][ok]'), we_base_util::WE_MESSAGE_NOTICE);
		}

		if(we_editor_functions::isSeeMode() && $_SESSION['weS']['fragDel']['we_go_seem_start']){
			$this->jsCmd->addCmd('start_multi_editor');
		}
		$this->jsCmd->addCmd('close');
		unset($_SESSION['weS']['fragDel']);
	}

}
