<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 4db68978af03 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-11-26 01:18 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_gui_progressBar{
	const TOP = 0;
	const RIGHT = 1;
	const BOTTOM = 2;
	const LEFT = 3;

	/** @var float */
	private $progress = 0;

	/** @var array<int,array{name:string,place:string}> */
	private $texts = [];

	/** @var int */
	private $stud_len = 100;

	/** @var int */
	private $progressTextPlace = self::RIGHT;

	/** @var string */
	private $name = '';

	/** @var array<string,array> */
	private static $studLen = [];

	public function __construct(float $progress = 0, int $stud_len = 100, string $name = ''){
		$this->setProgress($progress);
		$this->stud_len = $stud_len;
		$this->name = $name;
		self::$studLen[$name] = [
			'koef' => $stud_len / 100
		];
	}

	public static function getJSCode(): string{
		return we_html_element::jsScript(JS_DIR . 'we_progressBar.js', '', ['id' => 'loadVarProgressBar', 'data-progress' => setDynamicVar(self::$studLen)]);
	}

	public function addText(string $text = '', int $place = self::TOP, string $id = ''): self{
		$this->texts[] = ['name' => $id, 'text' => $text, 'place' => $place];
		return $this;
	}

	private function setProgress(float $progress = 0): void{
		$this->progress = min(100, $progress);
	}

	public function setProgressTextPlace(int $place = 0): void{
		$this->progressTextPlace = $place;
	}

	public function getHTML(string $class = '', string $style = ''): string{
		$left = $right = $top = $bottom = '';

		$this->addText((string) $this->progress . '%', $this->progressTextPlace, 'progress_text');
		$this->addText('', self::BOTTOM, 'elapsedTime');

		foreach($this->texts as $text){
			$tmp = '<td class="small bold"><span class="cutTextLeft" ' . ($text['name'] ? 'id="' . $text['name'] . $this->name . '" ' : "") . '>' . $text["text"] . '</span></td>';
			switch($text['place']){
				case self::TOP:
					$top .= $tmp;
					break;
				case self::RIGHT:
					$right .= $tmp;
					break;
				case self::BOTTOM:
					$bottom .= $tmp;
					break;
				case self::LEFT:
					$left .= $tmp;
					break;
			}
		}


		$progress_len = ($this->stud_len / 100) * $this->progress;
		$rest_len = $this->stud_len - $progress_len;

		return '<table class="default progressbar ' . $class . '" style="' . $style . '">' .
			($top ? '<tr>' . $top . '</tr>' : '') .
			'<tr>' . $left .
			'<td style="padding-right:5px;"><div id="progress_image' . $this->name . '" class="progress_image" style="width:' . $progress_len . 'px;"></div><div id="progress_image_bg' . $this->name . '" class="progress_image_bg" style="width:' . $rest_len . 'px;"></div></td>' .
			$right .
			'</tr>' .
			($bottom ? '<tr>' . $bottom . '</tr>' : '' )
			. '</table>';
	}

}
