<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 07a9a048026d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-18 00:22 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class
 * @desc    class for tag <we:listview>
 *
 */
class we_listview_document extends we_listview_base{
	/** @var int[] doctype */
	var $docType = [];

	/** @var bool set to true when a search should be case sensitive */
	var $casesensitive = false;

	/** @var string */
	var $contentTypes = '';

	/** @var bool regard subfolders */
	var $subfolders = true;

	/** @var string[] of Languages */
	protected $languages = [];

	/** @var bool */
	var $numorder = false; // #3846

	/** @var int */
	public $triggerID = 0;

	/** @var string[] */
	protected $joins = [];

	/** @var string */
	protected $table = FILE_TABLE;

	/** @var string */
	protected $group = '';

	/** @var array<int,array<string,mixed>> */
	protected $customerArray = [];

	/**
	 *
	 * constructor of class
	 *
	 * @param   string $name            name of listview
	 * @param   int $rows          number of rows to display per page
	 * @param   int $offset        start offset of first page
	 * @param   string $order         field name(s) to order by
	 * @param   bool $desc          set to true, if order should be descendend
	 * @param   int[] $docType       doctype
	 * @param   string[] $cats          comma separated categories
	 * @param   bool $catOr         set to true if it should be an "OR condition"
	 * @param   bool $casesensitive boolean - set to true when a search should be case sensitive
	 * @param   int[] $workspaceID   commaseperated list of id's of workspace
	 * @param   string $contentTypes    - contenttypes of documents (image,text ...)
	 * @param   int $cols   		  to display a table this is the number of cols
	 * @param   bool $searchable 	  if false then show also documents which are not marked as searchable
	 * @param string $condition
	 * @param string $calendar
	 * @param string $datefield
	 * @param string $date
	 * @param string $weekstart
	 * @param int[] $categoryids
	 * @param int[] $customers
	 * @param int[] $id all ids to be found
	 * @param string[] $languages
	 */
	function __construct(string $name, int $rows, int $offset, string $order, bool $desc, array $docType, array $cats, bool $catOr, bool $casesensitive, array $workspaceID, string $contentTypes, int $cols, bool $searchable, string $condition, string $calendar, string $datefield, string $date, string $weekstart, array $categoryids, string $customerFilterType, bool $subfolders, array $customers, array $id, array $languages, bool $numorder, bool $hidedirindex, int $triggerID){
		parent::__construct($name, $rows, $offset, $order, $desc, $cats, $catOr, $workspaceID, $cols, $calendar, $datefield, $date, $weekstart, $categoryids, $customerFilterType, $id);
		$this->docType = $docType;
		$this->casesensitive = $casesensitive;
		$this->contentTypes = $contentTypes;
		$this->searchable = $searchable;
		$this->subfolders = $subfolders;
		$this->customers = $customers;
		$this->languages = $languages;
		$this->condition = $condition;
		$this->numorder = $numorder;
		$this->hidedirindex = $hidedirindex;
		$this->triggerID = $triggerID;

		if($this->table === VFILE_TABLE){
			$id = $this->id = [];
		}

		$this->group .= ($this->group ? ',' : '') . 'f.ID';

		$calendar_select = $calendar_where = '';

		if($calendar){
			$this->fetchCalendar($condition, $calendar_select, $calendar_where);
		}

		$cond_where = // #3763
			($this->condition != '' && ($condition_sql = $this->makeConditionSql($this->condition)) ? '(' . $condition_sql . ')' : '');

		$where_lang = ($this->languages ? 'f.Language IN("","' . implode('","', array_map('escape_sql_query', $this->languages)) . '") ' : '');

		if(str_contains(strtolower($this->order), ' desc')){//was #3849
			$this->order = str_ireplace(' desc', '', $this->order);
			$this->desc = true;
		}

		$this->order = trim($this->order);
		$random = false;

		$order = [];
		$tmpOrder = explode(',', $this->order);
		foreach($tmpOrder as $ord){
			switch(strtolower(trim($ord))){
				case self::ID_FIELD:
					$order[] = 'f.ID' . ($this->desc ? ' DESC' : '');
					break;
				case self::DATECREATION_FIELD:
					$order[] = 'f.Creation_Date' . ($this->desc ? ' DESC' : '');
					break;
				case self::PATH_FIELD:
					$order[] = 'f.Path' . ($this->desc ? ' DESC' : '');
					break;
				case self::FILENAME_FIELD:
					$order[] = 'f.Text' . ($this->desc ? ' DESC' : '');
					break;
				case self::DATEMOD_FIELD:
					$order[] = 'f.Mod_Date' . ($this->desc ? ' DESC' : '');
					break;
				case self::DATEPUBLISHED_FIELD:
					$order[] = 'f.Publish_Date' . ($this->desc ? ' DESC' : '');
					break;
				case self::RANDOM_FIELD:
					$random = true;
					$order[] = 'RANDOM';
				case '':
					break;
				case 'vfile':
					$order[] = 'fl.position';
					break;
				default:
					$cnt = count($this->joins);
					$this->joins[] = ' LEFT JOIN ' . CONTENT_TABLE . ' cc' . $cnt . ' ON (cc' . $cnt . '.DID=f.ID AND cc' . $cnt . '.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND cc' . $cnt . '.nHash=x\'' . md5($ord) . '\')';
					if($this->search){
						$order[] = 'ranking';
					}
					//FIXME how to know if we sort by dat or bdid?! otherwise numeric sort is not correct unless numorder is set
					$order[] = ($this->numorder ? 'CAST(' : '') . 'COALESCE(cc' . $cnt . '.Dat,cc' . $cnt . '.BDID)' . ($this->numorder ? ' AS UNSIGNED) ' : '') . ($this->desc ? ' DESC' : '');
					break;
			}
		}
		$orderstring = $order ? ' ORDER BY ' . implode(',', $order) : '';
		$joinstring = implode('', $this->joins);

		$cat_tail = ($this->cats || $this->categoryids ? we_category_category::getCatSQLTail($this->cats, 'f', $this->catOr, 'Category', $this->categoryids) : '');

		$ws_where = '';

		if($this->contentTypes){
			$this->contentTypes = str_replace(['img', 'wepage', 'binary'], [we_base_ContentTypes::IMAGE, we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::APPLICATION,
				we_base_ContentTypes::PDF], $this->contentTypes);
			$CtArr = explode(',', $this->contentTypes);
			$ctTail = ($CtArr ? 'f.ContentType IN ("' . implode('","', array_map('escape_sql_query', $CtArr)) . '")' : '');
		} else {
			$ctTail = '';
		}
		$sql_tail = (we_modules_info::isActive(we_modules_info::CUSTOMER) ? we_customer_documentFilter::getConditionForListviewQuery($this->customerFilterType, $this, 0, $id) : '');

		$cust = '';
		if($this->customers){
			$cust = implode(',', $this->customers);
			$this->customerArray = $this->DB_WE->getAllFirstq('SELECT ID,* FROM ' . CUSTOMER_TABLE . ' WHERE ID IN (' . $cust . ')', true, MYSQLI_ASSOC);

			$cust = '(f.WebUserID IN(' . $cust . '))';
		}

		$ranking = '0';
		if($this->search){
			if($this->workspaceID){
				$folders = getAllFoldersRecursive($this->workspaceID);
				if($folders){
					$ws_where = 'f.isFolder=0 AND f.ParentID IN(' . implode(',', $folders) . ')';
				}
			}
			$bedingungen = preg_split('/ +/', $this->search);

			$spalten = [($this->casesensitive ? 'BINARY ' : '') . ($this->searchable ? 'i.Text' : 'ci.Dat')];
			$bedingungen_sql = $bedingungen2_sql = [];

			foreach($bedingungen as $v1){
				if(preg_match('|^[-\+]|', $v1)){
					$not = (preg_match('/^-/', $v1));
					$bed = preg_replace('/^[-\+]/', '', $v1);
					$klammer = [];
					reset($spalten);
					foreach($spalten as $v){
						$klammer[] = sprintf('%s LIKE "%%%s%%"', $v, addslashes($bed));
					}
					if($not){
						$bedingungen_sql[] = ' NOT (' . implode(' OR ', $klammer) . ')';
					} else {
						$bedingungen_sql[] = '(' . implode(' OR ', $klammer) . ')';
					}
				} else {
					$klammer = [];
					foreach($spalten as $v){
						$klammer[] = sprintf('%s LIKE "%%%s%%"', $v, addslashes($v1));
					}
					$bed2 = '(' . implode(' OR ', $klammer) . ')';
					$ranking .= '-' . $bed2;
					$bedingungen2_sql[] = $bed2;
				}
			}

			if($bedingungen2_sql){
				$bedingung_sql = '(' .
					implode(' OR ', $bedingungen2_sql) . ')' .
					(empty($bedingungen_sql) ? '' : ' AND ' . implode(' AND ', $bedingungen_sql));
			} else if($bedingungen_sql){
				$bedingung_sql = implode(' AND ', $bedingungen_sql);
			} else {
				$bedingung_sql = '';
			}


			$extraSelect = ',' . ($random ? ' RAND() AS RANDOM ' : $ranking . ' AS ranking ') . $calendar_select;
			$limit = (($this->maxItemsPerPage) ? (' LIMIT ' . abs($this->start) . ',' . abs($this->maxItemsPerPage)) : '');
		} else {
			$bedingung_sql = '';
			if($this->workspaceID){
				$workspaces = $this->workspaceID;
				if($this->subfolders){ // all entries with given parentIds
					$workspacePaths = array_map('escape_sql_query', id2pathA($workspaces, FILE_TABLE));
					if($workspacePaths){
						$workspaces = array_unique(array_merge($workspaces, $this->DB_WE->getAllq('SELECT ID FROM ' . FILE_TABLE . ' WHERE IsFolder=1 AND (Path REGEXP "^(' . implode('|', $workspacePaths) . ')/")', true)));
					}
				}
				$ws_where = '(f.ParentID IN (' . implode(', ', $workspaces) . '))';
			}
			$extraSelect = ($random ? ', RAND() AS RANDOM' : '') . $calendar_select;
			$limit = (($rows > 0) ? (' LIMIT ' . abs($this->start) . ',' . abs($this->maxItemsPerPage)) : "");
		}
		$where = implode(' AND ', array_filter([
				'default' => 'f.IsFolder=0 AND f.IsPublished=1',
				'search' => ($this->searchable ? ' f.IsSearchable=1' : 1),
				'lang' => $where_lang,
				'cond' => $cond_where,
				'beding' => $bedingung_sql,
				'docT' => ($this->docType ? 'f.DocType IN(' . implode(',', $this->docType) . ')' : ''),
				'cal' => $calendar_where,
				'ws' => $ws_where,
				'cat' => $cat_tail,
				'contentType' => $ctTail,
				'cust' => $cust,
				'id' => $this->getIdQuery('f.ID'),
			])) . ' ' . $sql_tail .
			' GROUP BY ' . $this->group;

		$all = $this->DB_WE->getAllFirstq(
			'SELECT
	f.ID,f.WebUserID' . $extraSelect . '
FROM ' .
			FILE_TABLE . ' f JOIN ' .
			CONTENT_TABLE . ' c ON (f.ID=c.DID AND c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '") ' . $joinstring .
			($this->search ? ' JOIN ' .
			($this->searchable ?
			INDEX_TABLE . ' i ON (i.ID=f.ID AND i.ClassID=0) ' :
			CONTENT_TABLE . ' ci ON (f.ID=ci.DID AND ci.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND ci.Type IN ("input","txt")) ') : '') . '
WHERE ' .
			$where . ' ' .
			$orderstring . $limit, true, MYSQLI_ASSOC
		);
		'@phan-var array<int,array{WebUserID:int,RANDOM?:int,ranking?:int,Calendar?:string}> $all';
		$this->anz = count($all);
		$this->IDs = array_keys($all);

		if($calendar_select){
			foreach($all as $id => $cur){
				$this->calendar_struct['storage'][(int) $id] = new DateTime($cur['Calendar']);
			}
		}
		unset($all);
		$this->DB_WE->query(
			'SELECT f.ID,f.WebUserID' .
			($random ? ',RAND() AS RANDOM' : ($this->search ? ',' . $ranking . ' AS ranking' : '')) .
			' FROM ' . FILE_TABLE . ' f JOIN ' . CONTENT_TABLE . ' c ON (f.ID=c.DID AND c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '") ' .
			($this->search ? ' JOIN ' .
				($this->searchable ?
					INDEX_TABLE . ' i ON (i.ID=f.ID AND i.ClassID=0) ' :
					CONTENT_TABLE . ' ci ON (f.ID=ci.DID AND ci.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND ci.Type IN ("input","txt")) ') : '') .
			$joinstring .
			' WHERE ' . $where . ' ' . $orderstring);
//FIXME use window function COUNT(*) OVER() if mariadb 10.2/mysql 8
		$this->anz_all = $this->DB_WE->num_rows();
		if($calendar != ''){
			$this->postFetchCalendar();
		}
	}

	public function next_record(): bool{
		if($this->count < $this->anz){
			$count = $this->count;
			$fetch = false;
			if($this->calendar_struct){
				parent::next_record();
				$count = $this->calendar_struct['pos'];
				$fetch = $this->calendar_struct['forceFetch'];
			}

			if(!$this->calendar_struct || $fetch){
				$id = (int) $this->IDs[$count];
				$data = getHash('SELECT ID,ParentID,Text,IsFolder,ContentType,CreationDate,Creation_Date,Creation_Date,Mod_Date,ModDate,Path,TemplateID,Filename,Extension,IsDynamic,IsSearchable,DocType,Category,Publish_Date,Published,CreatorID,ModifierID,RestrictOwners,Owners,OwnersReadOnly,Language,WebUserID,InGlossar
FROM ' .
					FILE_TABLE .
					' WHERE ID=' . $id);

				$this->Record = array_change_key_case(array_merge($this->DB_WE->getAllFirstq('SELECT
	c.Name,
	COALESCE(c.Dat,c.BDID) AS data
FROM ' .
							CONTENT_TABLE . ' c
WHERE
	c.DID=' . $id . ' AND
	c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '"', false), self::preFixArrayKeys($data)));

				if(!empty($this->Record[we_base_variants::ELEMENT_NAME])){
					$variants = is_string($this->Record[we_base_variants::ELEMENT_NAME]) ? we_unserialize((string) $this->Record[we_base_variants::ELEMENT_NAME]) : [];
					$this->Record[self::SHOPVARIANTS_FIELD] = (!empty($variants) && is_array($variants) ? count($variants) : 0);
				} else {
					$this->Record[self::SHOPVARIANTS_FIELD] = 0; //check this for global variants
				}

				if($this->customers && $this->Record[self::CUSTOMER_FIELD] && !empty($this->customerArray[$this->Record[self::CUSTOMER_FIELD]])){
					$this->Record = array_merge($this->Record, self::preFixArrayKeys($this->customerArray[$this->Record[self::CUSTOMER_FIELD]]));
				}

				$this->count++;
			}

			return true;
		}
		$this->stop_next_row = $this->shouldPrintEndTR();
		if($this->cols && ($this->count <= $this->maxItemsPerPage) && !$this->stop_next_row){
			$this->Record = [];
			$this->count++;
			return true;
		}
		return false;
	}

	public function f(string $key, bool $stripWE = false): string{
		return parent::f($key, $stripWE);
	}

	public function makeConditionSql(string $cond): string{
		//FIXME: a>5 AND b>5 will not work, we have a flat join, so both conditions on different link-values cannot match
		$cond = strtr($cond, ['&gt;' => '>', '&lt;' => '<']);
		$func = function($value){
			return trim($value, " \t\n\r\0\x0B()");
		};
		$arr = array_map($func, preg_split('/( AND | OR | && | \|\| | FIND_IN_SET\()/i', $cond, -1, PREG_SPLIT_NO_EMPTY));
		$opPatterns = ['<>', '!=', '<=', '>=', '=', '<', '>', 'NOT LIKE', 'LIKE', 'NOT IN', '[^_]IN'];
		$funcPatterns = ['FIND_IN_SET'];
		foreach($arr as $exp){
			foreach($opPatterns as $pattern){
				$match = preg_split('/' . $pattern . '/', $exp, -1, PREG_SPLIT_NO_EMPTY);
				if(count($match) > 1){
					$sqlarr = $this->makeFieldOpCondition(trim($match[0]), $pattern, trim($match[1]));
					$cond = str_replace($match[0] . $pattern . $match[1], $sqlarr, $cond);
					break;
				}
			}
			foreach($funcPatterns as $pattern){
				$match = preg_split('/' . $pattern . '/', $exp, -1);
				if(!empty($match[1])){
					[$value, $name] = explode(',', trim($match[1], '( '), 2);
					$sqlarr = $this->makeFieldFuncCondition($pattern, trim($value), trim($name));
					$cond = str_replace($pattern . $match[1], $sqlarr, $cond);
				}
			}
		}
		return $cond;
	}

	private function makeFieldFuncCondition(string $function, string $value, string $name): string{
		return (str_starts_with(strtolower($name), self::PROPPREFIX)) ? //Fix: #9389
			'(' . $function . '(' . $value . ',f.' . substr($name, 3) . '))' :
			'(c.nHash=x\'' . md5($name) . '\' AND ' . $function . '(' . $value . ',COALESCE(c.Dat,c.BDID))';
	}

	private function makeFieldOpCondition(string $name, string $operation, string $value): string{
		return (str_starts_with(strtolower($name), self::PROPPREFIX)) ? //Fix: #9389
			'(f.' . substr($name, 3) . ' ' . $operation . ' ' . $value . ')' :
			'(c.nHash=x\'' . md5($name) . '\' AND COALESCE(c.Dat,c.BDID) ' . $operation . ' ' . $value . ')';
	}

	public function getCustomerRestrictionQuery(int $classID): array{
		return ['', 'cf.modelTable="' . stripTblPrefix(FILE_TABLE) . '" AND '];
	}

	/**
	 *
	 * @return array{id:int,type:string,req:array}
	 */
	public function getShopArticleID(): array{
		return ['id' => (int) $this->f(we_listview_base::ID_FIELD), 'type' => we_shop_shop::DOCUMENT, 'req' => []];
	}

	public function getHrefArray(string $name): array{
		return [
			'int' => $this->f($name . we_base_link::MAGIC_INT_LINK) ?: $this->f(we_weTag_tag::getPostName($name) . we_base_link::MAGIC_INT_LINK),
			'intID' => $this->f($name . we_base_link::MAGIC_INT_LINK_ID) ?: $this->f(we_weTag_tag::getPostName($name) . we_base_link::MAGIC_INT_LINK_ID),
			'extPath' => $this->f($name) ?: $this->f(we_weTag_tag::getPostName($name))
		];
	}

	public function getListviewRequest(string $lvname): array{
		return array_merge(parent::getListviewRequest($lvname), ($this->contentTypes ? ['we_lv_ct_' . $lvname => $this->contentTypes] : []));
	}

}
