<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 6cc64d37551c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-25 11:32 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Class which contains all functions for the
 * rebuild dialog and the rebuild function
 * @static
 */
class we_rebuild_wizard{
	const REBUILD_BTYPE_ALL = 'rebuild_all';
	const REBUILD_BTYPE_FILTER = 'rebuild_filter';
	const REBUILD_BTYPE_DYNAMIC = 'rebuild_dynamic';
	const REBUILD_BTYPE_TEMPLATES = 'rebuild_templates';
	const REBUILD_TYPE_DOCUMENTS = 'rebuild_documents';
	const REBUILD_TYPE_INDEX = 'rebuild_index';
	const REBUILD_TYPE_MEDIALINKS = 'rebuild_medialinks';
	const REBUILD_TYPE_METADATA = 'rebuild_metadata';
	const REBUILD_TYPE_NAVIGATION = 'rebuild_navigation';
	const REBUILD_TYPE_OBJECTS = 'rebuild_objects';
	const REBUILD_TYPE_THUMBNAILS = 'rebuild_thumbnails';

	/** @var we_base_jsCmd */
	private $jsCmd;

	/** @var int */
	private $step;

	public function __construct(){
		$this->jsCmd = new we_base_jsCmd();
		$this->step = we_base_request::int('step', 0);
	}

	/**
	 * returns HTML for the Body Frame
	 *
	 * @return string
	 */
	private function getBody(): string{
		switch($this->step){
			default:
			case 0:
				return $this->getPage($this->getStep0());
			case 1:
				return $this->getPage($this->getStep1());
			case 2:
				return $this->getPage($this->getStep2());
		}
	}

	/**
	 * returns HTML for the Frame with the progress bar
	 *
	 * @return string
	 */
	private function getBusy(): string{
		$dc = we_base_request::int('dc', 0);

		$WE_PB = new we_gui_progressBar(0, ($dc ? 490 : 200));
		$WE_PB->addText(/* g_l('rebuild', '[savingDocument]') */'', we_gui_progressBar::TOP, 'pb1');

		$cancelButton = we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();");
		$refreshButton = we_html_button::create_button(we_html_button::REFRESH, "javascript:parent.wizcmd.location.reload();", '', "", false, false);

		$nextbutdisabled = !(we_base_permission::hasPerm([we_base_perm::REBUILD_ALL, we_base_perm::REBUILD_FILTERD, we_base_perm::REBUILD_OBJECTS, we_base_perm::REBUILD_INDEX,
				we_base_perm::REBUILD_THUMBS, we_base_perm::REBUILD_META]));

		if($dc){
			$pb = we_html_tools::dialogLayout($WE_PB->getHTML(), g_l('rebuild', '[rebuild]'), $refreshButton . $cancelButton);
			$this->jsCmd->addCmd('initProgress');
		} else {
			$content = new we_html_table(['width' => "100%"], 1, 2);
			$content->setCol(0, 0, $WE_PB->getHTML('', 'display:none;'), ['id' => 'progr', 'style' => 'padding-left:1em;text-align:left']);
			$content->setCol(0, 1, we_html_element::span(['id' => 'prev', 'style' => 'padding-left:10px;text-align:right'], we_html_button::create_button(we_html_button::BACK, "javascript:parent.wizbody.handle_event('previous');", '', "", true, false)) .
				we_html_element::span(['id' => 'next', 'style' => "padding-left:10px;text-align:right"], we_html_button::create_button(we_html_button::NEXT, "javascript:parent.wizbody.handle_event('next');", '', "", $nextbutdisabled, false)) .
				we_html_element::span(['id' => 'refresh', 'style' => "display:none; padding-left:10px;text-align:right"], $refreshButton) .
				we_html_element::span(['id' => 'cancel', 'style' => "padding-left:10px;text-align:right"], $cancelButton), ['style' => 'text-align:right']);
			$pb = $content->getHtml();
		}

		return we_html_tools::getHtmlTop(g_l('rebuild', '[rebuild]'), we_html_element::jsScript(JS_DIR . 'rebuildBuzy.js') .
				we_gui_progressBar::getJSCode() .
				$this->jsCmd->getCmds() .
				we_html_element::jsScript(JS_DIR . 'nextButtons.js'), we_html_element::body([
					'style' => 'overflow:hidden', 'class' => ($dc ? 'weDialogBody' : 'editfooter footerWithProgress')], $pb
				)
		);
	}

	/**
	 * returns HTML for the Cmd Frame
	 *
	 * @return string for now it is an empty page
	 */
	private function getCmd(): string{
		return $this->getPage(['', '']);
	}

	/**
	 * returns the HTML for the First Step (0) of the wizard
	 *
	 * @return array
	 */
	private function getStep0(): array{
		$dws = we_users_util::get_def_ws();
		$btype = we_base_request::string('btype', self::REBUILD_BTYPE_ALL);
		$categories = implode(',', we_base_request::intList('categories'));
		$doctypes = implode(',', we_base_request::intList('doctypes'));
		$folders = implode(',', we_base_request::intList('folders', [($dws ?: 0)]));
		$thumbsFolders = implode(',', we_base_request::intList('thumbsFolders', [($dws ?: 0)]));
		$thumbs = implode(',', we_base_request::intList('thumbs'));
		$catAnd = we_base_request::bool('catAnd');
		$metaFolders = we_base_request::string('metaFolders', (string) ($dws ?: ''));
		$metaFields = we_base_request::_(we_base_request::RAW, '_field', []);
		$onlyEmpty = we_base_request::bool('onlyEmpty');

		if(($type = we_base_request::string('type'))){

		} elseif(we_base_permission::hasPerm([we_base_perm::REBUILD_ALL, we_base_perm::REBUILD_FILTERD])){
			$type = self::REBUILD_TYPE_DOCUMENTS;
		} else if(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::REBUILD_OBJECTS])){
			$type = self::REBUILD_TYPE_OBJECTS;
		} else if(we_base_permission::hasPerm([we_base_perm::REBUILD_INDEX])){
			$type = self::REBUILD_TYPE_INDEX;
		} else if(we_base_permission::hasPerm([we_base_perm::REBUILD_THUMBS])){
			$type = self::REBUILD_TYPE_THUMBNAILS;
			/* } else if(we_base_permission::hasPerm([we_base_perm::REBUILD_NAVIGATION])){
			  $type = self::REBUILD_TYPE_NAVIGATION; */
		} else if(we_base_permission::hasPerm([we_base_perm::REBUILD_META])){
			$type = self::REBUILD_TYPE_METADATA;
		} else {
			$type = '';
		}

		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine('', we_html_forms::radiobuttonV(self::REBUILD_TYPE_DOCUMENTS, $type, 'type', g_l('rebuild', '[rebuild_documents][caption]') . ' ' . we_html_tools::alertAttentionBox(g_l('rebuild', '[rebuild_documents][description]'), we_html_tools::TYPE_HELP), true, 'defaultfont', '', (!(we_base_permission::hasPerm([
						we_base_perm::REBUILD_ALL, we_base_perm::REBUILD_FILTERD]))))
		));

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$parts->add(new we_html_multiIconBoxContentLine('', we_html_forms::radiobuttonV(self::REBUILD_TYPE_OBJECTS, $type, 'type', g_l('rebuild', '[rebuild_objects][caption]') . ' ' . we_html_tools::alertAttentionBox(g_l('rebuild', '[rebuild_objects][description]'), we_html_tools::TYPE_HELP), true, 'defaultfont', '', (!we_base_permission::hasPerm([
							we_base_perm::REBUILD_OBJECTS])))));
		}

		$parts->add(new we_html_multiIconBoxContentLine('', we_html_forms::radiobuttonV(self::REBUILD_TYPE_INDEX, $type, 'type', g_l('rebuild', '[rebuild_index][caption]') . ' ' . we_html_tools::alertAttentionBox(g_l('rebuild', '[rebuild_index][description]'), we_html_tools::TYPE_HELP), true, 'defaultfont', '', (!we_base_permission::hasPerm([
							we_base_perm::REBUILD_INDEX])))
			))
			->add(new we_html_multiIconBoxContentLine('', we_html_forms::radiobuttonV(self::REBUILD_TYPE_THUMBNAILS, $type, 'type', g_l('rebuild', '[rebuild_thumbnails][caption]') . ' ' . we_html_tools::alertAttentionBox(g_l('rebuild', '[rebuild_thumbnails][description]'), we_html_tools::TYPE_HELP), true, 'defaultfont', '', (we_base_imageEdit::gd_version() == 0 || (!we_base_permission::hasPerm([
							we_base_perm::REBUILD_THUMBS]))))));

		/*
		  $navRebuildHTML = '<div>' .
		  we_html_forms::radiobutton(self::REBUILD_TYPE_NAVIGATION, $type === self::REBUILD_TYPE_NAVIGATION && we_base_permission::hasPerm([we_base_perm::REBUILD_NAVIGATION])), 'type', g_l('rebuild', '[rebuild_navigation][caption]') . ' ' . we_html_tools::alertAttentionBox(g_l('rebuild', '[rebuild_navigation][description]'), we_html_tools::TYPE_HELP), false, 'defaultfont', '', !we_base_permission::hasPerm([
		  we_base_perm::REBUILD_NAVIGATION])) .
		  '</div><div style="padding:10px 20px;">' .
		  we_html_forms::checkbox('1', false, 'rebuildStaticAfterNavi', g_l('rebuild', '[rebuildStaticAfterNaviCheck]') . ' ' . we_html_tools::alertAttentionBox(g_l('rebuild', '[rebuildStaticAfterNaviHint]'), we_html_tools::TYPE_HELP), false, 'defaultfont', '', true) .
		  '</div>';

		  $parts->add('', 'html' => $navRebuildHTML,]);
		 */
		$metaDataFields = we_metadata_metaData::getDefinedMetaDataFields();

		$rebuildMetaDisabled = true;
		foreach($metaDataFields as $md){
			if($md['importFrom'] !== ''){
				$rebuildMetaDisabled = false;
				break;
			}
		}

		$parts->add(new we_html_multiIconBoxContentLine('', we_html_forms::radiobuttonV(self::REBUILD_TYPE_METADATA, $type, 'type', g_l('rebuild', '[rebuild_metadata][caption]') . ' ' . we_html_tools::alertAttentionBox(g_l('rebuild', '[rebuild_metadata][description]'), we_html_tools::TYPE_HELP), true, 'defaultfont', '', (!we_base_permission::hasPerm([
							we_base_perm::REBUILD_META])) || $rebuildMetaDisabled)))
			->add(new we_html_multiIconBoxContentLine('', we_html_forms::radiobuttonV(self::REBUILD_TYPE_MEDIALINKS, $type, 'type', g_l('rebuild', '[media_links][caption]') . ' ' . we_html_tools::alertAttentionBox(g_l('rebuild', '[media_links][description]'), we_html_tools::TYPE_HELP), true, 'defaultfont', '', false))
		);

		$allbutdisabled = !(we_base_permission::hasPerm([we_base_perm::REBUILD_ALL, we_base_perm::REBUILD_FILTERD, we_base_perm::REBUILD_OBJECTS, we_base_perm::REBUILD_INDEX,
				we_base_perm::REBUILD_THUMBS, we_base_perm::REBUILD_META]));

		$dthidden = $metaFieldsHidden = $thumbsHidden = '';
		$doctypesArray = array_filter(explode(',', $doctypes));
		foreach($doctypesArray as $k => $v){
			$dthidden .= we_html_element::hidden('doctypes[' . $k . ']', $v);
		}

		$thumbsArray = explode(',', $thumbs);
		foreach($thumbsArray as $k => $v){
			$thumbsHidden .= we_html_element::hidden('thumbs[' . $k . ']', $v);
		}

		foreach($metaFields as $key => $val){
			$metaFieldsHidden .= we_html_element::hidden('_field[' . $key . ']', $val);
		}

		return [we_html_element::jsScript(JS_DIR . 'rebuild0.js', 'WE().util.dialogTop(window).focus();initRebuild();set_button_state(' . ($allbutdisabled ? 1 : 0) . ');'),
			(new we_html_multiIconBox($parts))->marginLeft()->headline(g_l('rebuild', '[rebuild]'))->get() .
			$dthidden .
			$thumbsHidden .
			$metaFieldsHidden .
			we_html_element::hiddens([
				'catAnd' => (string) ($catAnd),
				'thumbsFolders' => (string) ($thumbsFolders),
				'metaFolders' => $metaFolders,
				'categories' => $categories,
				'folders' => $folders,
				'fr' => 'body',
				'btype' => $btype,
				'onlyEmpty' => (string) ($onlyEmpty),
				'we_cmd[0]' => 'rebuild',
				'step' => '1'
		])];
	}

	/**
	 * returns the HTML for the Second Step (1) of the wizard
	 *
	 * @return array
	 */
	private function getStep1(): array{
		switch(we_base_request::string('type', self::REBUILD_TYPE_DOCUMENTS)){
			default:
			case self::REBUILD_TYPE_DOCUMENTS:
				return $this->getRebuildDocuments();
			case self::REBUILD_TYPE_THUMBNAILS:
				return $this->getRebuildThumbnails();
			case self::REBUILD_TYPE_METADATA:
				return $this->getRebuildMetadata();
		}
	}

	/**
	 * returns the HTML for the Third Step (2) of the wizard. - Here the real work (loop) is done - it should be displayed in the cmd frame
	 *
	 * @return array
	 */
	private function getStep2(): array{
		$btype = we_base_request::string('btype', self::REBUILD_BTYPE_ALL);
		$thumbsFolders = we_base_request::intList('thumbsFolders', []);
		$thumbs = we_base_request::intList('thumbs', []);
		$catAnd = we_base_request::bool('catAnd');
		$metaFolders = implode(',', we_base_request::intList('metaFolders', [(we_users_util::get_def_ws() ?: 0)]));
		$metaFields = we_base_request::intList('_field');
		$onlyEmpty = we_base_request::bool('onlyEmpty');

		$taskname = md5(session_id() . '_rebuild');
		$currentTask = we_base_request::bool('doFragments');
		$taskFilename = WE_FRAGMENT_PATH . '/' . $taskname;

		$data = [];
		if(!(file_exists($taskFilename) && $currentTask)){
			switch(we_base_request::string('type', self::REBUILD_TYPE_DOCUMENTS)){
				case self::REBUILD_TYPE_DOCUMENTS:
					$categories = we_base_request::intList('categories', []);
					$doctypes = we_base_request::intList('doctypes');
					$folders = we_base_request::intList('folders', []);
					$templateID = we_base_request::int('templateID', 0);
					$data = we_rebuild_base::getDocuments($btype, $categories, $catAnd, $doctypes, $folders, $templateID);
					break;
				case self::REBUILD_TYPE_THUMBNAILS:
					if(!$thumbs){
						$this->jsCmd->addMsg(g_l('rebuild', '[no_thumbs_selected]'), we_base_util::WE_MESSAGE_ERROR);
						return [we_html_element::jsScript(JS_DIR . 'rebuild0.js', 'set_button_stateStep2();WE().util.dialogTop(window).wizbusy.showPrevNextButton();'),
							''];
					}
					$data = we_rebuild_base::getThumbnails($thumbs, $thumbsFolders);
					break;
				case self::REBUILD_TYPE_INDEX:
					$data = we_rebuild_base::getIndex();
					break;
				case self::REBUILD_TYPE_OBJECTS:
					$data = we_rebuild_base::getObjects();
					break;
				/* case self::REBUILD_TYPE_NAVIGATION:
				  $data = we_rebuild_base::getNavigation();
				  break; */
				case self::REBUILD_TYPE_METADATA:
					$data = we_rebuild_base::getMetadata($metaFields, $onlyEmpty, $metaFolders);
					break;
				case self::REBUILD_TYPE_MEDIALINKS:
					$data = we_rebuild_base::getMedialinks();
					break;
			}
			if($data){
				$frag = new we_rebuild_fragment($taskname, 5, [], $data, $this->jsCmd);
				$frag->run();
				return [];
			}
			$this->jsCmd->addMsg(g_l('rebuild', '[nothing_to_rebuild]'), we_base_util::WE_MESSAGE_ERROR);
			return [we_html_element::jsScript(JS_DIR . 'rebuild0.js', 'set_button_stateStep2();WE().util.dialogTop(window).wizbusy.showPrevNextButton();'),
				''];
		}
		$count = 0;
		switch(we_base_request::string('type', self::REBUILD_TYPE_DOCUMENTS)){
			case self::REBUILD_TYPE_DOCUMENTS:
				$count = 1; //FIXME: we need a solution for static documents with e.g. <we:ifSelf> in navigation
				break;
			case self::REBUILD_TYPE_THUMBNAILS:
				$count = 4;
				break;
			case self::REBUILD_TYPE_INDEX:
				$count = 8;
				break;
			case self::REBUILD_TYPE_OBJECTS:
				$count = 10;
				break;
			/* case self::REBUILD_TYPE_NAVIGATION:
			  if(we_base_request::bool('rebuildStaticAfterNavi')){//static documents consume more time, we have to be patient
			  $count = 4;
			  break;
			  } */
			case self::REBUILD_TYPE_METADATA:
				$count = 15;
				break;
			case self::REBUILD_TYPE_MEDIALINKS:
				$count = 8;
				break;
		}

		$frag = new we_rebuild_fragment($taskname, $count, [], [], $this->jsCmd);
		$frag->run();
		return [];
	}

	/**
	 * returns HTML for the category form
	 *
	 * @return string
	 * @param int[] $categories csv value with category IDs
	 * @param boolean $catAnd if the categories should be connected with AND
	 */
	private function formCategory(array $categories, bool $catAnd): string{
		$catAndCheck = we_html_forms::checkbox('1', $catAnd, 'catAnd', g_l('rebuild', '[catAnd]'), false, "defaultfont", "document.we_form.btype[3].checked=true;");
		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:document.we_form.btype[3].checked=true;we_cmd('del_all_cats')");
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:document.we_form.btype[3].checked=true;we_cmd('we_selector_document',{id:-1,table:WE().consts.tables.CATEGORY_TABLE,jsCmd:'add_cat'})");
		$upperTable = '<table class="default" style="width:495px;"><tr><td style="text-align:left">' . $catAndCheck . '</td><td style="text-align:right">' . $delallbut . $addbut . '</td></tr></table>';

		$cats = new we_chooser_multiDir(array_filter(array_unique($categories)), 'del_cat', $upperTable, [], '"' . we_base_ContentTypes::CATEGORY . '"', CATEGORY_TABLE);
		return g_l('global', '[categorys]') . we_html_element::br(2) . $cats->get($this->jsCmd);
	}

	/**
	 * returns HTML for the doctypes form
	 *
	 * @return string
	 * @param int[] $doctypes csv value with doctype IDs
	 */
	private static function formDoctypes(array $doctypes): string{

		$all = DB_WE::$inst->getAllFirstq('SELECT dt.ID,dt.Text FROM ' . DOC_TYPES_TABLE . ' dt WHERE dt.IsFolder=0 ORDER BY dt.Text', false);
		$DTselect = g_l('global', '[doctypes]') . we_html_element::br(2) . '<select class="defaultfont" name="doctypes[]" size="5" multiple style="width: 495px" onchange="document.we_form.btype[3].checked=true;">';

		foreach($all as $id => $text){
			$DTselect .= '<option value="' . $id . '"' . (in_array($id, $doctypes) ? " selected" : "") . '>' . $text . "</option>";
		}
		$DTselect .= '</select>';
		return $DTselect;
	}

	/**
	 * returns HTML for the directories form
	 *
	 * @return string
	 * @param int[] $folders csv value with directory IDs
	 * @param boolean $thumnailpage if it should displayed in the thumbnails page or on an other page
	 */
	private function formFolders(array $folders, bool $thumnailpage = false): string{
		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:" . ($thumnailpage ? "" : "document.we_form.btype[3].checked=true;") . "we_cmd('del_all_folders')");
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:" . ($thumnailpage ? "" : "document.we_form.btype[3].checked=true;") . "we_cmd('we_selector_directory',{table:WE().consts.tables.FILE_TABLE,jsCmd:'add_folder',multiple:true})");

		$dirs = new we_chooser_multiDir(array_filter(array_unique($folders)), 'del_folder', $delallbut . $addbut, [], 'ContentType', FILE_TABLE);

		return g_l('rebuild', ($thumnailpage ? '[thumbdirs]' : '[dirs]')) . we_html_element::br(2) . $dirs->get($this->jsCmd);
	}

	/**
	 * returns HTML for the thumbnails form
	 *
	 * @return string
	 * @param int[] $thumbs csv value with thumb IDs
	 */
	private static function formThumbs(array $thumbs): string{
		$all = DB_WE::$inst->getAllq('SELECT ID,Text,description FROM ' . THUMBNAILS_TABLE . ' ORDER BY Text');
		$sel = g_l('rebuild', '[rebuild_thumbnails][description]') . we_html_element::br(2) .
			'<select class="defaultfont" name="thumbs[]" size="10" multiple style="width: 520px">';

		foreach($all as $rec){
			$sel .= '<option title="' . $rec['description'] . '" value="' . $rec['ID'] . '"' . (in_array($rec['ID'], $thumbs) ? ' selected="selected"' : '') . '>' . $rec['Text'] . "</option>";
		}
		$sel .= '</select>';
		return $sel;
	}

	static function formMetadata($metaFields, bool $onlyEmpty): string{
		$html = we_html_element::span(['name' => '_errorMessage'], (empty($metaFields) ? g_l('rebuild', '[noFieldsChecked]') : '')) .
			we_html_tools::alertAttentionBox(g_l('rebuild', '[expl_rebuild_metadata]'), we_html_tools::TYPE_INFO, 520) .
			'<div class="defaultfont" style="margin:10px 0 5px 0;">' . g_l('rebuild', '[metadata]') . ':</div>';

		$selAllBut = we_html_button::create_button(we_html_button::TOGGLE, "javascript:we_cmd('toggle_all_fields');");

		foreach(we_metadata_metaData::getDefinedMetaDataFields() as $md){
			if($md['importFrom']){
				$html .= we_html_forms::checkbox('1', (!empty($metaFields[$md['tag']])), '_field[' . $md['tag'] . ']', $md['tag'], false, 'defaultfont', "checkForError()");
			}
		}

		$html .= we_html_element::span(['style' => 'margin:10px 0 20px 0;'], $selAllBut) .
			we_html_forms::checkbox('1', $onlyEmpty, 'onlyEmpty', g_l('rebuild', '[onlyEmpty]'));

		return $html;
	}

	/**
	 * returns Array with javascript (array[0]) and HTML Content (array[1]) for the rebuild document page
	 *
	 * @return array
	 */
	private function getRebuildDocuments(): array{
		$thumbsFolders = implode(',', we_base_request::intList('thumbsFolders', []));
		$metaFolders = implode(',', we_base_request::intList('metaFolders', []));
		$metaFields = we_base_request::intList('_field');
		$thumbs = implode(',', we_base_request::intList('thumbs'));
		$type = we_base_request::string('type', self::REBUILD_TYPE_DOCUMENTS);
		$btype = we_base_request::string('btype', self::REBUILD_BTYPE_ALL);
		$categories = we_base_request::intList('categories', []);
		$doctypes = we_base_request::intList('doctypes');
		$folders = we_base_request::intList('folders');
		$catAnd = we_base_request::bool('catAnd');
		$onlyEmpty = we_base_request::bool('onlyEmpty');

		$ws = we_users_util::get_ws(FILE_TABLE);
		if($ws && !in_array(0, $ws) && (!$folders)){
			$folders = [we_users_util::get_def_ws(FILE_TABLE)];
		}

		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine('', we_html_forms::radiobuttonV(self::REBUILD_BTYPE_ALL, $btype, 'btype', g_l('rebuild', '[rebuild_all][caption]'), true, 'defaultfont', '', (!we_base_permission::hasPerm([
							we_base_perm::REBUILD_ALL])), g_l('rebuild', '[rebuild_all][description]'), 0, 495))
			)
			->add(new we_html_multiIconBoxContentLine('', we_html_forms::radiobuttonV(self::REBUILD_BTYPE_TEMPLATES, $btype, 'btype', g_l('rebuild', '[rebuild_templates][caption]'), true, 'defaultfont', '', (!we_base_permission::hasPerm([
							we_base_perm::REBUILD_TEMPLATES])), g_l('rebuild', '[rebuild_templates][description]'), 0, 495))
			)
			->add(new we_html_multiIconBoxContentLine('', we_html_forms::radiobuttonV(self::REBUILD_BTYPE_DYNAMIC, $btype, 'btype', g_l('rebuild', '[rebuild_dynamic][caption]'), true, 'defaultfont', '', (!we_base_permission::hasPerm([
							we_base_perm::REBUILD_FILTERD])), g_l('rebuild', '[rebuild_dynamic][description]'), 0, 495))
			)
			->add(new we_html_multiIconBoxContentLine('', we_html_forms::radiobuttonV(self::REBUILD_BTYPE_FILTER, $btype, 'btype', g_l('rebuild', '[rebuild_filter][caption]'), true, 'defaultfont', '', (!we_base_permission::hasPerm([
						we_base_perm::REBUILD_FILTERD])), g_l('rebuild', '[rebuild_filter][description]'), 0, 495, '', (we_modules_info::isActive(we_modules_info::CATEGORY) ? $this->formCategory($categories, $catAnd) . we_html_element::br(2) : '') .
					self::formDoctypes($doctypes) . we_html_element::br(2) .
					self::formFolders($folders)))
		);

		$thumbsHidden = '';
		$thumbsArray = explode(',', $thumbs);
		foreach($thumbsArray as $cur){
			$thumbsHidden .= we_html_element::hidden('thumbs[]', $cur);
		}

		$metaFieldsHidden = '';
		if($metaFields){
			foreach($metaFields as $val){
				$metaFieldsHidden .= we_html_element::hidden('_field[]', $val);
			}
		}

		return [we_html_element::jsScript(JS_DIR . 'rebuild2.js', "WE().session.rebuild.folders='folders';"),
			(new we_html_multiIconBox($parts))->marginLeft()->headline(g_l('rebuild', '[rebuild_documents][caption]'))->get() .
			$thumbsHidden .
			$metaFieldsHidden .
			we_html_element::hiddens([
				'thumbsFolders' => $thumbsFolders,
				'metaFolders' => $metaFolders,
				'metaFields' => implode(',', $metaFields),
				'onlyEmpty' => (string) $onlyEmpty,
				'folders' => implode(',', $folders),
				'categories' => implode(',', $categories),
				'fr' => 'body',
				'type' => $type,
				'we_cmd[0]' => 'rebuild',
				'step' => '2'])];
	}

	/**
	 * returns Array with javascript (array[0]) and HTML Content (array[1]) for the rebuild metadata page
	 *
	 * @return array
	 */
	private function getRebuildThumbnails(): array{

		$thumbsFolders = we_base_request::intList('thumbsFolders', []);
		$metaFolders = implode(',', we_base_request::intList('metaFolders', []));
		$metaFields = we_base_request::_(we_base_request::RAW, '_field', []);
		$thumbs = we_base_request::intList('thumbs');
		$type = we_base_request::string('type', self::REBUILD_TYPE_DOCUMENTS);
		$categories = implode(',', we_base_request::intList('categories', []));
		$doctypes = implode(',', we_base_request::intList('doctypes', []));
		$folders = we_base_request::intList('folders');
		$catAnd = we_base_request::bool('catAnd');
		$onlyEmpty = we_base_request::bool('onlyEmpty');

		$ws = we_users_util::get_ws(FILE_TABLE);

		// check if folers are in Workspace of User

		if($ws && $folders){
			$newFolders = [];
			foreach($folders as $folder){
				'@phan-var int $folder';
				if(we_users_util::in_workspace($folder, $ws, FILE_TABLE)){
					$newFolders[] = $folder;
				}
			}
			$folders = $newFolders;
		}

		if($ws && !in_array(0, $ws) && empty($thumbsFolders)){
			$thumbsFolders = [we_users_util::get_def_ws(FILE_TABLE)];
		}

		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine('', we_rebuild_wizard::formThumbs($thumbs) .
				we_html_element::br(2) .
				self::formFolders($thumbsFolders, true)
		));

		$dthidden = '';
		$doctypesArray = array_filter(explode(',', $doctypes));
		foreach($doctypesArray as $key => $val){
			$dthidden .= we_html_element::hidden('doctypes[' . $key . ']', $val);
		}
		$metaFieldsHidden = '';
		foreach($metaFields as $key => $val){
			$metaFieldsHidden .= we_html_element::hidden("_field[$key]", $val);
		}

		return [we_html_element::jsScript(JS_DIR . 'rebuild2.js', "WE().session.rebuild.folders='thumbsFolders';"),
			(new we_html_multiIconBox($parts))->marginLeft()->headline(g_l('rebuild', '[rebuild_thumbnails][caption]'))->get() .
			$dthidden .
			$metaFieldsHidden .
			we_html_element::hiddens([
				'catAnd' => $catAnd,
				'thumbsFolders' => implode(',', $thumbsFolders),
				'metaFolders' => $metaFolders,
				'onlyEmpty' => $onlyEmpty,
				'folders' => implode(',', $folders),
				'categories' => $categories,
				'fr' => 'body',
				'type' => $type,
				'we_cmd[0]' => 'rebuild',
				'step' => '2'
		])];
	}

	/**
	 * @return array
	 */
	private function getRebuildMetadata(): array{
		$thumbsFolders = implode(',', we_base_request::intList('thumbsFolders', []));
		$metaFolders = we_base_request::intList('metaFolders', []);
		$onlyEmpty = we_base_request::bool('onlyEmpty');
		$metaFields = we_base_request::_(we_base_request::RAW, '_field', []);
		$thumbs = we_base_request::intList('thumbs');
		$type = we_base_request::string('type', self::REBUILD_TYPE_DOCUMENTS);
		$categories = implode(',', we_base_request::intList('categories', []));
		$doctypes = we_base_request::intList('doctypes');
		$folders = we_base_request::intList('folders');
		$catAnd = we_base_request::bool('catAnd');

		$ws = we_users_util::get_ws(FILE_TABLE);

		// check if folers are in Workspace of User

		if($ws && $folders){
			$newFolders = [];
			foreach($folders as $folder){
				if(we_users_util::in_workspace((int) $folder, $ws, FILE_TABLE)){
					$newFolders[] = $folder;
				}
			}
			$folders = $newFolders;
		}

		if($ws && !in_array(0, $ws) && !$metaFolders){
			$metaFolders = [we_users_util::get_def_ws(FILE_TABLE)];
		}

		$content = we_rebuild_wizard::formMetadata($metaFields, $onlyEmpty) .
			we_html_element::br(2) .
			self::formFolders($metaFolders ?: [], true);

		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine('', $content));

		$dthidden = '';
		foreach($doctypes as $doctype){
			$dthidden .= we_html_element::hidden('doctypes[]', $doctype);
		}
		$thumbsHidden = '';
		foreach($thumbs as $thumb){
			$thumbsHidden .= we_html_element::hidden('thumbs[]', $thumb);
		}

		return [we_html_element::jsScript(JS_DIR . 'rebuild2.js', "WE().session.rebuild.folders='metaFolders';"),
			(new we_html_multiIconBox($parts))->marginLeft()->headline(g_l('rebuild', '[rebuild_metadata][caption]'))->get() .
			$dthidden .
			$thumbsHidden .
			we_html_element::hiddens([
				'catAnd' => $catAnd,
				'metaFolders' => implode(',', $metaFolders),
				'thumbsFolders' => $thumbsFolders,
				'folders' => implode(',', $folders),
				'categories' => $categories,
				'fr' => 'body',
				'type' => $type,
				'we_cmd[0]' => 'rebuild',
				'step' => '2'
		])];
	}

	/**
	 * returns HTML for the frameset
	 *
	 * @return string
	 */
	private function getFrameset(): string{
		$btype = we_base_request::string('btype');
		$type = we_base_request::string('type');
		$tid = we_base_request::int('templateID');
		$resp = we_base_request::string('responseText');
		$tail = ($btype ? '&amp;btype=' . rawurlencode($btype) : '') .
			($type ? '&amp;type=' . rawurlencode($type) : '') .
			($tid ? '&amp;templateID=' . $tid : '') .
			($this->step ? '&amp;step=' . $this->step : '') .
			($resp ? '&amp;responseText=' . rawurlencode($resp) : '');

		$taskname = md5(session_id() . '_rebuild');
		$taskFilename = WE_FRAGMENT_PATH . '/' . $taskname;
		if(file_exists($taskFilename)){
			we_base_file::delete($taskFilename);
		}

		if($tail){
			$body = we_html_element::body(['id' => 'weMainBody', 'onload' => "wizcmd.location=WE().consts.dirs.WEBEDITION_DIR+'we_cmd.php?we_cmd[0]=rebuild&amp;fr=body" . $tail . "';"]
					, we_html_element::iframe('wizbusy', WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=rebuild&amp;fr=busy&amp;dc=1', 'position:absolute;top:0;bottom:0;left:0;right:0;overflow: hidden') .
					we_html_element::iframe('wizcmd', 'about:blank', 'position:absolute;bottom:0;height:0;left:0;right:0;overflow: hidden;')
			);
		} else {
			$body = we_html_element::body(['id' => 'weMainBody']
					, we_html_element::iframe('wizbody', WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=rebuild&amp;fr=body', 'position:absolute;top:0;bottom:60px;left:0;right:0;overflow: hidden') .
					we_html_element::iframe('wizbusy', WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=rebuild&amp;fr=busy', 'position:absolute;height:60px;bottom:0;left:0;right:0;overflow: hidden') .
					we_html_element::iframe('wizcmd', WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=rebuild&amp;fr=cmd', 'position:absolute;bottom:0;height:0;left:0;right:0;overflow: hidden;')
			);
		}

		return we_html_tools::getHtmlTop(g_l('rebuild', '[rebuild]'), $this->jsCmd->getCmds(), $body);
	}

	/**
	 * returns Javascript for step 2 (1)
	 *
	 * @return string
	 * @param array<int,string> $contents first element (array[0]) must be a javascript, second element (array[1]) must be the Body HTML
	 */
	private function getPage(array $contents): string{
		if(!$contents){
			$js = $this->jsCmd->getCmds();
			return ($js ?
				we_html_tools::getHtmlTop(g_l('rebuild', '[rebuild]'), $js, we_html_element::body()) :
				'');
		}
		return we_html_tools::getHtmlTop(g_l('rebuild', '[rebuild]'), $contents[0] .
				$this->jsCmd->getCmds(), we_html_element::body(['class' => 'weDialogBody'], we_html_element::form(['name' => we_base_constants::WE_FORM, 'method' => 'post',
						'action' => WEBEDITION_DIR . 'we_cmd.php'], $contents[1])
				)
		);
	}

	public function showFrameset(): void{
		switch(we_base_request::string('fr')){
			case 'body':
				echo $this->getBody();
				break;
			case 'busy':
				echo $this->getBusy();
				break;
			case 'cmd':
				echo $this->getCmd();
				break;
			default:
				echo $this->getFrameset();
		}
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
				'noFieldsChecked' => (g_l('rebuild', '[noFieldsChecked]'))
		]);
	}

}
