<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 07a9a048026d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-18 00:22 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_rpcCmd_navigation extends we_rpc_cmd{

	function execute(): we_rpc_response{
		switch($this->CmdShell->getCmd()){
			case 'navigationCopyFolder':
				return $this->navigationCopyFolder();
			case 'navigationGetItems':
				return $this->navigationGetItems();
		}


		return $this->resp;
	}

	private function navigationGetItems(): we_rpc_response{
		$navi = new we_navigation_navigation(we_base_request::int('nid', 0));
		$items = $navi->getChilds();
		$data = [];

		foreach($items as $item){
			$data[] = [$item['ID'], $item['Text']];
		}
		$this->resp->setData('data', $data);
		return $this->resp;
	}

	private function navigationCopyFolder(): we_rpc_response{
		$targetFolder = we_base_request::_(we_base_request::FILE, 'we_cmd', '', 0);
		$sourceFolderID = we_base_request::int('we_cmd', 0, 3);

		if($targetFolder &&
			($targetFolderID = we_base_request::int('we_cmd', 0, 1)) &&
			($sourceFolder = we_base_request::_(we_base_request::FILE, 'we_cmd', '', 2)) &&
			$sourceFolderID &&
			(!str_starts_with($sourceFolder, $targetFolder))
		){

			$db = DB_WE::$inst;
			$sourceFolder = fS('SELECT Path FROM ' . NAVIGATION_TABLE . ' WHERE ID=' . $sourceFolderID);
			$result = $db->getAllq('SELECT * FROM ' . NAVIGATION_TABLE . ' WHERE Path LIKE "' . $db->escape($sourceFolder) . '/%" ORDER BY IsFolder DESC,Path ASC');
			'@phan-var array $result';

			$folders = [$targetFolderID];
			$mapedId = [$sourceFolderID => ['id' => $targetFolderID, 'path' => $targetFolder]];
			$itemsQuery = [];
			$now = new DateTime();
			$keys = '';
			foreach($result as $row){
				$oldparent = $row['ParentID'];
				if(!isset($mapedId[$oldparent])){
					t_e('Parentid not found', $oldparent, $mapedId);
				}
				$rowID = $row['ID'];
				//remove calculated entry
				unset($row['IsPublished'], $row['ID']);
				$rowInsert = array_merge($row, [
					'ParentID' => $mapedId[$oldparent]['id'],
					'Path' => $mapedId[$oldparent]['path'] . '/' . $row['Text'],
					'ModifierID' => 0,
					'Mod_Date' => $now,
					'Creation_Date' => $now,
					'CreatorID' => we_users_user::getID()
				]);

				$querySet = we_database_base::arraySetter($rowInsert, ',', true);
				$keys = $keys ?: implode(',', array_keys($row));
				if($row['IsFolder']){
					$db->query('INSERT INTO ' . NAVIGATION_TABLE . ' (' . $keys . ') VALUES ' . $querySet);
					$newID = $db->getInsertId();
					$mapedId[$rowID] = ['id' => $newID, 'path' => $rowInsert['Path']];
					$folders[] = $newID;
				} else {
					$itemsQuery[] = $querySet;
				}
			}
			if($itemsQuery){
				$db->query('INSERT INTO ' . NAVIGATION_TABLE . ' (' . $keys . ') VALUES ' . implode(',', $itemsQuery));
			}
			foreach($folders as $folder){
				$newNavi = new we_navigation_navigation($folder);
				$newNavi->save(false, true);
			}
			we_navigation_cache::delNavigationTree($targetFolderID);
			$this->resp->setData('status', 'ok');
			$this->resp->setData('folders', $folders);
		} else {
			$this->resp->setData('folders', '');
		}
		return $this->resp;
	}

}
