<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 4e4ff85098e9 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-03 00:26 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_search_resultMedia extends we_search_resultBase{
	protected $numMedialinks = 0;

	public function __construct(we_search_search $searchclass){
		parent::__construct($searchclass);
		$this->type = we_search_view::SEARCH_MEDIA;
	}

	public function setNumMedialinks(int $numMedialinks = 0): void{
		$this->numMedialinks = $numMedialinks;
	}

	public static function getListColumns(): array{
		return [
			['sort' => '', 'title' => ''],
			['sort' => 'Text', 'title' => g_l('searchtool', '[dateiname]')],
			['sort' => 'IsUsed', 'title' => g_l('searchtool', '[Status]')],
			['sort' => 'media_filesize', 'title' => g_l('searchtool', '[groesse]')],
			['sort' => 'media_alt', 'title' => 'alt'],
			['sort' => 'media_title', 'title' => 'title'],
			['sort' => 'Mod_Date', 'title' => g_l('searchtool', '[modified]')],
		];
	}

	protected function processElementList(): void{
		switch($this->ContentType){
			case we_base_ContentTypes::IMAGE:
			case we_base_ContentTypes::AUDIO:
			case we_base_ContentTypes::VIDEO:
			case we_base_ContentTypes::APPLICATION:
			case we_base_ContentTypes::PDF:
				if($this->IsProtected){
					$htmlCheckbox = we_html_element::span(['class' => 'wealertIcon', 'style' => 'margin-left:4px;', 'title' => g_l('searchtool', '[image_protected]')]);
				} else if(!in_array($this->docID, $this->searchclass->getUsedMedia())){
					$htmlCheckbox = we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_DELETE]) && fB('SELECT 1 FROM ' . DB_WE::$inst->escape(addTblPrefix($this->docTable)) . ' WHERE ID=' . intval($this->docID)) ?
						we_html_forms::checkbox($this->docID . '_' . addTblPrefix($this->docTable), false, 'delete_docs_MediaSearch', '', false, 'middlefont', '') : '';
				} else {
					$htmlCheckbox = '';
				}
				break;
			default:
				$htmlCheckbox = '';
		}
		$textFilesize = we_base_file::getHumanFileSize($this->media_filesize ?: 0);
		$htmlIcon = we_html_element::div([], we_html_element::div([],
																														we_html_element::div(['class' => 'iconListview' . ($this->ContentType === we_base_ContentTypes::IMAGE ? '' : ' icon')], $this->getHtmlIconThmubnail(90, 140, we_search_view::VIEW_LIST)['imageView']) .
					we_html_element::div(['class' => 'open'], we_html_button::create_button(we_html_button::OPEN_TO_EDIT, 'javascript:' . $this->getJsOpenToEdit()))) .
				we_html_element::div(['class' => 'filename', 'title' => $this->Text . ' (ID: ' . $this->docID . ')'], $this->Text)
		);
		$htmlCircle = '<i class="fa fa-lg fa-circle"></i>';
		$htmlAlt = we_html_element::span(['class' => ($this->media_alt ? 'set' : 'notset'), 'title' => ($this->media_alt ?: 'Alt-Attribut nicht gesetzt" ')], $htmlCircle);
		$htmlTitle = we_html_element::span(['class' => ($this->media_title ? 'set' : 'notset'), 'title' => ($this->media_title ?: 'Title-Attribut nicht gesetzt" ')], $htmlCircle);
		$htmlModDate = $this->Mod_Date ? (new DateTime($this->Mod_Date))->format(g_l('searchtool', '[date_format]')) : '-';

		$this->elementList = (new we_html_tr([
				new we_html_td($htmlCheckbox, ['class' => 'checkbox']),
				new we_html_td($htmlIcon, ['class' => 'filename']),
				new we_html_td($this->getHtmlReferences(), ['class' => 'references']),
				new we_html_td($textFilesize, ['class' => 'filesize']),
				new we_html_td($htmlAlt, ['class' => 'alt']),
				new we_html_td($htmlTitle, ['class' => 'title']),
				new we_html_td($htmlModDate, ['class' => 'moddate']),
				]))->getHtml();
	}

	private function getHtmlReferences(): string{
		$mediaLinks = $this->searchclass->getUsedMediaLinks();
		$accessibles = $mediaLinks['accessible']['mediaID_' . $this->docID] ?? [];
		$notaccessibles = $mediaLinks['notaccessible']['mediaID_' . $this->docID] ?? [];
		$groups = $mediaLinks['groups']['mediaID_' . $this->docID] ?? [];

		if(empty($groups)){
			return we_html_element::div(['class' => 'plaintext'], 'Das Medien-Dokument wird nicht benutzt.'); // FIXME: G_L()
		}

		$btnMore = we_html_button::create_button(we_html_button::DIRRIGHT, "javascript:weSearch.toggleAdditionalContent(this, " . $this->docID . ")", '', "", false, false, '__' . $this->docID, false, 'Verwendet in:');
		$colText = 'Referenzen auf dieses Medien-Doukument'; //g_l('searchtool', '[mediaRef][title]');

		$table = new we_html_table([], 0, 2);
		foreach($groups as $group){
			$numNotaccessible = isset($notaccessibles[$group]) && is_array($notaccessibles[$group]) ? count($notaccessibles[$group]) : 0;
			$numAccessibles = isset($accessibles[$group]) && is_array($accessibles[$group]) ? count($accessibles[$group]) : 0;
			$groupTitle = $group . ' (' . ($numNotaccessible + $numAccessibles) . ($numNotaccessible ? ', davon ' . $numNotaccessible . ' ' . g_l('weClass', '[medialinks_unaccessible]') : '') . '):';
			$table->addRow(new we_html_tr([new we_html_td($groupTitle, ['class' => 'groupTitle']), new we_html_td()]));

			$references = isset($accessibles[$group]) && is_array($accessibles[$group]) ? $accessibles[$group] : [];
			ksort($references);
			// not quite elegant...
			$limit = $this->numMedialinks;
			$c = 0;

			foreach($references as $reference){
				if($limit > -1 && $c++ >= $limit){
					$table->addRow(new we_html_tr([
							new we_html_td('[ + ' . ($numAccessibles - $limit) . ' ' . g_l('weClass', '[medialinks_more]') . ' ]', ['class' => 'entryText']),
							new we_html_td()
							], ['class' => 'tableReferences']));
					break;
				}

				$class = '';
				$makeLink = true;
				// FIXME: establishing document state is buggy
				switch($reference['referencedIn']){
					case 'temp':
					case 'both':
						if($reference['isUnpublished']){
							$class = 'notpublished';
						} else {
							$class = 'modified';
						}
						break;
					case 'main':
						if($reference['isModified']){
							$class = 'gray';
							$makeLink = false;
						} else if($reference['isUnpublished']){
							$class = 'notpublished';
						}
				}

				if($makeLink){
					$element = preg_replace('|NN[0-9]\]+$|', 'NN]', $reference['element']);
					$text = '<a href="javascript:' . $reference['onclick'] . '" title="ID ' . $reference['id'] . ': ' . $reference['path'] . ($element ? ', in: ' . $reference['element'] : '') . '">' . we_html_element::span([
							'class' => '' . $class], $reference['path']) . '</a>' . ($element ? we_html_element::br() . 'in: ' . $element : '');
					$btn = we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:" . $reference['onclick'] . "; ");
				} else {
					$text = we_html_element::span(['class' => $class], $reference['path']);
					$btn = we_html_button::create_button(we_html_button::OPEN_TO_EDIT, '', '', '', true, false, '', false, g_l('searchtool', '[linkPublishedOnly]'));
				}
				$table->addRow(new we_html_tr([
						new we_html_td($text, ['class' => 'entryText']),
						new we_html_td($btn, ['class' => 'entryBtn'])
						], ['class' => 'tableReferences']));
			}
		}

		return we_html_element::div(['class' => 'referencesHead'], $btnMore . $colText) . we_html_element::div(['id' => 'infoTable_' . $this->docID, 'class' => 'referencesContent'], $table->getHtml());
	}

}
