<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 818ead29ce48 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-20 02:08 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_categorySelect extends we_weTag_tag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @return string parsed tag for php
	 */
	public static function parse(array $attribs, string $content): string{
		return '<?php ob_start();?>' . $content . '<?= ' . we_weTag_parser::printTag('categorySelect', $attribs, 'ob_get_clean()?:\'\'', false, true) . '; ?>';
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$isuserinput = empty($name);
		$name = $isuserinput ? 'we_ui_' . we_tag_form::$name . '_categories' : $name;

		$type = self::getAttributeR('type', $attribs, '', we_base_request::STRING);
		$rootdir = self::getAttributeR('rootdir', $attribs, '/', we_base_request::FILE);
		$firstentry = self::getAttributeR('firstentry', $attribs, '', we_base_request::STRING);
		$showpath = self::getAttributeR('showpath', $attribs, false, we_base_request::BOOL);
		$indent = self::getAttribute('indent', $attribs, '', we_base_request::RAW_CHECKED);
		$multiple = self::getAttribute('multiple', $attribs, false, we_base_request::BOOL);

		$catIDs = self::getAttributeR('categoryids', $attribs, [], we_base_request::INTLISTA);
		$fromTag = self::getAttributeR('fromTag', $attribs, false, we_base_request::STRING);
		self::$doc = we_base_WE::getWeDoc();

		$values = [];
		if($isuserinput && we_tag_form::$name){
			$objekt = we_tag_write::$objElements[we_tag_form::$name] ?? we_tag_write::$docElements[we_tag_form::$name] ?? self::$doc ?? false;
			if($objekt){
				$values = $objekt->Category;
				'@phan-var int[] $values';
			}
			$valuesArray = id2pathA($values, CATEGORY_TABLE);
		} elseif($type === 'request'){
			// Bug Fix #750
			$valuesArray = we_base_request::stringList($name);
		} else {
			// Bug Fix #750
			$valuesArray = (isset($GLOBALS[$name]) ? (is_array($GLOBALS[$name]) ?
				$GLOBALS[$name] :
				explode(',', $GLOBALS[$name])) : []);
		}

		$attribs['name'] = $name;

		// Bug Fix #750
		if($multiple){
			$attribs['name'] .= '[]';
			$attribs['multiple'] = 'multiple';
		} else {
			$attribs = self::removeAttribs($attribs, ['size', 'multiple']);
		}

		$attribs = self::removeAttribs($attribs, [
				'showpath',
				'rootdir',
				'firstentry',
				'type',
				'shopCat',
				'categoryids',
				'fromTag',
				'field'
		]);

		$content = trim($content);
		if(!$content){
			if($firstentry){
				$content .= self::htmlTag('option', ['value' => ($fromTag === 'shopcategory' ? 0 : '')], $firstentry);
			}
			$db = DB_WE::$inst;
			$dbfield = $showpath || $indent ? 'Path' : 'Text';
			$valueField = $fromTag ? 'ID' : 'Path';
			//$whereTag = !$fromTag ? '' : ($fromTag === 'shopcategory' ? ' AND IsFolder=0' : ' AND ID IN('. trim($catIDs, ',') .')');
			$whereTag = !$fromTag ? '' : ' AND ID IN(' . implode(',', $catIDs) . ')';

			$data = $db->getAllq('SELECT ID,Path,Text FROM ' . CATEGORY_TABLE . ' WHERE ' . ($rootdir === '/' ? 1 : ' Path LIKE "' . $db->escape(rtrim($rootdir, '/')) . '/%"') . $whereTag . ' ORDER BY ' . $dbfield);
			foreach($data as $cur){
				$deep = count(explode('/', $cur['Path'])) - 2;
				$field = ($rootdir && ($rootdir != '/') && $showpath ?
					preg_replace('|^' . preg_quote($rootdir, '|') . '|', '', $cur[$dbfield]) :
					$cur[$dbfield]);

				if($field){
					$content .= self::htmlTag('option', ['value' => $cur[$valueField],
							(in_array($cur[$valueField], $valuesArray) ? 'selected' : null) => 'selected'
							], str_repeat($indent, $deep) . $field);
				}
			}
		} else {
			foreach($valuesArray as $catPaths){
				if(str_contains(strtolower($content), '<option>')){
					$content = preg_replace('/<option>' . preg_quote((string) $catPaths) . '( ?[<\n\r\t])/i', '<option selected="selected">' . $catPaths . '${1}', $content);
				}
				$content = str_replace('<option value="' . $catPaths . '">', '<option value="' . $catPaths . '" selected="selected">', $content);
			}
		}

		return we_html_element::span(['class' => 'we we_category'], self::htmlTag('select', $attribs, $content, true));
	}

}
