<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 44db4d7c3342 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 18:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_customerResetPasswordLink extends we_weTag_tag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		if(!$internal){
			self::attributeMissing($attribs, ['id' => false], __CLASS__);
		}

		if(we_tag_customerResetPassword::$status !== we_customer_customer::PWD_ALL_OK || !isset($_SESSION['webuser'])){
			return '';
		}

		$attribs['href'] = self::getUrl($attribs, $internal) . '?user=' . $_SESSION['webuser']['ID'] . '&token=' . $_SESSION['webuser']['WE_token'];

		return (self::getAttribute('plain', $attribs, false, we_base_request::BOOL) ?
			$attribs['href'] :
			self::htmlTag('a', self::removeAttribs($attribs, ['id', 'plain', 'host']), $content ?: $attribs['href'])
			);
	}

	protected static function getUrl(array $attribs, bool $internal): string{
		if($internal){
			$docPath = WEBEDITION_DIR . 'resetpwd.php';
			$cnt = 0;
			$url = '';
		} else {
			$id = self::getAttribute('id', $attribs, 0, we_base_request::INT);
			$docPath = (string) we_weTag_util::tag('a', ['hrefonly' => true, 'id' => $id]);
			$cnt = 0;
			//Fix #10071
			$urlReplace = we_folder_base::getUrlReplacements(DB_WE::$inst, true);
			$url = preg_replace($urlReplace, array_keys($urlReplace), $docPath, -1, $cnt);
		}
		$host = self::getAttribute('host', $attribs, we_http_util::getServerUrl(), we_base_request::URL);
		return $host . ($cnt ? $url : $docPath);
	}

}
