<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 31f4c1918051 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-03 14:29 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_users_user extends we_base_model{
	const DEFAULT_PASS_REGEX = '(.{8,20})';
	const ERR_USER_PATH_NOK = -5;
	const INVALID_CREDENTIALS = 1;
	const LOGIN_CREDENTIALS_INVALID = 1;
	const LOGIN_CREDENTIALS_INVALID_HOOK = 6;
	const LOGIN_DENIED = 4;
	const LOGIN_DENIED_MAINTENANCE = 7;
	const LOGIN_DENIED_SECURITY = 3;
	const LOGIN_DENIED_TOTP = 5;
	const LOGIN_OK = 2;
	const LOGIN_UNKNOWN = 0;
	const MAX_LOGIN_COUNT_REACHED = 2;
	const SALT_CRYPT = 2;
	const SALT_MD5 = 1;
	const SALT_NONE = 0;
	const TAB_DATA = 0;
	const TAB_PERMISSION = 1;
	const TAB_SETTINGS = 3;
	const TAB_WORKSPACES = 2;
	const TYPE_ALIAS = 'alias';
	const TYPE_USER = 'user';
	const TYPE_USER_GROUP = 'group';
	public const MAX_PWD_LEN = 64;
	public const TOTP_PATTERN = '[A-Za-z2-7=]{16,128}';

	/** @var array <string,int> Name of the class => important for reconstructing the class from outside the class
	  In this array are all storagable class variables */
	public $persistent_slots = [
		'ID' => we_base_request::INT,
		'Type' => we_base_request::STRING,
		'ParentID' => we_base_request::INT,
		'Salutation' => we_base_request::STRING,
		'First' => we_base_request::STRING,
		'Second' => we_base_request::STRING,
		'Address' => we_base_request::STRING,
		'HouseNo' => we_base_request::STRING, //e.g. 2a
		'City' => we_base_request::STRING,
		'PLZ' => we_base_request::STRING, //leading 0
		'State' => we_base_request::STRING,
		'Country' => we_base_request::STRING,
		'Telephone' => we_base_request::STRING,
		'Fax' => we_base_request::STRING,
		'Handy' => we_base_request::STRING,
		'Email' => we_base_request::EMAIL,
		'Company' => we_base_request::STRING,
		'Department' => we_base_request::STRING,
		'username' => we_base_request::STRING,
		'passwd' => we_base_request::RAW_CHECKED,
		'TOTP' => we_base_request::STRING,
		'Path' => we_base_request::FILE,
		'Permissions' => we_base_request::SERIALIZED_JSON,
		'ParentPerms' => we_base_request::BOOL,
		'Description' => we_base_request::STRING,
		'Alias' => we_base_request::INT,
		//'IsFolder' => we_base_request::BOOL,
		'CreatorID' => we_base_request::INT,
		'Creation_Date' => we_base_request::DATE_TIME,
		'ModifierID' => we_base_request::INT,
		'Mod_Date' => we_base_request::DATE_TIME,
		'Ping' => we_base_request::DATE_TIME,
		'LastLogin_Date' => we_base_request::DATE_TIME,
		'workSpaceDoc' => we_base_request::INTLISTA,
		'workSpaceTmp' => we_base_request::INTLISTA,
		'workSpaceNav' => we_base_request::INTLISTA,
		'workSpaceNwl' => we_base_request::INTLISTA,
		'workSpaceObj' => we_base_request::INTLISTA,
		'workSpaceCust' => we_base_request::SERIALIZED_JSON,
		'ParentWs' => we_base_request::BOOL,
		'ParentWst' => we_base_request::BOOL,
		'ParentWsn' => we_base_request::BOOL,
		'ParentWso' => we_base_request::BOOL,
		'ParentWsnl' => we_base_request::BOOL,
		'ParentWsCust' => we_base_request::BOOL,
		'LoginDenied' => we_base_request::BOOL,
	];
	// Name of the Object that was createt from this class

	/** @var string Name of the Object that was createt from this class */
	var $Name = '';

	/** @var int */
	var $ParentID = 0;

	/** @var string Flag which indicates which kind of user is 0-user;1-group;2-owner group;3 - alias */
	var $Type = self::TYPE_USER;

	/** @var string Salutation */
	var $Salutation = '';

	/** @var string  User first name */
	var $First = '';

	/** @var string  User second name */
	var $Second = '';

	/** @var string  Address */
	var $Address = '';

	/** @var string  House number */
	var $HouseNo = '';

	/** @var string  Time-based One-time Password */
	var $TOTP = '';

	/** @var string  City */
	var $City = '';

	/** @var string */
	var $State = '';

	/** @var string  ZIP Code */
	var $PLZ = '';

	/** @var string  Country */
	var $Country = '';

	/** @var string  Telephone */
	var $Telephone = '';

	/** @var string  Fax */
	var $Fax = '';

	/** @var string  Cell phone */
	var $Handy = '';

	/** @var string  Email */
	var $Email = '';

	/** @var string */
	var $Company = '';

	/** @var string */
	var $Department = '';

	/** @var string  Username */
	var $username = '';

	/** @var string  User password (md5 salted) */
	var $passwd = '';

	/** @var string  User password */
	var $clearpasswd = '';

	/** @var array<string,bool> User permissions Flag which indicated if user inherits permissions from parent */
	var $Permissions = [];

	/** @var bool Flag which indicated if user inherits permissions from parent */
	var $ParentPerms = false;

	/** @var string Description */
	var $Description = '';

	/** @var array<string,?string> User Prefrences */
	var $Preferences = [];

	/** @var string */
	var $Path = '';

	/** @var int */
	var $Alias = 0;

	/** @var int */
	var $CreatorID = 0;

	/** @var DateTime */
	var $CreateDate;

	/** @var int */
	var $ModifierID = 0;

	/** @var DateTime */
	var $Mod_Date;

	/** @var ?DateTime Ping flag */
	var $Ping = null;

	/** @var ?DateTime Last login flag */
	var $LastLogin_Date = null;

	/** @var int[] */
	var $workSpaceDoc = [];

	/** @var int[] Newsletter workspaces */
	var $workSpaceTmp = [];

	/** @var int[] Flag which indicated if user inherits files workspaces from parent */
	var $workSpaceNav = [];

	/** @var int[] Flag which indicated if user inherits templates workspaces from parent */
	var $workSpaceObj = [];

	/** @var int[] Flag which indicated if user inherits newsletters workspaces from parent */
	var $workSpaceNwl = [];

	/** @var array<int,array{logic:string,field:string,operation:int,value:string}> Flag which indicated if user inherits templates workspaces from parent */
	protected $workSpaceCust = [];

	/** @var bool Flag which indicated if user inherits files workspaces from parent */
	var $ParentWs = true;

	/** @var bool  Flag which indicated if user inherits templates workspaces from parent */
	var $ParentWst = true;

	/** @var bool  Flag which indicated if user inherits templates workspaces from parent */
	var $ParentWsn = true;

	/** @var bool  Flag which indicated if user inherits objetcs workspaces from parent */
	var $ParentWso = true;

	/** @var bool  Flag which indicated if user inherits newsletters workspaces from parent */
	var $ParentWsnl = true;

	/** @var bool  Flag which indicated if user inherits customer "workspaces" from parent */
	protected $ParentWsCust = true;

	/** @var bool */
	var $LoginDenied = false;

	/** @var array{tblFile:int[],tblTemplates:int[],tblnavigation:int,tblObjectFiles:int[],tblNewsletter:int[],tblWebUser:array{logic:string,field:string,operation:int,value:string}[]} Workspace array */
	//@phan-suppress-next-line PhanTypeMismatchPropertyDefault
	var $workspaces = [
		FILE_TABLE => [],
		TEMPLATES_TABLE => [],
		NAVIGATION_TABLE => [],
		OBJECT_FILES_TABLE => [],
		NEWSLETTER_TABLE => [],
		CUSTOMER_TABLE => [],
	];

	/** @var array<string,string> Permissions headers array */
	var $permissions_main_titles = [];

	/** @var array<string,array>  Permissions values array */
	var $permissions_slots = [];

	/** @var array<string,string[]> Permissions titles */
	var $permissions_titles = [];

	/** @var array<string,int> */
	protected $permissions_defaults = [];

	/** @var string[] Preferences array */
	private $preference_slots = [
		'Language', 'seem_start_file',
		'seem_start_type', 'editorSizeOpt', 'editorWidth', 'editorHeight', 'editorFontname', 'editorFontsize', 'editorFont', 'default_tree_count', /* 'force_glossary_action', 'force_glossary_check', */
		'cockpit_amount_columns', 'cockpit_amount_last_documents', 'editorMode'];
	private $editor_reloaded = false;

	public function __construct(){
		parent::__construct(USER_TABLE);

		$this->Name = 'user_' . md5(uniqid(__FILE__, true));

		$this->db = new DB_WE();

		foreach($this->preference_slots as $val){
			$this->Preferences[$val] = null;
		}

		$this->initType(self::TYPE_USER);
	}

	function initType(string $typ): void{
		$this->Type = $typ;
		$this->mapPermissions();
	}

	// Intialize the class

	function initFromDB(int $id): bool{
		if(!$id || !fB('SELECT 1 FROM ' . USER_TABLE . ' WHERE ID=' . $id)){
			return false;
		}

		$this->load($id);
//$this->getPersistentSlotsFromDB();
		$this->getPreferenceSlotsFromDB();
		$this->loadWorkspaces();
		$this->mapPermissions();

		return true;
	}

	function removeAccount(): void{
		we_base_history::deleteByUserID($this->ID);
	}

	/*
	  function getPersistentSlotsFromDB(): void{
	  $tmp = getHash('SELECT *,DATE_FORMAT(Ping,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS Ping,DATE_FORMAT(LastLogin_Date,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS LastLogin FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->ID), $this->db);
	  foreach($tmp as $fieldName => $val){
	  if(!isset($this->persistent_slots[$fieldName])){
	  continue;
	  }
	  $type = $this->persistent_slots[$fieldName];
	  $this->$fieldName = we_base_request::filterVar($val, $type); //@phan-suppress-current-line PhanDeprecatedFunction
	  }
	  } */

	function save(bool $force_new = false, bool $rebuild = false, array $skipWrite = []): bool{
		$isnew = $this->ID ? false : true;
		if($this->Type == self::TYPE_USER_GROUP && $this->ID != 0){
			$ppath = ($this->ParentID == 0 ? '/' : $this->getPath($this->ParentID));
			$dpath = $this->getPath($this->ID);
			if(preg_match('|' . $dpath . '|', $ppath)){
				throw new we_exception_base(g_l('modules_users', '[user_path_nok]'), self::ERR_USER_PATH_NOK);
			}
		}
		if($this->Type == self::TYPE_ALIAS){
			$foo = getHash('SELECT ID,username FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->Alias), $this->db);
			$uorginal = $foo['ID'];
			$search = true;
			$ount = 0;
			$try_name = '@' . $foo['username'];
//$try_text = $foo['username'];
			while($search){
				if(fB('SELECT 1 FROM ' . USER_TABLE . ' WHERE ID!=' . intval($this->ID) . ' AND ID!=' . intval($uorginal) . ' AND username="' . $this->db->escape($try_name) . '" LIMIT 1')){
					$try_name = $try_name . '_' . ++$ount;
				} else {
					$search = false;
				}
			}
			$this->username = $try_name;
		}
//$this->IsFolder = ($this->Type == self::TYPE_USER_GROUP ? true : false);
		$this->Path = $this->getPath($this->ID);
		$oldpath = $this->Path;
		$this->saveWorkspaces();
		$this->savePermissions();
		if($isnew){
			$this->CreatorID = we_users_user::getID();
			$this->CreateDate = new DateTime();
		}
		$this->ModifierID = we_users_user::getID();
		$this->Mod_Date = new DateTime();
		$this->passwd = ($this->clearpasswd === '' ? $this->passwd : self::makeSaltedPassword($this->clearpasswd));
		parent::save(false, false, ['Creation_Date', 'Ping',]);

		if($oldpath != '' && $oldpath != '/'){
			$entries = $this->db->getAllq('SELECT ID FROM ' . USER_TABLE . ' WHERE Path LIKE "' . $this->db->escape($oldpath) . '%"', true);
			foreach($entries as $id){
				$this->db->query('UPDATE ' . USER_TABLE . ' SET Path="' . $this->getPath((int) $id) . '" WHERE ID=' . $id);
			}
		}
		$this->savePreferenceSlotsInDB($isnew);

		$_REQUEST['uid'] = $this->ID;

		return true;
	}

	function saveToSession(we_base_jsCmd $jsCmd): void{
		if($this->ID != we_users_user::getID()){
			return;
		}

		$this->rememberPreference($jsCmd, $this->Preferences['Language'] ?? null, 'Language');
		$this->rememberPreference($jsCmd, $this->Preferences['default_tree_count'] ?? null, 'default_tree_count');

		if(isset($this->Preferences['seem_start_type'])){
			switch($this->Preferences['seem_start_type']){
				case 'cockpit':
					$this->rememberPreference($jsCmd, 0, 'seem_start_file');
					$this->rememberPreference($jsCmd, 'cockpit', 'seem_start_type');
//$this->rememberPreference($jsCmd, '', 'seem_start_weapp');
					break;
				case 'object':
					$this->rememberPreference($jsCmd, $this->Preferences['seem_start_object'] ?? 0, 'seem_start_file');
					$this->rememberPreference($jsCmd, 'object', 'seem_start_type');
//$this->rememberPreference($jsCmd, '', 'seem_start_weapp');
					break;
				/* case 'weapp':
				  $this->rememberPreference($jsCmd, ($this->Preferences['seem_start_weapp'] ?? ''), 'seem_start_weapp');
				  $this->rememberPreference($jsCmd, 'weapp', 'seem_start_type');
				  $this->rememberPreference($jsCmd, 0, 'seem_start_file');
				  break; */
				case 'document':
					$this->rememberPreference($jsCmd, $this->Preferences['seem_start_document'] ?? 0, 'seem_start_file');
					$this->rememberPreference($jsCmd, 'document', 'seem_start_type');
//$this->rememberPreference($jsCmd, '', 'seem_start_weapp');
					break;
				default:
					$this->rememberPreference($jsCmd, 'none', 'seem_start_type');
//$this->rememberPreference($jsCmd, '', 'seem_start_weapp');
					$this->rememberPreference($jsCmd, 0, 'seem_start_file');
			}
		}

		$this->rememberPreference($jsCmd, $this->Preferences['sizeOpt'] ?? null, 'sizeOpt');
		$this->rememberPreference($jsCmd, $this->Preferences['weWidth'] ?? null, 'weWidth');
		$this->rememberPreference($jsCmd, $this->Preferences['weHeight'] ?? null, 'weHeight');
		$this->rememberPreference($jsCmd, $this->Preferences['editorMode'] ?? null, 'editorMode');
		$this->rememberPreference($jsCmd, $this->Preferences['editorFont'] ?? null, 'editorFont');
		$this->rememberPreference($jsCmd, $this->Preferences['editorFontname'] ?? null, 'editorFontname');
		$this->rememberPreference($jsCmd, $this->Preferences['editorFontsize'] ?? null, 'editorFontsize');
		$this->rememberPreference($jsCmd, $this->Preferences['editorSizeOpt'] ?? null, 'editorSizeOpt');
		$this->rememberPreference($jsCmd, $this->Preferences['editorWidth'] ?? null, 'editorWidth');
		$this->rememberPreference($jsCmd, $this->Preferences['editorHeight'] ?? null, 'editorHeight'); /*
		  $this->rememberPreference($jsCmd, isset($this->Preferences['force_glossary_action']) ? $this->Preferences['force_glossary_action'] : null, 'force_glossary_action') .;
		  $this->rememberPreference($jsCmd, isset($this->Preferences['force_glossary_check']) ? $this->Preferences['force_glossary_check'] : null, 'force_glossary_check') */
	}

	private function mapPermissions(): void{
		$this->permissions_main_titles = [];
		$this->permissions_slots = [];
		$this->permissions_titles = [];
		$this->permissions_defaults = [];
		$permissions = $this->Permissions;

		$entries = we_base_permission::getDefaultPermissions();

		foreach($entries as $perm_group_name => $entry){
			'@phan-var array{title:string,defaults:array<string,int>} $entry';
			if(empty($entry['defaults'])){//in case the module is not activated
				continue;
			}
			if(!($perm_group_name === 'administrator' && $this->Type != self::TYPE_USER) && $perm_group_name){
				$this->permissions_main_titles[$perm_group_name] = $entry['title'];
				$this->permissions_defaults += $entry['defaults'];
				$this->permissions_slots[$perm_group_name] = [];
				$this->permissions_titles[$perm_group_name] = [];

				foreach(array_keys($entry['defaults']) as $cur){
					$this->permissions_titles[$perm_group_name][$cur] = g_l('perms_' . $perm_group_name, '[' . $cur . ']');
					$this->permissions_slots[$perm_group_name][$cur] = (is_array($permissions) && isset($permissions[$cur]) ?
						$permissions[$cur] :
						0); //always set unknown fields to 0
				}
			}
		}
	}

	private function setPermission(string $slotkey, string $perm_name, $perm_value): void{
		$this->permissions_slots[$slotkey][$perm_name] = $perm_value;
	}

	private function savePermissions(): void{
		$permissions = [];
		$isAdmin = false;
		foreach($this->permissions_slots as $val){
			foreach($val as $k => $v){
				switch($k){
					case we_base_perm::ADMINISTRATOR:
						$isAdmin = $v;
					default:
						$permissions[$k] = $v;
				}
			}
		}
		if($isAdmin){
			$this->Permissions = [we_base_perm::ADMINISTRATOR => true];
			return;
		}
		ksort($permissions);
		$this->Permissions = array_filter($permissions);
	}

	private function loadWorkspaces(): void{
		if($this->workSpaceDoc){
			$this->workspaces[FILE_TABLE] = array_filter($this->workSpaceDoc);
		}
		if($this->workSpaceTmp){
			$this->workspaces[TEMPLATES_TABLE] = array_filter($this->workSpaceTmp);
		}
		if($this->workSpaceNav){
			$this->workspaces[NAVIGATION_TABLE] = array_filter($this->workSpaceNav);
		}
		if(we_modules_info::isActive(we_modules_info::OBJECT) && $this->workSpaceObj){
			$this->workspaces[OBJECT_FILES_TABLE] = array_filter($this->workSpaceObj);
		}
		if(we_modules_info::isActive(we_modules_info::NEWSLETTER) && $this->workSpaceNwl){
			$this->workspaces[NEWSLETTER_TABLE] = array_filter($this->workSpaceNwl);
		}
		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$this->workspaces[CUSTOMER_TABLE] = $this->workSpaceCust;
		}
	}

	private function saveWorkspaces(): void{
		foreach($this->workspaces as $k => $v){
			if($k === CUSTOMER_TABLE){
				continue;
			}
			$new_array = [];

			foreach($v as $key => $val){
				if($val != 0){
					$new_array[] = $this->workspaces[$k][$key];
				}
			}
			$this->workspaces[$k] = $new_array;
		}

		$this->workSpaceDoc = $this->workspaces[FILE_TABLE];
		if(empty($this->workSpaceDoc)){
			$uws = we_users_util::get_ws(FILE_TABLE);
			if(!empty($uws)){
				$this->workSpaceDoc = $this->workspaces[FILE_TABLE] = $uws;
			}
		}

		$this->workSpaceTmp = $this->workspaces[TEMPLATES_TABLE];
		if(empty($this->workSpaceTmp)){
			$uws = we_users_util::get_ws(TEMPLATES_TABLE);
			if(!empty($uws)){
				$this->workSpaceTmp = $this->workspaces[TEMPLATES_TABLE] = $uws;
			}
		}

		$this->workSpaceNav = $this->workspaces[NAVIGATION_TABLE];
		if(empty($this->workSpaceNav)){
			$uws = we_users_util::get_ws(NAVIGATION_TABLE);
			if(!empty($uws)){
				$this->workSpaceNav = $this->workspaces[NAVIGATION_TABLE] = $uws;
			}
		}

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$this->workSpaceObj = $this->workspaces[OBJECT_FILES_TABLE];
			if(empty($this->workSpaceObj)){
				$uws = we_users_util::get_ws(OBJECT_FILES_TABLE);
				if(!empty($uws)){
					$this->workSpaceObj = $this->workspaces[OBJECT_FILES_TABLE] = $uws;
				}
			}
		}
		if(we_modules_info::isActive(we_modules_info::NEWSLETTER)){
			$this->workSpaceNwl = $this->workspaces[NEWSLETTER_TABLE];
			if(empty($this->workSpaceNwl)){
				$uws = we_users_util::get_ws(NEWSLETTER_TABLE);
				if(!empty($uws)){
					$this->workSpaceNwl = $this->workspaces[NEWSLETTER_TABLE] = $uws;
				}
			}
		}

		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$this->workSpaceCust = $this->workspaces[CUSTOMER_TABLE] ?: [];
		}
	}

	private function getPreferenceSlotsFromDB(): void{
		$tmp = self::readPrefs($this->ID);
		$this->Preferences = array_intersect_key($tmp, array_flip($this->preference_slots));
	}

	private function setPreference(string $name, string $value): void{
		if(in_array($name, $this->preference_slots, true)){
			$this->Preferences[$name] = $value;
		}
	}

	private function savePreferenceSlotsInDB(bool $isnew = false): void{
		if($this->Type != self::TYPE_USER){
			return;
		}

		$this->Mod_Date = new DateTime();
		$updt = ['userID' => $this->ID];
		foreach($this->preference_slots as $fieldName){
			switch($fieldName){
				case 'editorFontsize':
				case 'editorFontname':
					if($this->Preferences['editorFont'] != '1'){
						$this->Preferences[$fieldName] = '-1';
					}
				default:
					$updt[$fieldName] = $this->Preferences[$fieldName];
			}
		}
		if($isnew){
			$updt['userID'] = $this->ID;
			//$updt['FileFilter'] = '0';
			$updt['openFolders_tblFile'] = [];
			$updt['openFolders_tblTemplates'] = [];
			$updt['DefaultTemplateID'] = '0';
		}

		self::writePrefs($this->ID, $updt);
		$_SESSION['prefs'] = ($_SESSION['prefs']['userID'] === $this->ID ? self::readPrefs($this->ID) : $_SESSION['prefs']);
	}

	private function rememberPreference(we_base_jsCmd $jsCmd, /* mixed */ $settingvalue, string $settingname): void{
		if(isset($settingvalue) && ($settingvalue != null)){
			switch($settingname){
				case 'Language'://FIXME: reload corrupt. fix when needless frames are eliminated
					$_SESSION['prefs']['Language'] = $settingvalue;

					if($settingvalue != we_base_WE::$Language){
						$jsCmd->addCmd('rememberPreference', 'Language', we_base_request::int('tab', 0));
					}
					break;

				case 'seem_start_type':
					switch($settingvalue){
						case 'cockpit':
							$_SESSION['prefs']['seem_start_file'] = 0;
							$_SESSION['prefs']['seem_start_type'] = 'cockpit';
							break;
						case 'object':
							$_SESSION['prefs']['seem_start_file'] = we_base_request::int('seem_start_object', 0);
							$_SESSION['prefs']['seem_start_type'] = 'object';
							break;
						/* case 'weapp':
						  $_SESSION['prefs']['seem_start_weapp'] = we_base_request::string('seem_start_weapp', '');
						  $_SESSION['prefs']['seem_start_type'] = 'weapp';
						  break; */
						case 'document':
							$_SESSION['prefs']['seem_start_file'] = we_base_request::int('seem_start_document', 0);
							$_SESSION['prefs']['seem_start_type'] = 'document';
							break;
						default:
							$_SESSION['prefs']['seem_start_file'] = 0;
							$_SESSION['prefs']['seem_start_type'] = '0';
					}
					break;

				case 'sizeOpt':
					if($settingvalue === 0){
						$_SESSION['prefs']['weWidth'] = $_SESSION['prefs']['weHeight'] = $_SESSION['prefs']['sizeOpt'] = 0;
					} else if(($settingvalue === 1) && (we_base_request::int('weWidth')) && we_base_request::int('weHeight')){
						$_SESSION['prefs']['sizeOpt'] = 1;
					}
					break;

				case 'weWidth':
					if($_SESSION['prefs']['sizeOpt'] == 1){
						$generate_java_script = false;

						if($_SESSION['prefs']['weWidth'] != $settingvalue){
							$generate_java_script = true;
						}

						$_SESSION['prefs']['weWidth'] = $settingvalue;

						if($generate_java_script){
							$height = we_base_request::int('weHeight');
							$jsCmd->addCmd('rememberPreference', 'weWidth', [$settingvalue, $height]);
						}
					}
					break;

				case 'weHeight':
					if($_SESSION['prefs']['sizeOpt'] == 1){
						$_SESSION['prefs']['weHeight'] = $settingvalue;
					}
					break;

				case 'editorMode':
					$_SESSION['prefs']['editorMode'] = $settingvalue;
					break;

				case 'editorFont':
					if($settingvalue === 0){
						$_SESSION['prefs']['editorFontname'] = 'none';
						$_SESSION['prefs']['editorFontsize'] = -1;
						$_SESSION['prefs']['editorFont'] = 0;
					} else if(($settingvalue === 1) && we_base_request::string('editorFontname') && we_base_request::int('editorFontsize')){
						$_SESSION['prefs']['editorFont'] = 1;
					}
					$jsCmd->addCmd('rememberPreference', 'reload', '');
					break;

				case 'editorFontname':
					if($_SESSION['prefs']['editorFont'] == 1){
						$_SESSION['prefs']['editorFontname'] = $settingvalue;
					}
					break;

				case 'editorFontsize':
					if($_SESSION['prefs']['editorFont'] == 1){
						$_SESSION['prefs']['editorFontsize'] = $settingvalue;
					}
					break;

				case 'editorSizeOpt':
					if($settingvalue === 0){
						$_SESSION['prefs']['editorWidth'] = $_SESSION['prefs']['editorHeight'] = $_SESSION['prefs']['editorSizeOpt'] = 0;
					} else if(($settingvalue === 1) && we_base_request::int('editorWidth') && we_base_request::int('editorHeight')){
						$_SESSION['prefs']['editorSizeOpt'] = 1;
					}

					if(!$this->editor_reloaded){
						$this->editor_reloaded = true;
						$jsCmd->addCmd('rememberPreference', 'reload', '');
					}
					break;

				case 'editorWidth':
					if($_SESSION['prefs']['editorSizeOpt'] == 1){
						$_SESSION['prefs']['editorWidth'] = $settingvalue;
					}
					break;

				case 'editorHeight':
					if($_SESSION['prefs']['editorSizeOpt'] == 1){
						$_SESSION['prefs']['editorHeight'] = $settingvalue;
					}
					break;

				case 'default_tree_count':
					$_SESSION['prefs']['default_tree_count'] = $settingvalue;
					break;
				/*
				  case 'force_glossary_check':
				  $_SESSION['prefs']['force_glossary_check'] = $settingvalue;
				  break;

				  case 'force_glossary_action':
				  $_SESSION['prefs']['force_glossary_action'] = $settingvalue;
				  break;
				 */
				case 'cockpit_amount_columns':
					$_SESSION['prefs']['cockpit_amount_columns'] = $settingvalue;
					break;

				default:
					break;
			}
		} else {

			switch($settingname){

				case 'editorFont':
					$_SESSION['prefs']['editorFontname'] = 'none';
					$_SESSION['prefs']['editorFontsize'] = -1;
					$_SESSION['prefs']['editorFont'] = 0;
					$jsCmd->addCmd('rememberPreference', 'reload', '');
					break;

				case 'force_glossary_check':
					$_SESSION['prefs']['force_glossary_check'] = 0;
					break;

				case 'force_glossary_action':
					$_SESSION['prefs']['force_glossary_action'] = 0;
					break;

				default:
					break;
			}
		}
	}

	function preserveState(int $tab): void{
		switch($tab){
			case self::TAB_DATA:
				$this->clearpasswd = we_base_request::string($this->Name . '_clearpasswd');
				foreach($this->persistent_slots as $name => $type){
					if(($val = we_base_request::_((string) $type, $this->Name . '_' . $name, '__no')) !== '__no'){
						$this->{$name} = $val;
					}
				}

				if($this->Type == self::TYPE_ALIAS){
					$this->ParentPerms = we_base_request::bool($this->Name . '_ParentPerms');
					$this->ParentWs = we_base_request::bool($this->Name . '_ParentWs');
					$this->ParentWst = we_base_request::bool($this->Name . '_ParentWst');
					$this->ParentWso = we_base_request::bool($this->Name . '_ParentWso');
					$this->ParentWsn = we_base_request::bool($this->Name . '_ParentWsn');
					$this->ParentWsnl = we_base_request::bool($this->Name . '_ParentWsnl');
					$this->ParentWsCust = we_base_request::bool($this->Name . '_ParentWsCust');
				}
				break;
			case self::TAB_PERMISSION:
				foreach($this->permissions_slots as $sKey => $pval){
					foreach(array_keys($pval) as $k){
						$this->setPermission((string) $sKey, (string) $k, we_base_request::bool($this->Name . '_Permission_' . $k));
					}
				}
				$this->ParentPerms = we_base_request::bool($this->Name . '_ParentPerms');
				break;
			case self::TAB_WORKSPACES:
				foreach(array_keys($this->workspaces) as $k){
					$obj = $this->Name . '_Workspace_' . $k;
					if(($val = we_base_request::_(we_base_request::RAW, $obj, we_base_request::NOT_VALID_RAW, 'id')) !== we_base_request::NOT_VALID_RAW){
						$this->workspaces[$k] = $val;
					}
					$obj = $this->Name . '_Workspace_' . $k . '_AddDel';
					if(($val = we_base_request::_(we_base_request::RAW, $obj, '')) != ''){
						if($val === 'new'){//add
							$this->workspaces[$k][] = 0;
						} else {
							unset($this->workspaces[$k][$val]);
						}
					}
				}
				if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
					$this->workspaces[CUSTOMER_TABLE] = we_customer_abstractFilter::getFilterFromRequest();
				}

				$this->ParentWs = we_base_request::bool($this->Name . '_ParentWs');
				$this->ParentWst = we_base_request::bool($this->Name . '_ParentWst');
				$this->ParentWso = we_base_request::bool($this->Name . '_ParentWso');
				$this->ParentWsn = we_base_request::bool($this->Name . '_ParentWsn');
				$this->ParentWsnl = we_base_request::bool($this->Name . '_ParentWsnl');
				$this->ParentWsCust = we_base_request::bool($this->Name . '_ParentWsCust');
				break;
			case self::TAB_SETTINGS:
				foreach($this->preference_slots as $val){
					switch($val){
						case 'seem_start_file':
						case 'seem_start_type':
//case 'seem_start_weapp':
							$obj = '';
							break;
						default:
							$obj = $this->Name . '_Preference_' . $val;
					}
					$this->setPreference($val, (string) we_base_request::_(we_base_request::RAW_CHECKED, $obj, 0));
				}
				switch(we_base_request::string('seem_start_type')){
					case 'cockpit':
						$this->setPreference('seem_start_file', '0');
						$this->setPreference('seem_start_type', 'cockpit');
						break;
					case 'object':
						$this->setPreference('seem_start_file', (string) we_base_request::int('seem_start_object'));
						$this->setPreference('seem_start_type', 'object');
						break;
					/* case 'weapp':
					  $this->setPreference('seem_start_weapp', we_base_request::string('seem_start_weapp'));
					  $this->setPreference('seem_start_type', 'weapp');
					  break; */
					case 'document':
						$this->setPreference('seem_start_file', (string) we_base_request::int('seem_start_document'));
						$this->setPreference('seem_start_type', 'document');
						break;
					default:
						$this->setPreference('seem_start_file', '0');
						$this->setPreference('seem_start_type', '0');
				}
				break;
		}
	}

	function checkPermission(string $perm): bool{
		foreach($this->permissions_slots as $val){
			foreach($val as $k => $v){
				if($k === $perm){
					return ($v ? true : false);
				}
			}
		}
		return false;
	}

	function resetOwnersCreatorModifier(): void{
		//FIXME: replace will not work for "1,2" if 1 is removed
		$newID = we_users_user::getID();
		$this->ID = intval($this->ID);
		$this->db->query('UPDATE ' . FILE_TABLE . " SET Owners=REPLACE(Owners,'," . $this->ID . ",',',')");
		$this->db->query('UPDATE ' . FILE_TABLE . " SET Owners='' WHERE Owners=','");
		$this->db->query('UPDATE ' . TEMPLATES_TABLE . " SET Owners=REPLACE(Owners,'," . $this->ID . ",',',')");
		$this->db->query('UPDATE ' . TEMPLATES_TABLE . " SET Owners='' WHERE Owners=','");
		$this->db->query('UPDATE ' . FILE_TABLE . " SET CreatorID=$newID WHERE CreatorID=" . $this->ID);
		$this->db->query('UPDATE ' . TEMPLATES_TABLE . " SET CreatorID=$newID WHERE CreatorID=" . $this->ID);
		$this->db->query('UPDATE ' . FILE_TABLE . " SET ModifierID=$newID WHERE ModifierID=" . $this->ID);
		$this->db->query('UPDATE ' . TEMPLATES_TABLE . ' SET ModifierID=' . $newID . ' WHERE ModifierID=' . $this->ID);
		$this->db->query('UPDATE ' . USER_TABLE . ' SET CreatorID=' . $newID . ' WHERE CreatorID=' . $this->ID);
		$this->db->query('UPDATE ' . USER_TABLE . ' SET ModifierID=' . $newID . ' WHERE ModifierID=' . $this->ID);

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$this->db->query('UPDATE ' . OBJECT_TABLE . " SET Owners=REPLACE(Owners,'," . $this->ID . ",',',')");
			$this->db->query('UPDATE ' . OBJECT_TABLE . " SET Owners='' WHERE Owners=','");
			$this->db->query('UPDATE ' . OBJECT_FILES_TABLE . " SET Owners=REPLACE(Owners,'," . $this->ID . ",',',')");
			$this->db->query('UPDATE ' . OBJECT_FILES_TABLE . " SET Owners='' WHERE Owners=','");
			$this->db->query('UPDATE ' . OBJECT_TABLE . " SET CreatorID=$newID WHERE CreatorID=" . $this->ID);
			$this->db->query('UPDATE ' . OBJECT_FILES_TABLE . " SET CreatorID=$newID WHERE CreatorID=" . $this->ID);
			$this->db->query('UPDATE ' . OBJECT_TABLE . " SET ModifierID=$newID WHERE ModifierID=" . $this->ID);
			$this->db->query('UPDATE ' . OBJECT_FILES_TABLE . " SET ModifierID=$newID WHERE ModifierID=" . $this->ID);
		}
	}

	function deleteMe(): bool{
		$this->ID = intval($this->ID);
		switch($this->Type){
			case '':
			case self::TYPE_USER:
				$this->db->query('DELETE FROM ' . USER_TABLE . ' WHERE ID=' . $this->ID);
				$this->db->query('DELETE FROM ' . PREFS_TABLE . ' WHERE userID=' . $this->ID);
				$this->resetOwnersCreatorModifier();
				$this->removeAccount();
				return true;
			case self::TYPE_USER_GROUP:
				$items = $this->db->getAllq('SELECT ID FROM ' . USER_TABLE . ' WHERE ParentID=' . $this->ID, true);
				foreach($items as $id){
					$tmpobj = new we_users_user();
					$tmpobj->initFromDB((int) $id);
					$tmpobj->deleteMe();
				}
				$this->db->query('DELETE FROM ' . USER_TABLE . ' WHERE ID=' . $this->ID);
				$this->resetOwnersCreatorModifier();
				return true;
			case self::TYPE_ALIAS:
				$this->db->query('DELETE FROM ' . USER_TABLE . ' WHERE ID=' . $this->ID);
				return true;
		}
		return false;
	}

	function isLastAdmin(): bool{
		$this->ID = intval($this->ID);

		if(fB('SELECT 1 FROM ' . USER_TABLE . " WHERE Permissions LIKE ('%\"ADMINISTRATOR\";%') AND ID!=" . $this->ID . ' LIMIT 1')){
			return false;
		}
		if(fI('SELECT ID FROM ' . USER_TABLE . " WHERE (Permissions LIKE ('%\"ADMINISTRATOR\";s:1:\"1\";%') OR Permissions LIKE ('%\"ADMINISTRATOR\":\"1\"%') OR Permissions LIKE ('%\"ADMINISTRATOR\":true%')) AND ID!=" . $this->ID)){
			return false;
		}

		return true;
	}

	function getPath(int $id = 0): string{
		if($id == 0){
			$id = intval($this->ParentID);
			$path = [$this->username];
		} else {
			$path = [];
		}
		$pid = $id;

		while($pid > 0){
			$foo = getHash('SELECT username,ParentID FROM ' . USER_TABLE . ' WHERE ID=' . $pid);
			$path[] = ($foo['username'] ?? '');
			$pid = $foo['ParentID'] ?? 0;
		}
		return '/' . implode('/', $path);
	}

	public static function getAllPermissions(int $uid, bool $onlyParent = false): array{
		$user_permissions = [];

		$db = DB_WE::$inst;
		$all = $db->getAllq('SELECT ParentID,' . ($onlyParent ? '1 AS ' : '') . ' ParentPerms,' . ($onlyParent ? '"{}" AS ' : '') . 'Permissions,Alias FROM ' . USER_TABLE . ' WHERE ID=' . $uid . ($onlyParent ? '' : ' OR Alias=' . $uid));
		foreach($all as $rec){
			if($rec['Alias'] != $uid){
				$group_permissions = we_unserialize($rec['Permissions']);
				foreach($group_permissions as $key => $val){
					$user_permissions[$key] = ($user_permissions[$key] ?? false) || ((bool) $val);
				}
			}
			$lpid = $rec['ParentID'];
			if($rec['ParentPerms']){
				while($lpid){
					$hash = getHash('SELECT ParentID,ParentPerms,Permissions FROM ' . USER_TABLE . ' WHERE ID=' . intval($lpid));
					if($hash){
						$group_permissions = we_unserialize($hash['Permissions']);
						foreach(array_keys($group_permissions) as $key){
							$user_permissions[$key] = ($user_permissions[$key] ?? false) || ((bool) $group_permissions[$key]);
						}
						$lpid = ($hash['ParentPerms'] ? $hash['ParentID'] : 0);
						continue;
					}
					$lpid = 0;
				}
			}
		}
		/* 		if(!$onlyParent && !array_filter($user_permissions)){
		  t_e('error reading user permissions! Check parent permissions & resave parent folders! UID: ' . $uid, $user_permissions);
		  } */
		return (!empty($user_permissions[we_base_perm::ADMINISTRATOR]) ? [we_base_perm::ADMINISTRATOR => 1] : array_filter($user_permissions));
	}

	/**
	 * LAYOUT FUNCTIONS
	 */
	function formDefinition(we_base_jsCmd $jsCmd, int $tab): string{
		switch($tab){
			case self::TAB_DATA:
				return we_html_element::jsScript(JS_DIR . 'comparePwd.js', '', ['id' => 'loadVarComparePwd', 'data-passwd' => setDynamicVar([
							'pwdCheck' => SECURITY_USER_PASS_REGEX
					])]) .
					$this->formGeneralData();
			case self::TAB_PERMISSION:
				return $this->formPermissions($jsCmd);
			case self::TAB_WORKSPACES:
				return $this->formWorkspace();
			case self::TAB_SETTINGS:
				return $this->formPreferences($jsCmd);
		}
		return $this->formGeneralData();
	}

	function formGeneralData(): string{
		switch($this->Type){
			default:
			case self::TYPE_USER:
				return $this->formUserData();
			case self::TYPE_USER_GROUP:
				return $this->formGroupData();
			case self::TYPE_ALIAS:
				return $this->formAliasData();
		}
	}

	function formGroupData(): string{
		$username = $this->getUserfield('username', 'group_name', 'text', 255, false, ['id' => 'yuiAcInputPathName', 'onblur' => "parent.frames[0].weTabs.setTitlePath(this.value);",
			'title' => g_l('global', '[allowedChars][extended]'), 'pattern' => we_base_constants::CHARS_PATTERN_EXTENDED]);
		$description = '<textarea name="' . $this->Name . '_Description" cols="25" rows="5" style="width:560px" class="defaultfont" onchange="top.content.setHot();">' . $this->Description . '</textarea>';

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('PathGroup');
		$weSuggest->setContentType([we_users_user::TYPE_USER_GROUP]);
		$weSuggest->setInput($this->Name . '_ParentID_Text', (fS('SELECT Path FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->ParentID)) ?: '/'), [
			], false, true);
		$weSuggest->setMaxResults(10);
		$weSuggest->setResult($this->Name . '_ParentID', (string) ($this->ParentID));
		$weSuggest->setSelector(we_gui_suggest::DirSelector);
		$weSuggest->setTable(USER_TABLE);
		$weSuggest->setWidth(450);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_users_selector',{id:document.we_form." . $this->Name . "_ParentID.value,targetName:'" . $this->Name . "_ParentID_Text',filter:'" . we_users_user::TYPE_USER_GROUP . "',targetID:'" . $this->Name . "_ParentID'});"));

		$weAcSelector = $weSuggest->getHTML();
		$tableObj = (new we_html_table([], 3, 1))
			->setCol(0, 0, $username)
			->setCol(1, 0, we_html_tools::formElementTable(g_l('modules_users', '[description]'), $description), ['style' => 'padding-top:5px;'])
			->setCol(2, 0, we_html_tools::formElementTable(g_l('modules_users', '[group]'), $weAcSelector), ['style' => 'padding-top:10px;']);

		$content = '<select name="' . $this->Name . '_Users" size="8" style="width:560px" onchange="if(this.selectedIndex > -1){WE().layout.button.switch_button_state(document, \'edit\', \'enabled\');}else{WE().layout.button.switch_button_state(document, \'edit\', \'disabled\');}" ondblclick="top.content.we_cmd(\'display_user\',document.we_form.' . $this->Name . '_Users.value)">';
		if($this->ID){
			$all = $this->db->getAllFirstq('SELECT ID,username,Type FROM ' . USER_TABLE . ' WHERE Type IN ("' . self::TYPE_USER . '","' . self::TYPE_ALIAS . '") AND ParentID=' . intval($this->ID));
			foreach($all as $id => $rec){
				$content .= '<option value="' . $id . '">' . (($rec['Type'] == self::TYPE_ALIAS) ? '[' : '') . $rec['username'] . (($rec['Type'] == self::TYPE_ALIAS) ? ']' : '');
			}
		}

		$content .= '</select>' . we_html_element::br(2) . we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:we_cmd('display_user',document.we_form." . $this->Name . "_Users.value)", '', "", true, false);

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('modules_users', '[group_data]'), $tableObj->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED)
			)
			->add((new we_html_multiIconBoxContentLine(g_l('modules_users', '[user]'), $content))
			->space(we_html_multiIconBox::SPACE_MED)
		);

		return (new we_html_multiIconBox($parts))->marginLeft()->get();
	}

	function getUserfield(string $name, string $lngkey, string $type = 'text', int $maxlen = 255, bool $noNull = false, array $attribs = [], int $width = 240): string{
		$val = $this->{$name};
		if($noNull && !$val){
			$val = '';
		}
		$attribs['onchange'] = 'top.content.setHot()';
		return g_l('modules_users', '[' . $lngkey . ']') . we_html_element::br() . we_html_tools::textInput($this->Name . '_' . $name, $val, $maxlen, $attribs, $type, $width . 'px');
	}

	function formUserData(): string{
		$tableAdvObj = new we_html_table(['class' => 'withMediumSpace'], 12, 2, [
			new we_html_tr([
				new we_html_td($this->getUserfield('Salutation', 'salutation'))]),
			new we_html_tr([
				new we_html_td($this->getUserfield('First', 'first_name')),
				new we_html_td($this->getUserfield('Second', 'second_name'))
				]),
			new we_html_tr([
				new we_html_td($this->getUserfield('Address', 'address')),
				new we_html_td($this->getUserfield('HouseNo', 'houseno'))
				]),
			new we_html_tr([
				new we_html_td($this->getUserfield('PLZ', 'PLZ', 'text', 16, true)),
				new we_html_td($this->getUserfield('City', 'city'))
				]),
			new we_html_tr([
				new we_html_td($this->getUserfield('State', 'state')),
				new we_html_td($this->getUserfield('Country', 'country'))
				]),
			new we_html_tr([
				new we_html_td($this->getUserfield('Telephone', 'telephone', 'tel')),
				new we_html_td($this->getUserfield('Fax', 'fax', 'tel'))
				]),
			new we_html_tr([
				new we_html_td($this->getUserfield('Handy', 'mobile'))
				]),
			new we_html_tr([
				new we_html_td($this->getUserfield('Company', 'company')),
				new we_html_td($this->getUserfield('Department', 'department'))
				]),
			new we_html_tr([
				new we_html_td(we_html_tools::formElementTable(g_l('modules_users', '[description]'), '<textarea name="' . $this->Name . '_Description" cols="25" rows="5" style="width:520px" class="defaultfont" onchange="top.content.setHot();">' . $this->Description . '</textarea>'), [
					'colspan' => '2']
				)
				])
		]);


		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('PathGroup')
			->setContentType([we_users_user::TYPE_USER_GROUP])
			->setInput($this->Name . '_ParentID_Text', (fS('SELECT Path FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->ParentID)) ?: '/'), [
				], false, true)
			->setMaxResults(10)
			->setResult($this->Name . '_ParentID', (string) ($this->ParentID))
			->setSelector(we_gui_suggest::DirSelector)
			->setTable(USER_TABLE)
			->setWidth(403)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_users_selector',{id:document.we_form." . $this->Name . "_ParentID.value,targetName:'" . $this->Name . "_ParentID_Text',filter:'" . we_users_user::TYPE_USER_GROUP . "',targetID:'" . $this->Name . "_ParentID'});"));

		$auth = new PHPGangsta_GoogleAuthenticator();
		$pass = new we_customer_generatePassword(8, ['caps' => 3, 'small' => 3, 'nums' => 2, 'specs' => 0]);
		$tableObj = new we_html_table([], 5, 2, [
			new we_html_tr([
				new we_html_td($this->getUserfield('username', 'username', 'text', 255, false, ['id' => 'yuiAcInputPathName', 'onblur' => "parent.frames[0].weTabs.setTitlePath(document.getElementsByName('" . $this->Name . "_ParentID_Text')[0].value,this.value);",
						'required' => 'required', 'autocomplete' => 'new-password', 'title' => g_l('global', '[allowedChars][extended]'), 'pattern' => we_base_constants::CHARS_PATTERN_EXTENDED]), [
					'style' => 'width:280px;'])
				]),
			new we_html_tr([
				new we_html_td(we_html_tools::formElementTable(g_l('modules_users', '[password]'), '<div id="badPwd" style="display:none;" class="arrow_box">' . g_l('global', '[pass_to_short]') . '</div>' .
						(we_users_user::getID() && we_users_user::getID() == $this->ID && !we_base_permission::hasPerm([we_base_perm::USER_EDIT_PASSWD]) ?
						'****************' :
						'<input id="newSecretPass" type="hidden" name="_none" value="' . $pass->PassGen() . '" /><input type="hidden" name="' . $this->Name . '_clearpasswd" value="' . $this->clearpasswd . '" />' . we_html_tools::textInput('input_pass', '', 255, [
							'onchange' => "if(comparePwd('input_pass','input_pass')){document.getElementById('badPwd').style.display='block';}else{document.getElementById('badPwd').style.display='none';}top.content.setHot();",
							'autocomplete' => 'new-password'], 'password', '190px') . we_html_button::create_button(we_html_button::RELOAD, '', '', "var el=document.getElementsByName('input_pass')[0];el.type='text';el.value=document.getElementById('newSecretPass').value;"))), [
					'style' => 'width:280px;'])
				]),
			new we_html_tr([
				new we_html_td($this->getUserfield('Email', 'email', 'email', 255, false, ['required' => 'required'])),
				new we_html_td($this->getUserfield('TOTP', 'TOTP', 'text', 128, false, ['pattern' => self::TOTP_PATTERN, 'id' => 'totp', 'placeholder' => ($this->TOTP ? g_l('modules_users', '[TOTP_secret_set]') : '-')], 190) . '<input id="newSecretTOTP" type="hidden" name="_none" value="' . $auth->createSecret() . '" />' . we_html_button::create_button(we_html_button::RELOAD, '', '', "document.getElementById('totp').value=document.getElementById('newSecretTOTP').value;")),
				]),
			new we_html_tr([
				new we_html_td(we_html_tools::formElementTable(g_l('modules_users', '[group]'), $weSuggest->getHTML()), ['colspan' => '2'])
				]),
			new we_html_tr([
				new we_html_td(g_l('modules_users', '[CreatorID]') . ' ' . ($this->CreatorID ?
					(fS('SELECT CONCAT(username," (",First," ",Second,")") FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->CreatorID)) ?:
					g_l('modules_users', '[lostID]') . $this->CreatorID . g_l('modules_users', '[lostID2]')) :
					'-')),
				new we_html_td(g_l('modules_users', '[CreateDate]') . ' ' . (($this->CreateDate) ? $this->CreateDate->format(g_l('weEditorInfo', '[date_format]')) : '-'))
				]),
			new we_html_tr([
				new we_html_td(we_html_forms::checkboxWithHidden((bool) $this->LoginDenied, $this->Name . '_LoginDenied', g_l('modules_users', '[login_denied]'), false, '', "top.content.setHot();", (we_users_user::getID() == $this->ID || !we_base_permission::hasPerm([
							we_base_perm::ADMINISTRATOR]))))
				]),
			new we_html_tr([
				new we_html_td(g_l('modules_users', '[ModifierID]') . ' ' . ($this->ModifierID ?
					($this->ModifierID == $this->ID ?
					$this->username . ' (' . $this->First . ' ' . $this->Second . ')' :
					(fS('SELECT CONCAT(username," (",First," ",Second,")") FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->ModifierID)) ?:
					g_l('modules_users', '[lostID]') . $this->ModifierID . g_l('modules_users', '[lostID2]'))
					) :
					'-')),
				new we_html_td(g_l('modules_users', '[ModifyDate]') . ' ' . (($this->Mod_Date) ? $this->Mod_Date->format(g_l('weEditorInfo', '[date_format]')) : '-'))
				]),
			new we_html_tr([
				new we_html_td(g_l('modules_customer', '[LastLogin]') . ' ' . ($this->LastLogin_Date ? $this->LastLogin_Date->format(g_l('weEditorInfo', '[date_format]')) : '-')),
				new we_html_td(g_l('modules_users', '[lastPing]') . ' ' . ($this->Ping ? $this->Ping->format(g_l('weEditorInfo', '[date_format]')) : '-'))
				])
		]);

		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine(g_l('modules_users', '[user_data]'), $tableObj->getHtml(), we_html_multiIconBox::PROP_PATH))
			->add(new we_html_multiIconBoxContentLine(g_l('modules_users', '[general_data]'), $tableAdvObj->getHtml(), we_html_multiIconBox::PROP_USER));

		return (new we_html_multiIconBox($parts))->marginLeft()->get();
	}

	private function fold_checkbox_groups(array $filter): we_html_multiIconBoxContent{
		asort($this->permissions_main_titles);
		$parentGroups = $this->ParentID ? self::getAllPermissions($this->ID, true) : false;
		$parts = new we_html_multiIconBoxContent();
		foreach($this->permissions_main_titles as $groupKey => $groupName){
			// Filter out groups not to be shown
			if(in_array($groupKey, $filter)){
				continue;
			}
			$cur = '<table class="default" style="width:500px;margin-left:5em;">';

			foreach($this->permissions_titles[$groupKey] as $perm => $permTitle){
				'@phan-var string $permTitle';
				if(!(we_base_permission::hasPerm([$perm]))){
					continue;
				}
				$cur .= '<tr><td>' . ($parentGroups === false ? '' : '<i class="showParentPerms fa fa-' . (isset($parentGroups[$perm]) ? 'check" style="color:lightgreen"' : 'close" style="color:red"') . '></i>') . '</td>
	<td style="padding:5px 0;">' . we_html_forms::checkbox('1', ($this->permissions_slots[$groupKey][$perm] ? true : false), $this->Name . '_Permission_' . $perm, $permTitle, false, "defaultfont", "top.content.setHot();") . '</td></tr>';
			}

			$parts->add((new we_html_multiIconBoxContentLine($groupName, $cur . '</table>'))
					->noline()
					->fold()
			);
		}

		return $parts;
	}

	/**
	 * This function outputs the group of selectable user permissions
	 *
	 * @return     string
	 */
	private function formPermissions(we_base_jsCmd $jsCmd): string{
		// Set output text
		// Create a object of the class dynamicControls
		//FIXME: change we_html_dynamicControls
		// Now we create the overview of the user rights
		$parts = $this->fold_checkbox_groups(['administrator']);
		$handler = '';
		$javascript = 'function toggleRebuildPerm(disabledOnly) {';
		if(!empty($this->permissions_slots['rebuildpermissions'])){
			foreach(array_keys($this->permissions_slots['rebuildpermissions']) as $pname){
				if($pname != 'REBUILD'){
					$javascript .= '
	if (document.we_form.' . $this->Name . '_Permission_REBUILD && document.we_form.' . $this->Name . '_Permission_' . $pname . ') {
		if(document.we_form.' . $this->Name . '_Permission_REBUILD.checked) {
			document.we_form.' . $this->Name . '_Permission_' . $pname . '.disabled = false;
			if (!disabledOnly) {
				document.we_form.' . $this->Name . '_Permission_' . $pname . '.checked = true;
			}
		} else {
			document.we_form.' . $this->Name . '_Permission_' . $pname . '.disabled = true;
			if (!disabledOnly) {
				document.we_form.' . $this->Name . '_Permission_' . $pname . '.checked = false;
			}
		}
	}';
				} else {
					$handler = '
	document.we_form.' . $this->Name . '_Permission_' . $pname . ".onclick =  (document.we_form." . $this->Name . '_Permission_' . $pname . "?
		rebuildCheckboxClicked():
		top.content.setHot());
	toggleRebuildPerm(true);";
				}
			}
		}
		$javascript .= '}' . $handler;

		// js to uncheck all permissions
		$uncheckjs = $checkjs = $defaultjs = '';
		foreach($this->permissions_defaults as $pname => $pvalue){
			if($pname != we_base_perm::ADMINISTRATOR){
				$uncheckjs .= 'document.we_form.' . $this->Name . '_Permission_' . $pname . '.checked = false;';
				$checkjs .= 'document.we_form.' . $this->Name . '_Permission_' . $pname . '.checked = true;';
				$defaultjs .= 'document.we_form.' . $this->Name . '_Permission_' . $pname . '.checked = ' . ($pvalue ? 'true' : 'false') . ';';
			}
		}

		$button_uncheckall = we_html_button::create_button('uncheckall', 'javascript:' . $uncheckjs . 'top.content.setHot();');
		$button_checkall = we_html_button::create_button('checkall', 'javascript:' . $checkjs . 'top.content.setHot();');
		$button_default = we_html_button::create_button('default', 'javascript:' . $defaultjs . 'top.content.setHot();');
		$parts->add(new we_html_multiIconBoxContentLine('', $button_default . $button_uncheckall . $button_checkall));

		// Check if user has right to decide to give administrative rights
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) && $this->Type == self::TYPE_USER && is_array($this->permissions_slots['administrator'])){
			$content = '';
			foreach($this->permissions_slots['administrator'] as $k => $v){
				$content .= '<tr><td>' . we_html_forms::checkbox('1', (bool) $v, $this->Name . '_Permission_' . $k, $this->permissions_titles['administrator'][$k], false, 'defaultfont', ($k === 'REBUILD' ? 'setRebuidPerms();top.content.setHot();' : 'top.content.setHot();')) . '</td></tr>';
			}
			$parts->add(new we_html_multiIconBoxContentLine($this->permissions_main_titles['administrator'], '<table class="default" style="width:500px;margin-top:5px;">' . $content . '</table>'));
		}
		if($this->ParentID){
			$jsCmd->addCmd('showParentPerms', ($this->ParentPerms ? 1 : 0));
			$parts->add(new we_html_multiIconBoxContentLine('', $this->formInherits('_ParentPerms', $this->ParentPerms, g_l('modules_users', '[inherit]'), 'showParentPerms(this.checked);')));
		}

		return (new we_html_multiIconBox($parts))->marginLeft()->get() . we_html_element::jsElement($javascript);
	}

	private function formWorkspace(): string{
		$parentWsp = self::setEffectiveWorkspaces($this->ID, true);
		$parts = new we_html_multiIconBoxContent();

		$weSuggest = we_gui_suggest::getInstance();
		$title = $setValue = $showParent = $content1 = '';
		foreach($this->workspaces as $table => $v){
			switch($table){
				case TEMPLATES_TABLE:
					$title = g_l('modules_users', '[workspace_templates]');
					$setValue = 'TEMPLATES_TABLE';
					$showParent = $this->ParentWst;
					$content1 = $this->ParentID ? $this->formInherits('_ParentWst', $this->ParentWst, g_l('modules_users', '[inherit_wst]'), 'document.getElementById(\'info' . $setValue . '\').style.display=(this.checked?\'inline\':\'none\');') : '';
					break;
				case NAVIGATION_TABLE:
					$title = g_l('modules_users', '[workspace_navigations]');
					$setValue = 'NAVIGATION_TABLE';
					$showParent = $this->ParentWsn;
					$content1 = $this->ParentID ? $this->formInherits('_ParentWsn', $this->ParentWsn, g_l('modules_users', '[inherit_wsn]'), 'document.getElementById(\'info' . $setValue . '\').style.display=(this.checked?\'inline\':\'none\');') : '';
					break;
				case NEWSLETTER_TABLE :
					$title = g_l('modules_users', '[workspace_newsletter]');
					$setValue = 'NEWSLETTER_TABLE';
					$showParent = $this->ParentWsnl;
					$content1 = $this->ParentID ? $this->formInherits('_ParentWsnl', $this->ParentWsnl, g_l('modules_users', '[inherit_wsnl]'), 'document.getElementById(\'info' . $setValue . '\').style.display=(this.checked?\'inline\':\'none\');') : '';
					break;
				case OBJECT_FILES_TABLE :
					$title = g_l('modules_users', '[workspace_objects]');
					$setValue = 'OBJECT_TABLE';
					$showParent = $this->ParentWso;
					$content1 = $this->ParentID ? $this->formInherits('_ParentWso', $this->ParentWso, g_l('modules_users', '[inherit_wso]'), 'document.getElementById(\'info' . $setValue . '\').style.display=(this.checked?\'inline\':\'none\');') : '';
					break;
				case FILE_TABLE:
					$title = g_l('modules_users', '[workspace_documents]');
					$setValue = 'FILE_TABLE';
					$showParent = $this->ParentWs;
					$content1 = $this->ParentID ? $this->formInherits('_ParentWs', $this->ParentWs, g_l('modules_users', '[inherit_ws]'), 'document.getElementById(\'info' . $setValue . '\').style.display=(this.checked?\'inline\':\'none\');') : '';
					break;
				default:
					continue 2;
			}
			$obj_values = $this->Name . '_Workspace_' . $table . '_AddDel';
			$obj_names = $this->Name . '_Workspace_' . $table;
			//$obj_def_names = $this->Name . '_defWorkspace_' . $k;
			//$content .= '<p>';

			$content1 .= '<input type="hidden" name="' . $obj_values . '" value="" /><table style="width:520px">';
			foreach($v as $key => $val){
				$value = $val;
				$path = fS('SELECT Path FROM ' . $table . ' WHERE ' . $table . '.ID=' . $value);
				if(!$path){
					$value = we_users_util::get_def_ws($table);
					$path = id2path($value);
				}


				$weSuggest->setAcId('WS' . $table . $key)
					->setContentType(we_base_ContentTypes::FOLDER_TYPES)
					->setInput($obj_names . '[Text][' . $key . ']', $path)
					->setMaxResults(10)
					->setResult($obj_names . '[id][' . $key . ']', (string) ($value))
					->setSelector(we_gui_suggest::DirSelector)
					->setTable($table)
					->setWidth(290);

				switch($table){
					case NEWSLETTER_TABLE :
						$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_newsletter_dirSelector',{id:document.getElementsByName('" . $obj_names . "[id][" . $key . "]')[0].value,targetID:'" . $obj_names . '[id][' . $key . "]',targetName:'" . $obj_names . '[Text][' . $key . "]',rootDirID:'" . we_base_request::int("rootDirID", 0) . "'})"));
						break;

					case NAVIGATION_TABLE:
						$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_navigation_dirSelector',{id:document.getElementsByName('" . $obj_names . "[id][" . $key . "]')[0].value,targetID:'" . $obj_names . '[id][' . $key . "]',targetName:'" . $obj_names . '[Text][' . $key . "]',rootDirID:'" . we_base_request::int('rootDirID', 0) . "'})"));
						break;

					default:
						$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:document.getElementsByName('" . $obj_names . "[id][" . $key . "]')[0].value,table:'" . $table . "',targetID:'" . $obj_names . '[id][' . $key . "]',targetName:'" . $obj_names . '[Text][' . $key . "]',rootDirID:'" . we_base_request::int("rootDirID", 0) . "'})"));
				}

				$weAcSelector = $weSuggest->getHTML();

				$content1 .= '
<tr><td colspan="2">' . $weAcSelector . '</td>
	<td><div style="position:relative; top:-1px">' . we_html_button::create_button(we_html_button::TRASH, "javascript:delElement(document.we_form." . $obj_values . "," . $key . ");switchPage(" . self::TAB_WORKSPACES . ");") . '</td></div>' . '
</tr>';
			}

			$content1 .= '<colgroup><col style="width:300px"/><col style="width:110px"/><col style="width:40px"/><col style="width:90px"/></colgroup>
	<tr><td colspan="4">' . we_html_button::create_button(we_html_button::PLUS, "javascript:top.content.setHot();addElement(document.we_form." . $obj_values . ");") . '</td></tr>
</table>';

			if($parentWsp[$table]){
				$parent = implode('<br/>', $this->db->getAllq('SELECT Path FROM ' . $table . ' WHERE ID IN(' . implode(',', $parentWsp[$table]) . ')', true));
			} else {
				$parent = ' - ';
			}

			$parts->add((new we_html_multiIconBoxContentLine($title, ($this->ParentID ? '<div id="info' . $setValue . '" style="' . ($showParent ? '' : 'display:none;') . '">' . we_html_tools::alertAttentionBox($parent, we_html_tools::TYPE_INFO, 600, false) . '</div>' : '') . $content1))
					->space(we_html_multiIconBox::SPACE_MED)
			);
		}

		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$filter = new we_navigation_customerFilter(0, we_customer_abstractFilter::FILTER, [], [], [], $this->workspaces[CUSTOMER_TABLE]);
			$view = new we_customer_filterView($filter, 520);
			if($parentWsp[CUSTOMER_TABLE]){

				$parent = '';
				foreach($parentWsp[CUSTOMER_TABLE] as $setting){
					'@phan-var string $setting';
					$setting = we_unserialize($setting);
					foreach($setting as $cur){
						$parent .= ($parent ? ' ' . $cur['logic'] . ' ' : '') . we_customer_abstractFilter::evalSingleFilterQuery($cur['operation'], $cur['field'], $cur['value']) . "\n";
					}
				}
			} else {
				$parent = ' - ';
			}
			$parts->add((new we_html_multiIconBoxContentLine(g_l('modules_users', '[workspace_customer]'), ($this->ParentID ?
							'<div id="infoCUSTOMER" style="' . ($this->ParentWsCust ? '' : 'display:none;') . '">' . we_html_tools::alertAttentionBox($parent, we_html_tools::TYPE_INFO, 600) . '</div>' .
							$this->formInherits('_ParentWsCust', $this->ParentWsCust, g_l('modules_users', '[inherit_cust]'), 'document.getElementById(\'infoCUSTOMER\').style.display=(this.checked?\'inline\':\'none\');') : '') .
						$view->getFilterCustomers()))
					->space(we_html_multiIconBox::SPACE_MED)
			);
		}

		return (new we_html_multiIconBox($parts))->marginLeft()->get();
	}

	/*
	  private function formPreferencesGlossary(): array{
	  $settings = [];
	  return [];
	  // Create checkboxes
	  $table = new we_html_table(['class' => 'default withSpace'], 2, 1);
	  //FIXME: where is the difference between force_glossary_check + force_glossary_action?!

	  $table->setCol(0, 0, we_html_forms::checkbox(1, $this->Preferences['force_glossary_check'], $this->Name . '_Preference_force_glossary_check', g_l('prefs', '[force_glossary_check]'), 'false', 'defaultfont', "top.content.setHot()"));
	  $table->setCol(1, 0, we_html_forms::checkbox(1, $this->Preferences['force_glossary_action'], $this->Name . "_Preference_force_glossary_action", g_l('prefs', '[force_glossary_action]'), "false", "defaultfont", "top.content.setHot()"));

	  // Build dialog if user has permission
	  if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
	  $settings[] = ['headline' => g_l('prefs', '[glossary_publishing]'), 'html' => $table->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true];
	  }

	  return $settings;
	  }
	 */

	private function formPreferences(we_base_jsCmd $jsCmd): string{
		$parts = new we_html_multiIconBoxContent();
		$language = [];
		// LANGUAGE
		//	Look which languages are installed ...
		$language_directory = dir(we_base_country::LANGDIR);
		while(false !== ($entry = $language_directory->read())){
			if($entry != '.' && $entry != '..'){
				if(is_dir(we_base_country::LANGDIR . $entry)){
					$language[$entry] = $entry;
				}
			}
		}
		global $languages;

		if(!empty($language)){ // Build language select box
			$languages = new we_html_select(['name' => $this->Name . '_Preference_Language', 'class' => 'weSelect']);
			$myCompLang = (string) (!empty($this->Preferences['Language']) ? $this->Preferences['Language'] : we_base_WE::$Language);

			foreach($language as $key => $value){
				$languages->addOption((string) $key, (string) $value);

				// Set selected extension
				if($key === $myCompLang){
					$languages->selectOption((string) $key);
				} else {
					// do nothing
				}
			}

			// Build dialog
			$parts->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[choose_language]'), $languages->getHtml()))
					->space(we_html_multiIconBox::SPACE_MED)
					->noline()
			);
		}



		//AMOUNT Number of Columns
		$amount = new we_html_select(['name' => $this->Name . '_Preference_cockpit_amount_columns', 'class' => 'weSelect', 'onchange' => "top.content.setHot();"]);
		if(!$this->Preferences['cockpit_amount_columns']){
			$this->setPreference('cockpit_amount_columns', '3');
		}
		for($i = 1; $i <= 10; $i++){
			$amount->addOption((string) ($i), (string) ($i));
			if($i == $this->Preferences['cockpit_amount_columns']){
				$amount->selectOption((string) ($i));
			}
		}

		$parts->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[cockpit_amount_columns]'), $amount->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED)
		);

		//SEEM
		// Cockpit
		$document_path = $object_path = '';
		$document_id = $object_id = 0;

		switch($this->Preferences['seem_start_type']){
			default:
				$seem_start_type = '0';
				break;
			case 'cockpit':
				$_SESSION['prefs']['seem_start_file'] = 0;
				$seem_start_type = 'cockpit';
				break;
			case 'object':
				$seem_start_type = 'object';
				if($this->Preferences['seem_start_file'] != 0){
					$object_id = (int) $this->Preferences['seem_start_file'];
					$object_path = $object_id ? id2path($object_id, OBJECT_FILES_TABLE) : '';
				}
				break;
			case 'weapp':
				$seem_start_type = 'weapp';
				break;
			// Document
			case 'document':
				$seem_start_type = 'document';
				if($this->Preferences['seem_start_file'] != 0){
					$document_id = (int) $this->Preferences['seem_start_file'];
					$document_path = $document_id ? id2path($document_id, FILE_TABLE) : '';
				}
		}

		$start_type = (new we_html_select([
				'name' => 'seem_start_type',
				'class' => 'weSelect',
				'id' => 'seem_start_type',
				'onchange' => "show_seem_chooser(this.value); top.content.setHot();"
				]))->addOptions([
			'0' => '-',
			'cockpit' => g_l('prefs', '[seem_start_type_cockpit]'),
			'document' => g_l('prefs', '[seem_start_type_document]')
		]);

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$start_type->addOption('object', g_l('prefs', '[seem_start_type_object]'));
		}

		$start_type->selectOption($seem_start_type);
		//$start_weapp->selectOption($this->Preferences['seem_start_weapp']);
		//$seem_weapp_chooser = we_html_element::span(['id' => 'span_seem_start_weapp', 'style' => 'display:none'], $start_weapp->getHtml());
		// Build SEEM select start document chooser
		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('Doc')
			->setContentType(array_merge(we_base_ContentTypes::LINK_ABLE, [we_base_ContentTypes::FOLDER_DOCUMENT]))
			->setInput('seem_start_document_name', $document_path)
			->setMaxResults(20)
			->setResult('seem_start_document', (string) ($document_id))
			->setSelector(we_gui_suggest::DocSelector)
			->setWidth(191)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, 'javascript:select_seem_start()', '', '', false, false));

		$seem_document_chooser = we_html_element::span(['id' => 'span_seem_start_document', 'style' => 'display:none'], $weSuggest->getHTML());

		// Build SEEM select start object chooser
		$weSuggest->setAcId('Obj')
			->setContentType([we_base_ContentTypes::FOLDER_OBJECTFILE, we_base_ContentTypes::OBJECT_FILE])
			->setInput('seem_start_object_name', $object_path)
			->setMaxResults(20)
			->setResult('seem_start_object', (string) ($object_id))
			->setSelector(we_gui_suggest::DocSelector)
			->setWidth(191)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, 'javascript:select_seem_start()', '', '', false, false));
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$weSuggest->setTable(OBJECT_FILES_TABLE);
		}

		$seem_object_chooser = we_html_element::span(['id' => 'span_seem_start_object', 'style' => 'display:none'], $weSuggest->getHTML());

		// Build final HTML code
		$seem_html = (new we_html_table(['class' => 'default'], 2, 1))
			->setCol(0, 0, $start_type->getHtml())
			->setCol(1, 0, $seem_document_chooser . $seem_object_chooser /* . $seem_weapp_chooser */);

		if(we_base_permission::hasPerm([we_base_perm::SEEM_CHANGE_START_DOCUMENT])){
			$jsCmd->addCmd('show_seem_chooser', $seem_start_type);
			$parts->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[seem_startdocument]'), $seem_html->getHtml()))
					->space(we_html_multiIconBox::SPACE_MED)
			);
		}

		// TREE

		$value_selected = false;
		$tree_count = $this->Preferences['default_tree_count'];

		$file_tree_count = (new we_html_select([
				'name' => $this->Name . '_Preference_default_tree_count',
				'class' => 'weSelect',
				'onchange' => 'top.content.setHot();'
				]))
			->addOption('0', g_l('prefs', '[all]'));

		if(0 == $tree_count){
			$file_tree_count->selectOption('0');
			$value_selected = true;
		}

		for($i = 10; $i < 51; $i += 10){
			$file_tree_count->addOption((string) ($i), (string) ($i));

			// Set selected extension
			if($i == $tree_count){
				$file_tree_count->selectOption((string) ($i));
				$value_selected = true;
			}
		}

		for($i = 100; $i < 501; $i += 100){
			$file_tree_count->addOption((string) $i, (string) $i);

			// Set selected extension
			if($i == $tree_count){
				$file_tree_count->selectOption((string) $i);
				$value_selected = true;
			}
		}

		if(!$value_selected){
			$file_tree_count->addOption((string) $tree_count, (string) $tree_count)
				// Set selected extension
				->selectOption((string) $tree_count);
		}

		$parts->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[tree_title]'), we_html_tools::alertAttentionBox(g_l('prefs', '[tree_count_description]'), we_html_tools::TYPE_INFO) . '<br/>' . $file_tree_count->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED)
		);

		return (new we_html_multiIconBox($parts))->marginLeft()->get();
	}

	function formAliasData(): string{
		$alias_text = ($this->ID ? fS('SELECT Path FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->Alias)) : '');
		$parent_text = ($this->ParentID == 0 ? '/' : fS('SELECT Path FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->ParentID)));

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('PathName')
			->setContentType([self::TYPE_USER, self::TYPE_USER_GROUP])
			->setInput($this->Name . '_Alias_Text', $alias_text, [], false, true)
			->setMaxResults(10)
			->setRequired(true)
			->setResult($this->Name . '_Alias', (string) $this->Alias)
			->setSelector(we_gui_suggest::DocSelector)
			->setTable(USER_TABLE)
			->setWidth(200)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_users_selector',{id:document.we_form." . $this->Name . "_Alias.value,targetName:'" . $this->Name . "_Alias_Text',filter:'noalias',targetID:'" . $this->Name . "_Alias'})"));

		$weAcSelectorName = $weSuggest->getHTML();

		$weSuggest->setAcId('PathGroup')
			->setContentType([we_base_ContentTypes::FOLDER_COMMON])
			->setInput($this->Name . '_ParentID_Text', $parent_text, [], false, true)
			->setMaxResults(10)
			->setResult($this->Name . '_ParentID', (string) ($this->ParentID))
			->setSelector(we_gui_suggest::DirSelector)
			->setTable(USER_TABLE)
			->setWidth(200)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_users_selector',{id:document.we_form." . $this->Name . "_ParentID.value,targetName:'" . $this->Name . "_ParentID_Text',filter:'" . we_users_user::TYPE_USER_GROUP . "',targetID:'" . $this->Name . "_ParentID'})"));

		$weAcSelectorGroup = $weSuggest->getHTML();

		$content = '
<table class="default" style="width:530px;">
<colgroup><col style="width:170px;"/><col style="width:330px;"/></colgroup>
	<tr>
		<td class="defaultfont">' . g_l('modules_users', '[user]') . ':</td>
		<td>' . $weAcSelectorName . '</td>
	</tr>
	<tr>
		<td colspan="2" style="height:5px;"></td>
	</tr>
	<tr>
		<td class="defaultfont">' . g_l('modules_users', '[group_member]') . ':</td>
		<td>' . $weAcSelectorGroup . '</td>
	</tr>
	<tr>
		<td colspan="2" style="height:1px;"></td>
	</tr>
</table>';

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('modules_users', '[alias_data]'), $content))
				->space(we_html_multiIconBox::SPACE_MED)
			)
			->add((new we_html_multiIconBoxContentLine(g_l('modules_users', '[rights_and_workspaces]', false, true),
																									$this->formInherits('_ParentPerms', $this->ParentPerms, g_l('modules_users', '[inherit]')) .
				$this->formInherits('_ParentWs', $this->ParentWs, g_l('modules_users', '[inherit_ws]')) .
				$this->formInherits('_ParentWst', $this->ParentWst, g_l('modules_users', '[inherit_wst]'))))
			->space(we_html_multiIconBox::SPACE_MED)
		);

		return (new we_html_multiIconBox($parts))->marginLeft()->get();
	}

	function formInherits(string $name, bool $value, string $title, string $onClick = ''): string{
		return '
<table class="default" style="width:500px;">
	<tr>
		<td>' .
			we_html_forms::checkbox('1', $value, $this->Name . $name, $title, false, '', 'top.content.setHot();' . $onClick) . '</td>
	</tr>
</table>';
	}

	function formHeader(int $tab = self::TAB_DATA): string{
		switch($this->Type){
			case self::TYPE_USER_GROUP:
				$headline1 = g_l('modules_users', '[group]') . ': ';
				$tabs = [
					self::TAB_DATA => ['data', 'fa-users'],
					self::TAB_PERMISSION => ['permissions', 'fa-user-secret'],
					self::TAB_WORKSPACES => ['workspace', we_gui_tabs::WE_ICON_WORKSPACE]
				];
				break;
			case self::TYPE_ALIAS:
				$tabs = [
					self::TAB_DATA => ['data', 'fa-user']
				];
				$headline1 = g_l('modules_users', '[menu][alias]') . ': ';
				break;
			case self::TYPE_USER:
			default:
				$tabs = [
					self::TAB_DATA => ['data', 'fa-user'],
					self::TAB_PERMISSION => ['permissions', 'fa-user-secret'],
					self::TAB_WORKSPACES => ['workspace', we_gui_tabs::WE_ICON_WORKSPACE],
					self::TAB_SETTINGS => ['preferences', we_gui_tabs::WE_ICON_PROPERTIES]
				];
				$headline1 = g_l('modules_users', '[menu][user]') . ': ';
		}

		$we_tabs = new we_gui_tabs();
		foreach($tabs as $key => $val){
			$we_tabs->addTab('', $val[1], ($tab == $key), (string) $key, ['title' => g_l('tabs', '[module][' . $val[0] . ']')]);
		}


		return we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
			we_html_element::jsScript(JS_DIR . 'initTabs.js') .
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'users/users_property.js') .
			we_html_element::div(['id' => 'main'],
												we_html_element::div(['id' => 'headrow'],
																 '<b>' . str_replace(' ', '&nbsp;', $headline1) . '&nbsp;</b>' .
					we_html_element::span(['id' => 'h_path', 'class' => 'header_small'], '<b id="titlePath">' . str_replace(' ', '&nbsp;', ($this->Path ?: $this->getPath($this->ParentID))) . '</b> (ID:' . $this->ID . ')') . we_html_element::div([
						'id' => 'mark'], '<i class="fa fa-asterisk modified"></i>')) . $we_tabs->getHTML()
		);
	}

	public static function getUsername(int $id): string{
		return fS('SELECT username FROM ' . USER_TABLE . ' WHERE ID=' . $id) ?: g_l('modules_messaging', '[userid_not_found]');
	}

	public static function getUserID(string $username): int{
		return (fI('SELECT ID FROM ' . USER_TABLE . ' WHERE username="' . DB_WE::$inst->escape(trim($username)) . '"') ?: -1);
	}

	static function filenameNotValid(string $username): bool{
		return preg_match('|^[A-Z0-9._\-][A-Z0-9.,_\-@]+$|i', $username) ? true : false;
	}

	/**
	 *
	 * @param int $user
	 * @param boolean $onlyParent
	 * @return array<string,array{key:string,value:array,parent:int,parentKey:string,explodeValue:bool}>|array<string,int[]>
	 */
	static function setEffectiveWorkspaces(int $user, bool $onlyParent = false): array{
		$workspaces = [
			FILE_TABLE => ['key' => 'workSpaceDoc', 'value' => [], 'parent' => 0, 'parentKey' => 'ParentWs', 'explodeValue' => true],
			TEMPLATES_TABLE => ['key' => 'workSpaceTmp', 'value' => [], 'parent' => 0, 'parentKey' => 'ParentWst', 'explodeValue' => true],
			NAVIGATION_TABLE => ['key' => 'workSpaceNav', 'value' => [], 'parent' => 0, 'parentKey' => 'ParentWsn', 'explodeValue' => true],
			OBJECT_FILES_TABLE => ['key' => 'workSpaceObj', 'value' => [], 'parent' => 0, 'parentKey' => 'ParentWso', 'explodeValue' => true],
			NEWSLETTER_TABLE => ['key' => 'workSpaceNwl', 'value' => [], 'parent' => 0, 'parentKey' => 'ParentWsnl', 'explodeValue' => true],
			CUSTOMER_TABLE => ['key' => 'workSpaceCust', 'value' => [], 'parent' => 0, 'parentKey' => 'ParentWsCust', 'explodeValue' => false],
		];
		'@phan-var array<string,array{key:string,value:array,parent:int,parentKey:string,explodeValue:bool}> $workspaces';

//FIXME: onlyParent doesn't work correctly
		$fields = ['ParentID'];
		foreach($workspaces as $cur){
			$fields[] = $cur['key'];
			$fields[] = $cur['parentKey'];
		}
		unset($cur);
		$fields = implode(',', $fields);

		$userGroups = []; //	Get Groups user belongs to.
		$db = DB_WE::$inst;
		$pids = [];
		$db->query('SELECT ' . $fields . ' FROM ' . USER_TABLE . ' WHERE ID=' . $user . ($onlyParent ? '' : ' OR Alias=' . $user));
		while($db->next_record()){
			$pids[] = $db->f('ParentID');
			foreach($workspaces as &$cur){
				$cur['parent'] = (int) $db->f($cur['parentKey']);
				if($onlyParent){//we only need parentID
					continue;
				}
				if($cur['explodeValue']){
					// get workspaces
					$a = explode(',', trim($db->f($cur['key']), ','));
					foreach($a as $v){
						$cur['value'][] = $v;
					}
				} else {
					$cur['value'][] = $db->f($cur['key']);
				}
				unset($cur);
			}
		}

		foreach($pids as $pid){
			while($pid){ //	For each group
				$userGroups[] = $pid;

				if(($row = getHash('SELECT ' . $fields . ' FROM ' . USER_TABLE . ' WHERE ID=' . intval($pid), $db))){
					$pid = $row['ParentID'];
					foreach($workspaces as &$cur){
						if($cur['parent']){
							// get workspaces
							if($cur['explodeValue']){

								$a = explode(',', trim($row[$cur['key']], ','));
								foreach($a as $v){
									$cur['value'][] = $v;
								}
							} else {
								$cur['value'][] = $row[$cur['key']];
							}
						}
						$cur['parent'] = $cur['parent'] && $row[$cur['parentKey']];
					}
					unset($cur);
				} else {
					$pid = 0;
				}
			}
		}
		foreach($workspaces as &$cur){
			$cur = array_values(array_unique(array_filter($cur['value'])));
		}
		unset($cur);
		if($onlyParent){
			return $workspaces;
		}
		$_SESSION['user']['workSpace'] = $workspaces;
		$_SESSION['user']['groups'] = $userGroups; //	order: first is folder with user himself (deepest in tree)

		if($_SESSION['user']['workSpace'][CUSTOMER_TABLE]){
			//setup customer
			$filter = [];
			foreach($_SESSION['user']['workSpace'][CUSTOMER_TABLE] as $cur){
				$filter[] = we_customer_abstractFilter::getQueryFromFilter(we_unserialize((string) $cur));
			}

			//FIXME: this won't hold for alias users
			$_SESSION['user']['workSpace'][CUSTOMER_TABLE] = [implode(' AND ', array_filter($filter))];
		}
		return [];
	}

	/**
	 *
	 * @param string $storedPassword DB-field!!! //needs to be cause of salt!
	 * @param string $clearPassword //posted password
	 * @return boolean
	 */
	static function comparePasswords(string $storedPassword, string $clearPassword): bool{
		$matches = [];
		$useSalt = (!preg_match('|^\$([^$]{2,4})\$([^$]+)\$(.+)$|', $storedPassword, $matches) ?
			//old md5
			self::SALT_MD5 :
			$matches[1]);
		$clearPassword = substr($clearPassword, 0, self::MAX_PWD_LEN);
		switch($useSalt){
			default:
			case self::SALT_MD5:
				return false;
			case '2y':
				$passwd = crypt($clearPassword, $storedPassword);
				break;
		}
		return ($passwd === $storedPassword);
	}

	public static function getHashIV(int $len): string{
		static $WE_SALTCHARS = './0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
		$salt = '';
		for($i = 0; $i < $len; $i++){
			$tmp_str = str_shuffle($WE_SALTCHARS);
			$salt .= $tmp_str[0];
		}
		return $salt;
	}

	public static function makeSaltedPassword(string $passwd, int $strength = 15): string{
		return crypt(substr($passwd, 0, self::MAX_PWD_LEN), '$2y$' . sprintf('%02d', $strength) . '$' . self::getHashIV(22)) ?: '';
	}

	static function readPrefs(int $id, bool $login = false): array{
//set defaults
		$ret = ['userID' => $id];
		foreach(we_base_preferencesConfig::USER as $key => $vals){
			$ret[$key] = $vals[2];
		}
		$db = DB_WE::$inst;
		if($login){
			$db->query('DELETE FROM ' . PREFS_TABLE . ' WHERE `key` NOT IN("' . implode('","', array_keys(we_base_preferencesConfig::USER)) . '")');
		}
		//read db
		foreach($db->getAllFirstq('SELECT `key`,`value` FROM ' . PREFS_TABLE . ' WHERE userID=' . $id, false) as $key => $val){
			switch(we_base_preferencesConfig::USER[$key][1]){
				case we_base_request::INTLISTA:
					$ret[$key] = we_base_request::filterIntList($val);
					break;
				default:
					$ret[$key] = $val;
					break;
			}
		}
		if($login){
			$_SESSION['prefs'] = $ret;
			self::writePrefs($id);
		}
		return $ret;
	}

	/** write settings for a user, all default values are applied before data is written
	 * @id int user id to write settings for
	 * @db socket database connection
	 * @data array optional if empty settings of current session are used.
	 */
	static function writePrefs(int $id, array $data = []): void{
		$db = DB_WE::$inst;
		if($data){//note data must be different to session data
			$old = ['userID' => $id];
			foreach(we_base_preferencesConfig::USER as $key => $vals){
				//only write config data, if data is read! otherwise we overwrite some settings
				if(isset($data[$key])){
					$old[$key] = $vals[2];
				}
			}
		} else {
			$old = self::readPrefs($id);
			$data = $_SESSION['prefs'];
		}
		$upd = [];
		foreach($old as $key => $val){
			if($key != 'userID' && (!isset($data[$key]) || $data[$key] != $val)){
				$upd[] = we_database_base::arraySetter([
						'userID' => $id,
						'key' => $key,
						'value' => isset($data[$key]) ? (is_array($data[$key]) ? implode(',', $data[$key]) : $data[$key]) : (is_array($val) ? implode(',', $val) : $val)
						], ',', true);
			}
		}
		if(!empty($upd)){
			$db->query('REPLACE INTO ' . PREFS_TABLE . ' (`userID`,`key`,`value`) VALUES ' . implode(',', $upd));
		}
	}

	/**
	 * @param string $table
	 * @param string $user
	 */
	public static function logLoginFailed(string $table, string $user): void{
		$db = DB_WE::$inst;
		$db->query('INSERT INTO ' . FAILED_LOGINS_TABLE . ' SET ' . we_database_base::arraySetter([
				'UserTable' => $table,
				'Username' => $user,
				'IP' => $_SERVER['REMOTE_ADDR'],
				'Servername' => $_SERVER['SERVER_NAME'],
				'Port' => $_SERVER['SERVER_PORT'],
				'Script' => we_http_util::getScriptName()
		]));
	}

	/**
	 *
	 * @param bool $onlyUpdate
	 * @param array<string,int[]> $activeDocuments
	 * @return ?array{requests:array{ID:int,tbl:string,text:string,forceDate:string,forceTime:string,User:string,username:string},unlock:array{tbl:string,ids:int[]},reply:array,open:array{tbl:string,ids:string}}
	 */
	public static function updateActiveUser(bool $onlyUpdate = false, array $activeDocuments = []): ?array{
		if(!we_users_user::getID()){
			throw new we_exception_base('User not logged-in');
		}

		DB_WE::$inst->query('UPDATE ' . USER_TABLE . ' SET Ping=NOW() WHERE ID=' . we_users_user::getID());
		if($onlyUpdate){
			return null;
		}

		$ret = [
			'requests' => [],
			'reply' => [],
			'unlock' => [],
			'open' => []
		];
		//FIXME do we need to check session_id??? for release requests?
		//check release requests
		$ret['requests'] = DB_WE::$inst->getAllq('SELECT l.ID,CONCAT("' . TBL_PREFIX . '",l.tbl) as tbl,l.releaseRequestText AS text,DATE_FORMAT(l.releaseRequestForce,"' . g_l('date', '[format][mysql]') . '") AS forceDate,TIMESTAMPDIFF(SECOND,NOW(),releaseRequestForce) AS forceTime,CONCAT(u.First," ",u.Second) AS User, u.username FROM ' . LOCK_TABLE . ' l JOIN ' . USER_TABLE . ' u ON (l.releaseRequestID=u.ID) WHERE l.UserID=' . we_users_user::getID() . ' AND l.releaseRequestText IS NOT NULL AND l.releaseRequestID!=' . we_users_user::getID());

		//check my release requests - currently unused
		$ret['reply'] = []; //DB_WE::$inst->getAllq('SELECT l.ID,l.tbl,l.releaseRequestReply AS text,CONCAT(u.First," ",u.Second," (",u.Username,")") AS User FROM ' . LOCK_TABLE . ' l JOIN ' . USER_TABLE . ' u ON (l.UserID=u.ID) WHERE l.releaseRequestReply IS NOT NULL AND l.releaseRequestID=' . intval($_SESSION['user']['ID']) . ' AND l.UserID!=' . intval($_SESSION['user']['ID']));
//get all documents that have been released
		if(($ret['unlock'] = DB_WE::$inst->getAllFirstq('SELECT CONCAT("' . TBL_PREFIX . '",l.tbl) as tbl,GROUP_CONCAT(l.ID) AS ids FROM ' . LOCK_TABLE . ' l WHERE l.UserID=' . we_users_user::getID() . ' AND l.releaseRequestID=l.UserID AND l.sessionID=x\'0000000000000000000000000000000000000000\' GROUP BY l.tbl', false))){
			foreach($ret['unlock'] as $table => &$ids){
				DB_WE::$inst->query('UPDATE ' . LOCK_TABLE . ' SET releaseRequestID=NULL, sessionID=x\'' . we_base_sessionHandler::getCurrentHex() . '\' WHERE UserID=' . we_users_user::getID() . ' AND releaseRequestID=UserID AND tbl="' . stripTblPrefix((string) $table) . '" AND ID IN(' . $ids . ') AND sessionID=x\'0000000000000000000000000000000000000000\'');
				$ids = we_base_request::filterIntList($ids);
			}
		}

		//reset all replies
//DB_WE::$inst->query('UPDATE ' . LOCK_TABLE . ' SET releaseRequestReply=NULL,releaseRequestForce=NULL WHERE releaseRequestReply IS NOT NULL AND releaseRequestID=' . intval($_SESSION['user']['ID']));
		//update my locks, reset releaseRequestText
		if($activeDocuments){
			$where = [];
			foreach($activeDocuments as $tab => $ids){
				$where[] = '(tbl="' . DB_WE::$inst->escape(stripTblPrefix((string) $tab)) . '" AND ID IN(' . implode(',', array_map('intval', $ids)) . '))';
			}
			DB_WE::$inst->query('UPDATE ' . LOCK_TABLE . ' SET
	lockTime=NOW()+INTERVAL ' . SYSTEM_WE_SESSION_TIME . ' SECOND,
	releaseRequestText=NULL
WHERE
	UserID=' . we_users_user::getID() . ' AND
	sessionID=x\'' . we_base_sessionHandler::getCurrentHex() . '\' AND
	(' . implode(' OR ', $where) . ')');

			$ret['open'] = DB_WE::$inst->getAllq('SELECT tbl,GROUP_CONCAT(ID) AS ids FROM ' . LOCK_TABLE . '
WHERE
	UserID=' . we_users_user::getID() . ' AND
	(' . implode(' OR ', $where) . ')
GROUP BY tbl');
		}
		return $ret;
	}

	public static function unlockDocuments(array $ids = [], array $tables = [], string $we_transaction = ''): void{
		// prepare the queries, 4 as maximum.
		$ids = $ids ?: we_base_request::intList('we_cmd', [], 1); // we_cmd[1] is commaseperated list of ids
//FIXME: make tblList???
		$tables = $tables ?: we_base_request::stringList('we_cmd', [], 3); // we_cmd[3] is commaseparated list of tables
		$we_transaction = $we_transaction ?: we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', null, 4); // we_cmd[4] is a single transaction, to delete data from session

		$queries = [];

		if($we_transaction){ // clean session
			try{
				$doc = we_contents_root::getSessionDocument($we_transaction);
				unset(
					$_SESSION['weS']['versions']['versionToCompare'][$doc->Table][$doc->ID],
		 $_SESSION['weS']['we_data'][$we_transaction]); // we_transaction is resetted here
			} catch (Exception $e){

			}
		}

		foreach($ids as $i => $id){
			if($tables[$i] && $id){
				$queries[$tables[$i]][] = $id;
			}
		}

		$DB_WE = DB_WE::$inst;
		//$uid = we_base_request::_(we_base_request::INT, 'we_cmd', 0, 2);
		foreach($queries as $table => $ids){
			//don't clean all locks! - is this really a needed statement???
			$strippedTable = $DB_WE->escape(stripTblPrefix((string) $table));
			$ids = implode(', ', array_filter($ids));
			if($ids){
				$DB_WE->query('UPDATE ' . LOCK_TABLE . ' SET sessionID=x\'00\',UserID=releaseRequestID WHERE releaseRequestID IS NOT NULL AND releaseRequestID!=UserID AND tbl="' . $strippedTable . '" AND ID IN (' . $ids . ') AND UserID=' . we_users_user::getID() . ' AND sessionID=x\'' . we_base_sessionHandler::getCurrentHex() . '\'');
				$DB_WE->query('DELETE FROM ' . LOCK_TABLE . ' WHERE tbl="' . $strippedTable . '" AND ID IN (' . $ids . ') AND sessionID!=x\'0000000000000000000000000000000000000000\' AND UserID=' . we_users_user::getID());
			}
		}
		echo '{"UNLOCKED":true}';
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
				'view' => [
					'give_org_name' => g_l('modules_users', '[give_org_name]'),
					'password_alert' => (g_l('modules_users', '[password_alert]')),
					'save_changed_user' => g_l('modules_users', '[save_changed_user]'),
					'delete_alert' => [
						self::TYPE_USER_GROUP => g_l('modules_users', '[delete_alert_group]'),
						self::TYPE_ALIAS => g_l('modules_users', '[delete_alert_alias]'),
						self::TYPE_USER => g_l('modules_users', '[delete_alert_user]'),
					]
				]
		]);
	}

	public static function getID(): int{
		return (int) ($_SESSION['user']['ID'] ?? 0);
	}

}
