<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 88b57b26b3c0 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-09-10 17:18 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_workflow_cmd implements we_interface_cmd{

	public static function we_cmd(string $cmd0): bool{
		$cmd = we_base_request::_(we_base_request::RAW, 'cmd', '');
		$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', we_base_request::_(we_base_request::TRANSACTION, 'we_transaction', 0), 1);

		$wf_text = we_base_request::string('wf_text', '');
//init document
		$we_doc = we_contents_root::getSessionDocument($we_transaction);
		$jsCmd = new we_base_jsCmd();
		$body = '';
		switch($cmd0){
			case 'we_workflow_finish':
				self::finishWorkflowWin($we_transaction, $we_doc);
				return true;
			case 'we_workflow_isIn':
				$body = self::inWorkflowWindow($cmd, $we_transaction, $we_doc, $jsCmd, $wf_text);
				break;
			case 'we_workflow_pass':
				$body = self::passWorkflowWin($cmd, $we_transaction, $we_doc, $jsCmd, $wf_text);
				break;
			case 'we_workflow_decline':
				$body = self::declineWindow($cmd, $we_transaction, $we_doc, $jsCmd, $wf_text);
				break;
		}
		echo we_html_tools::getHtmlTop('', we_html_element::jsScript(WE_JS_MODULES_DIR . 'workflow/we_workflow_win.js') . $jsCmd->getCmds(), we_html_element::body([
				'class' => 'weDialogBody'
				], $body));
		return true;
	}

	private static function declineWindow(string $cmd, string $we_transaction, we_contents_root $we_doc, we_base_jsCmd $jsCmd, string $wf_text): string{
		if($cmd === 'ok'){
			$force = (!we_workflow_utility::isUserInWorkflow($we_doc->ID, $we_doc->Table, we_users_user::getID()));

			if(we_workflow_utility::decline($we_doc->ID, $we_doc->Table, we_users_user::getID(), $wf_text, $force)){
				$jsCmd->addMsg(g_l('modules_workflow', '[' . stripTblPrefix($we_doc->Table) . '][decline_workflow_ok]'), we_base_util::WE_MESSAGE_NOTICE);
				//	in SEEM-Mode back to Preview page
				if(we_editor_functions::isSeeMode()){
					$jsCmd->addCmd('switch_edit_page', we_base_constants::WE_EDITPAGE_PREVIEW, $we_transaction);
				} else if(we_editor_functions::isNormalMode()){
					$jsCmd->addCmd('reloadFooter');
				}

				if(($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_PROPERTIES || $we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_INFO)){
					$jsCmd->addCmd('switch_edit_page', $we_doc->EditPageNr, $we_transaction); // will be inserted into the template
				}
			} else {
				$jsCmd->addMsg(g_l('modules_workflow', '[' . stripTblPrefix($we_doc->Table) . '][decline_workflow_notok]'), we_base_util::WE_MESSAGE_ERROR);
				//	in SEEM-Mode back to Preview page
				if(we_editor_functions::isSeeMode()){
					$jsCmd->addCmd('switch_edit_page', we_base_constants::WE_EDITPAGE_PREVIEW, $we_transaction);
				} else if(we_editor_functions::isNormalMode()){

				}
			}
			$jsCmd->addCmd('close_dialog_delayed', 2400);
		}

		return '<form action="' . WEBEDITION_DIR . 'we_cmd.php" method="post">' .
			we_html_tools::dialogLayout(/* g_l('modules_workflow', '[message]') . we_html_element::br() . */
				'<textarea name="wf_text" rows="7" cols="50" style="width:97%;height:190px"></textarea>', we_html_element::div(['style' => 'text-align:center'], g_l('modules_workflow', '[decline_workflow]')), we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:document.forms[0].submit()"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose()")), false) . we_html_element::hiddens([
				'cmd' => 'ok',
				'we_cmd[0]' => we_base_request::string('we_cmd', '', 0),
				'we_cmd[1]' => $we_transaction]) .
			'</form>';
	}

	private static function inWorkflowWindow(string $cmd, string $we_transaction, we_contents_root $we_doc, we_base_jsCmd $jsCmd, string $wf_text): string{
		$cmd2 = we_base_request::bool('we_cmd', false, 2);

		if($cmd === 'ok'){
			$wf = we_base_request::int('wf_select');
			if(we_workflow_utility::insertDocInWorkflow($we_doc->ID, $we_doc->Table, $wf, we_users_user::getID(), $wf_text)){
				$jsCmd->addMsg(g_l('modules_workflow', '[' . stripTblPrefix($we_doc->Table) . '][in_workflow_ok]'), we_base_util::WE_MESSAGE_NOTICE);
				switch($_SESSION['weS']['we_mode']){
					case we_base_constants::MODE_SEE:
						$jsCmd->addCmd('switch_edit_page', we_base_constants::WE_EDITPAGE_PREVIEW, $we_transaction);
						break;
					default:
					case we_base_constants::MODE_NORMAL:
						$jsCmd->addCmd('reloadFooter');
				}

				if($cmd2){ // make same new
					$we_doc->makeSameNew();
					$we_doc->saveInSession($we_transaction); // save the changed object in session
					$jsCmd->addCmd('switch_edit_page', $we_doc->EditPageNr, $we_transaction); // wird in Templ eingef�gt
				} elseif(in_array($we_doc->EditPageNr, [we_base_constants::WE_EDITPAGE_PROPERTIES, we_base_constants::WE_EDITPAGE_INFO], true)){
					$jsCmd->addCmd('switch_edit_page', $we_doc->EditPageNr, $we_transaction); // wird in Templ eingef�gt
				}
			} else {
				$jsCmd->addMsg(g_l('modules_workflow', '[' . stripTblPrefix($we_doc->Table) . '][in_workflow_notok]'), we_base_util::WE_MESSAGE_ERROR);
				switch($_SESSION['weS']['we_mode']){
					case we_base_constants::MODE_SEE:
						$jsCmd->addCmd('switch_edit_page', we_base_constants::WE_EDITPAGE_PREVIEW, $we_transaction);
						break;
					default:
					case we_base_constants::MODE_NORMAL:
				}
			}
			$jsCmd->addCmd('close_dialog_delayed', 2400);
		}

		$all = [];
		'@phan-var int[] $all';
		switch($we_doc->Table){
			case FILE_TABLE:
				assert($we_doc instanceof we_document_textContent);
				$wfDoc = we_workflow_utility::getWorkflowDocumentForDoc($we_doc->DocType, $we_doc->Category, $we_doc->ParentID, $all);
				break;
			case OBJECT_FILES_TABLE:
				assert($we_doc instanceof we_contents_objectFile);
				$wfDoc = we_workflow_utility::getWorkflowDocumentForObject($we_doc->TableID, $we_doc->Category, $we_doc->ParentID, $all);
				break;
			default:
				return '';
		}

		$wfID = $wfDoc->workflowID;
		if(!$wfID){
			$jsCmd->addMsg(g_l('modules_workflow', ($we_doc->Table === FILE_TABLE ? '[no_wf_defined]' : '[no_wf_defined_object]')), we_base_util::WE_MESSAGE_ERROR);
			$jsCmd->addCmd('close_dialog_delayed', 2400);
		}

		$wfs = we_workflow_utility::getAllWorkflows(we_workflow_workflow::ACTIVE, $we_doc->Table, $all);
		$wf_select = '<select style="width:97%" name="wf_select">';
		foreach($wfs as $wID => $wfname){
			$wf_select .= '<option value="' . $wID . '"' . (($wID == $wfID) ? ' selected' : '') . '>' . htmlspecialchars((string) $wfname) . '</option>';
		}
		$wf_select .= '</select>';

		$okbut = we_html_button::create_button(we_html_button::OK, "javascript:document.forms[0].submit()");
		$cancelbut = we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose()");

		return '<form action="' . WEBEDITION_DIR . 'we_cmd.php" method="post">' .
			we_html_tools::dialogLayout(/* g_l('modules_workflow', '[workflow]') . */ $wf_select . we_html_element::br() .
				/* g_l('modules_workflow', '[message]') . we_html_element::br() . */
				'<textarea name="wf_text" rows="5" cols="50" style="height:150px; width:97%;"></textarea>', we_html_element::div(['style' => 'text-align:center'], g_l('modules_workflow', '[in_workflow]')), we_html_button::position_yes_no_cancel($okbut, '', $cancelbut), false) .
			we_html_element::hiddens(['cmd' => 'ok',
				'we_cmd[0]' => we_base_request::string('we_cmd', '', 0),
				"we_cmd[1]" => $we_transaction,
				"we_cmd[2]" => $cmd2
			]) . '
		</form>';
	}

	private static function passWorkflowWin(string $cmd, string $we_transaction, we_contents_root $we_doc, we_base_jsCmd $jsCmd, string $wf_text): string{
		if($cmd === 'ok'){
			$force = (!we_workflow_utility::isUserInWorkflow($we_doc->ID, $we_doc->Table, we_users_user::getID()));

			if(we_workflow_utility::approve($we_doc->ID, $we_doc->Table, we_users_user::getID(), $wf_text, $force)){
				$jsCmd->addMsg(g_l('modules_workflow', '[' . stripTblPrefix($we_doc->Table) . '][pass_workflow_ok]'), we_base_util::WE_MESSAGE_NOTICE);

				//	in SEEM-Mode back to Preview page
				switch($_SESSION['weS']['we_mode']){
					case we_base_constants::MODE_SEE:
						$jsCmd->addCmd('switch_edit_page', we_base_constants::WE_EDITPAGE_PREVIEW, $we_transaction);
						break;
					default:
					case we_base_constants::MODE_NORMAL:
						$jsCmd->addCmd('reloadFooter');
				}

				if(($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_PROPERTIES || $we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_INFO)){
					$jsCmd->addCmd('switch_edit_page', $we_doc->EditPageNr, $we_transaction); // wird in Templ eingef�gt
				}
			} else {
				$jsCmd->addMsg(g_l('modules_workflow', '[' . stripTblPrefix($we_doc->Table) . '][pass_workflow_notok]'), we_base_util::WE_MESSAGE_ERROR);
				//	in SEEM-Mode back to Preview page
				switch($_SESSION['weS']['we_mode']){
					case we_base_constants::MODE_SEE:
						$jsCmd->addCmd('switch_edit_page', we_base_constants::WE_EDITPAGE_PREVIEW, $we_transaction);
						break;
					default:
					case we_base_constants::MODE_NORMAL:
				}
			}
			$jsCmd->addCmd('close_dialog_delayed', 2400);
		}
		$okbut = we_html_button::create_button(we_html_button::OK, "javascript:document.forms[0].submit()");
		$cancelbut = we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose()");

		return '<form action="' . WEBEDITION_DIR . 'we_cmd.php" method="post">' .
			we_html_tools::dialogLayout(/* g_l('modules_workflow', '[message]') . we_html_element::br() . */ '<textarea name="wf_text" rows="7" cols="50" style="width:97%;height:190px"></textarea>', we_html_element::div([
					'style' => 'text-align:center'], g_l('modules_workflow', '[pass_workflow]')), we_html_button::position_yes_no_cancel($okbut, "", $cancelbut), false) .
			we_html_element::hiddens(['cmd' => "ok",
				'we_cmd[0]' => we_base_request::string('we_cmd', '', 0),
				"we_cmd[1]" => $we_transaction
			]) . '
				</form>';
	}

	public static function finishWorkflowWin(string $we_transaction, we_contents_root $we_doc): void{
		$wejs = [];
		if(we_workflow_utility::approve($we_doc->ID, $we_doc->Table, we_users_user::getID(), '', true)){
			try{
				$time = $we_doc->publInScheduleTable();
				$we_responseText = sprintf(g_l('weEditor', '[' . $we_doc->ContentType . '][autoschedule]'), $time->format(g_l('date', '[format][default]')));
				$we_responseTextType = we_base_util::WE_MESSAGE_NOTICE;
			} catch (we_exception_base $e){
				if($we_doc->we_publish()){
					$wejs = [
						['setEditorDocumentId', $we_doc->ID],
						$we_doc->getUpdateTreeCmd(true, null)
					];
					$we_responseText = sprintf(g_l('weEditor', '[' . $we_doc->ContentType . '][response_publish_ok]'), $we_doc->Path);
					$we_responseTextType = we_base_util::WE_MESSAGE_NOTICE;
					if(($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_PROPERTIES || $we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_INFO)){
						we_editor_save::$responseJS[] = ['switch_edit_page', $we_doc->EditPageNr, we_base_WE::$transaction]; // wird in Templ eingef�gt
					}
					$wejs[] = ['we_gui_reloadEditFooter', $we_transaction];
					$wejs[] = ['setEditorDocumentId', $we_doc->ID];
					$wejs[] = $we_doc->getUpdateTreeCmd(true, null);
				} else {
					$we_responseText = sprintf(g_l('weEditor', '[' . $we_doc->ContentType . '][response_publish_notok]'), $we_doc->Path);
					$we_responseTextType = we_base_util::WE_MESSAGE_ERROR;
				}
			}
		} else {
			$we_responseText = g_l('modules_workflow', '[' . stripTblPrefix($we_doc->Table) . '][pass_workflow_notok]');
			$we_responseTextType = we_base_util::WE_MESSAGE_ERROR;
		}

		we_editor_functions::saveInc($we_transaction, $we_doc, $we_responseText, $we_responseTextType, $wejs, !empty($GLOBALS['wasSaved']), !empty($GLOBALS['saveTemplate']), we_editor_save::$responseJS, !empty($GLOBALS['isClose']), !empty($GLOBALS['showAlert']), we_editor_functions::$publish_doc);
	}

}
