/* global WE, CropTool, _EditorFrame */

/**
 * webEdition CMS
 *
 * $Rev: 6329b796be62 $
 * $Author: Lukas Imhof $
 * $Date: 2020-11-10 16:09 +0100 $
 *
 * This source is part of the webEdition SDK. The webEdition SDK is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU Lesser General Public License can be found at
 * http://www.gnu.org/licenses/lgpl-3.0.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionSDK/License.txt
 *
 *
 * @license    http://www.gnu.org/licenses/lgpl-3.0.html  LGPL
 */
'use strict';

WE().layout.checkInitImageEditTools = function (win) {
	if (!win.ImageEditTools) {
		var img = win.document.getElementById('weImage');
		var origW = win.document.we_form.origWidth.value ? win.document.we_form.origWidth.value : (img ? img.width : 0);
		var origH = win.document.we_form.origHeight.value ? win.document.we_form.origHeight.value : (img ? img.height : 0);

		win.ImageEditTools = new (WE().layout.ImageEditTools)(win, origW, origH);
	}
};

WE().layout.ImageEditTools = function (win, origWidth, origHeight) {
	this.size = {
		origW: origWidth ? origWidth : 0,
		origH: origHeight ? origHeight : 0
	};
	this.win = win;
	this.btns = null;

	this.deactivateAll = function () {
		switch (this.activeTool) {
			case 'focus':
				this.Focus.drop();
				break;
			case 'crop':
				this.Crop.drop();
				break;
			case 'resize':
				this.Resize.drop();
				break;
			case 'rotate':
				this.Rotate.drop();
				break;
			case 'convertJPEG':
				this.ConvertJPEG.drop();
				break;
			case 'editFull':
				this.EditFull.drop();
				break;
			default:
		}

		this.btns = this.btns || this.win.document.getElementsByClassName('imgEditBtn');
		Array.prototype.map.call(this.btns, function (btn) {
			btn.classList.remove('active');
		});

		this.activeTool = '';
	};

	this.setBtnActive = function (tool, active) {
		var btn;
		switch (tool) {
			case 'focus':
				btn = this.win.document.getElementsByClassName('imgFocus')[0];
				break;
			case 'crop':
				btn = this.win.document.getElementsByClassName('imgCrop')[0];
				break;
			case 'resize':
				btn = this.win.document.getElementsByClassName('imgResize')[0];
				break;
			case 'rotate':
				btn = this.win.document.getElementsByClassName('imgRotate')[0];
				break;
			case 'convertJPEG':
				btn = this.win.document.getElementsByClassName('imgJpg')[0];
				break;
			case 'editFull':
				btn = this.win.document.getElementsByClassName('imgFull')[0];
				break;
			default:
		}
		if (active) {
			btn.classList.add('active');
		} else {
			btn.classList.remove('active');
		}
	};

	this.Resize = {
		editTools: this,
		dialog: null,
		start: function (url, gdType) {
			if (this.editTools.activeTool === 'resize') {
				this.drop();
				return;
			}

			this.editTools.deactivateAll();
			this.editTools.setBtnActive('resize', true);
			this.editTools.activeTool = 'resize';
			this.dialog = (new (WE().layout.overlay)(this.editTools.win, this.editTools.win.document.getElementById('weImagePanel'), url, "we_image_resize", WE().consts.size.dialog.tiny, WE().consts.size.dialog.tiny)).setCmdClose('imageEditTools_reset').open(true);
		},
		drop: function () {
			if (this.dialog) {
				this.dialog.doClose();
				this.dialog = null;
			}
			this.editTools.activeTool = this.editTools.activeTool === 'resize' ? '' : this.editTools.activeTool;
			this.editTools.setBtnActive('resize', false);
		}
	};

	this.Rotate = {
		editTools: this,
		dialog: null,
		start: function (url, gdType) {
			if (this.editTools.activeTool === 'rotate') {
				this.drop();
				return;
			}

			this.editTools.deactivateAll();
			this.editTools.setBtnActive('rotate', true);
			this.editTools.activeTool = 'rotate';
			this.dialog = (new (WE().layout.overlay)(this.editTools.win, this.editTools.win.document.getElementById('weImagePanel'), url, "we_rotate", WE().consts.size.dialog.tiny, WE().consts.size.dialog.tiny)).setCmdClose('imageEditTools_reset').open(true);
		},
		drop: function () {
			if (this.dialog) {
				this.dialog.doClose();
				this.dialog = null;
			}
			this.editTools.activeTool = this.editTools.activeTool === 'rotate' ? '' : this.editTools.activeTool;
			this.editTools.setBtnActive('rotate', false);
		}
	};

	this.ConvertJPEG = {
		editTools: this,
		dialog: null,
		start: function (url) {
			if (this.editTools.activeTool === 'convertJPEG') {
				this.drop();
				return;
			}

			this.editTools.deactivateAll();
			this.editTools.setBtnActive('convertJPEG', true);
			this.editTools.activeTool = 'convertJPEG';
			this.dialog = (new (WE().layout.overlay)(this.editTools.win, this.editTools.win.document.getElementById('weImagePanel'), url, "e_convert_jpg", WE().consts.size.dialog.tiny, WE().consts.size.dialog.xtiny)).setCmdClose('imageEditTools_reset').open(true);
		},
		drop: function () {
			if (this.dialog) {
				this.dialog.doClose();
				this.dialog = null;
			}

			this.editTools.activeTool = this.editTools.activeTool === 'convertJPEG' ? '' : this.editTools.activeTool;
			this.editTools.setBtnActive('convertJPEG', false);
		}
	};

	this.Focus = {
		editTools: this,
		win: this.win,
		state: {
			isMoveFocus: false
		},
		elems: {
			focusPoint: null,
			image: null,
			x_focus: null,
			y_focus: null
		},
		vals: {
			referenceX: 0,
			referenceY: 0,
			origLeft: 0,
			origTop: 0
		},
		boundFNs: {
			setFocus: null,
			moveFocus: null,
			startMove: null,
			stopMove: null,
			leaveBody: null
		},
		start: function () {
			if (this.editTools.activeTool === 'focus') {
				this.drop();
				return;
			}

			this.editTools.deactivateAll();
			this.editTools.setBtnActive('focus', true);
			this.editTools.activeTool = 'focus';

			// references to some gui elements
			this.elems.focusPoint = this.win.document.getElementById('imgfocus_point');
			this.elems.image = this.win.document.getElementById('weImage');
			this.elems.x_focus = this.win.document.getElementById('x_focus');
			this.elems.y_focus = this.win.document.getElementById('y_focus');
			this.elems.info = this.win.document.getElementById('focus_info');

			// set tool visible
			this.elems.focusPoint.style.display = 'block';
			this.elems.focusPoint.style.cursor = 'move';
			this.elems.image.style.cursor = 'crosshair';
			this.elems.image.className = 'editfocus';
			this.elems.info.style.display = 'block';

			// add listeners (using late binding to set scope, and hold reference to bound fns!)
			this.elems.image.addEventListener('click', (this.boundFNs.setFocus = this.setFocusPositionByMouse.bind(this)), false);
			this.elems.focusPoint.addEventListener('mousedown', (this.boundFNs.startMove = this.startMoveFocusPosition.bind(this)), false);
			this.win.document.body.addEventListener('mouseup', (this.boundFNs.stopMove = this.stopMoveFocusPosition.bind(this)), false);
			this.win.document.body.addEventListener('mouseleave', (this.boundFNs.leaveBody = this.mouseLeaveBody.bind(this)), false);

			this.elems.focusPoint.addEventListener('dragstart', function (e) {
				e.preventDefault();
			}, false);

			var hot = this.win._EditorFrame.getEditorIsHot();
			this.setFocusPositionByValue();
			this.win._EditorFrame.setEditorIsHot(hot);
		},
		drop: function () { // when leaving edit focus
			this.editTools.activeTool = this.editTools.activeTool === 'focus' ? '' : this.editTools.activeTool;
			this.editTools.setBtnActive('focus', false);
			this.elems.focusPoint.style.display = 'none';
			this.elems.image.style.cursor = 'default';
			this.win.document.body.style.cursor = 'default';
			this.elems.image.removeEventListener('click', this.boundFNs.setFocus, false);
			this.elems.focusPoint.removeEventListener('mousedown', this.boundFNs.startMove, false);
			this.win.document.body.removeEventListener('mouseup', this.boundFNs.stopMove, false);
			this.win.document.body.removeEventListener('mousemove', this.boundFNs.moveFocus, false);
			this.elems.image.className = '';
			this.elems.info.style.display = 'none';
		},
		mouseLeaveBody: function (e) {
			if (this.state.isMoveFocus) {
				this.moveFocusPosition(e);
			}
			this.stopMoveFocusPosition();
		},
		startMoveFocusPosition: function (e) {
			this.state.isMoveFocus = true;
			this.elems.image.style.cursor = 'move';
			this.win.document.body.style.cursor = 'move';
			this.vals.referenceX = e.clientX;
			this.vals.referenceY = e.clientY;
			this.vals.origLeft = parseInt(this.elems.focusPoint.style.left);
			this.vals.origTop = parseInt(this.elems.focusPoint.style.top);
			this.boundFNs.moveFocus = this.moveFocusPosition.bind(this);
			this.win.document.body.addEventListener('mousemove', (this.boundFNs.moveFocus = this.moveFocusPosition.bind(this)), false);
		},
		stopMoveFocusPosition: function (e) {
			this.state.isMoveFocus = false;
			this.elems.image.style.cursor = 'crosshair';
			this.win.document.body.style.cursor = 'default';
			this.win.document.body.removeEventListener('mousemove', this.boundFNs.moveFocus, false);
		},
		moveFocusPosition: function (e) {
			var topVal = this.vals.origTop + (e.clientY - this.vals.referenceY),
				leftVal = this.vals.origLeft + (e.clientX - this.vals.referenceX);

			//window.getSelection().removeAllRanges();
			this.setFocusPositionByMouse(null, topVal, leftVal);
		},
		setFocusPositionByMouse: function (e, topVal, leftVal) {
			topVal = topVal !== undefined ? topVal : (e ? e.offsetY : false);
			leftVal = leftVal !== undefined ? leftVal : (e ? e.offsetX : false);

			topVal = topVal <= 0 ? 0 : (topVal >= this.elems.image.height ? this.elems.image.height : topVal);
			leftVal = leftVal <= 0 ? 0 : (leftVal >= this.elems.image.width ? this.elems.image.width : leftVal);

			this.elems.x_focus.value = ((leftVal - this.elems.image.width / 2) / (this.elems.image.width / 2)).toFixed(2);
			this.elems.y_focus.value = ((topVal - this.elems.image.height / 2) / (this.elems.image.height / 2)).toFixed(2);
			this.setFocusPositionByValue();
		},
		setFocusPositionByValue: function () {
			var x = this.win.document.getElementById('x_focus').value,
				y = this.win.document.getElementById('y_focus').value;

			if (Math.abs(this.elems.x_focus) > 1) {
				this.elems.x_focus = 0;
			}
			if (Math.abs(y) > 1) {
				y = 0;
			}
			this.win.document.getElementById('focus').value = '[' + x + ',' + y + ']';

			this.elems.focusPoint.style.left = ((this.elems.image.width / 2) + (x * (this.elems.image.width / 2))).toFixed(0) + 'px';
			this.elems.focusPoint.style.top = ((this.elems.image.height / 2) + (y * (this.elems.image.height / 2))).toFixed(0) + 'px';
			this.win._EditorFrame.setEditorIsHot(true);
		}
	};

	this.Crop = {
		editTools: this,
		win: this.win,

		imgSrc: "",
		imgW: 0,
		imgH: 0,
		defW: 0,
		defH: 0,
		imgDiv: "weImgDiv",
		imgBorder: "weImagePanelBorder",
		imgPanel: "weImagePanel",
		imgID: "weImage",
		coverID: "weCoverPanel",
		cropID: "weCropPanel",
		cropCtrl: "weControl",
		sel: null,
		scale: null,
		initScale: null,
		prevScale: {x: 0, y: 0},
		scaleX: 0,
		scaleY: 0,
		coords: null,
		posX1: 0,
		posX2: 0,
		posY1: 0,
		posY2: 0,
		snap: 8,
		altKey: false,
		shiftKey: false,
		ctrlKey: false,
		triggered: false,
		eventHandler: {},
		crop: function (x, y, w, h) {
			if (this.editTools.activeTool === 'crop') {
				this.drop();
				return;
			}

			this.editTools.deactivateAll();
			this.editTools.setBtnActive('crop', true);
			this.editTools.activeTool = 'crop';

			this.getElements();

			var CropTool = this.editTools.Crop,
				elIdDiv = this.editTools.win.document.getElementById(this.imgDiv),
				elIdImage = this.editTools.win.document.getElementById(this.imgID),
				elIdBorder = this.editTools.win.document.getElementById(this.imgBorder),
				elCropCtrl = this.editTools.win.document.getElementById(this.cropCtrl),
				elIds = this.editTools.win.document.getElementById(this.imgPanel),
				elIdCrop = this.editTools.win.document.getElementById(this.cropID),
				elIdLeft = this.editTools.win.document.getElementById(this.sel.idLeft),
				elIdTop = this.editTools.win.document.getElementById(this.sel.idTop),
				elIdRight = this.editTools.win.document.getElementById(this.sel.idRight),
				elIdBottom = this.editTools.win.document.getElementById(this.sel.idBottom);

			elIdDiv.style.border = "2px solid #CECECE";
			elIdBorder.style.border = "1px solid #808080";
			this.triggered = true;
			this.defW = elIdImage.width;
			this.defH = elIdImage.height;
			elIdImage.width = this.editTools.size.origW;
			elIdImage.height = this.editTools.size.origH;
			this.imgW = elIdImage.offsetWidth;
			this.imgH = elIdImage.offsetHeight;
			this.imgSrc = elIdImage.src;
			elIdImage.style.cursor = "crosshair";
			elCropCtrl.style.display = "block";
			//this.getElements();
			elIds.style.cursor = "crosshair";

			this.addLs(elIds, "onmousedown", CropTool.setSelectionEnabled);
			this.addLs(elIds, "onmouseup", CropTool.handleMouseReleased);
			this.addLs(this.editTools.win.document, "onmouseup", CropTool.handleMouseReleased);
			this.addLs(this.editTools.win.document, "onmousemove", CropTool.handleMouseDragged);
			this.addLs(elIdCrop, "onmousedown", CropTool.setDragEnabled);
			this.addLs(elIds, "onclick", CropTool.handleMousePressed);
			this.addLs(this.editTools.win.document, "onkeydown", CropTool.handleKeyPressed);
			this.addLs(this.editTools.win.document, "onkeyup", CropTool.handleKeyReleased);
			this.addLs(elIdLeft, "onmousedown", CropTool.dragLeftEnabled);
			this.addLs(elIdTop, "onmousedown", CropTool.dragTopEnabled);
			this.addLs(elIdRight, "onmousedown", CropTool.dragRightEnabled);
			this.addLs(elIdBottom, "onmousedown", CropTool.dragBottomEnabled);

			if (arguments.length > 0) {
				CropTool.sel.draw(x, y, w, h);
				this.editTools.win.document.getElementById(CropTool.coverID).style.visibility = "visible";
				CropTool.sel.setCursor("e-resize", "n-resize", "e-resize", "n-resize", "move");
				CropTool.setCropSize(x, y, w, h);
			}

			this.resetCropTool();
			//this.imgDiv = null;
			//this.imgBorder = null;
			//this.elIdImage = null;
			//this.elCropCtrl = null;
			//this.elIds = null;
			//this.elIdCrop = null;
			this.elIdLeft = null;
			this.elIdTop = null;
			this.elIdRight = null;
			this.elIdBottom = null;
		},
		addLs: function (object, eventName, listener) {
			var
				CropTool = this.editTools.Crop,
				listeners = object[eventName + "listeners"],
				setListener = true;

			if (CropTool === null || CropTool === undefined) {
				CropTool = this.editTools.win;
			}
			if (!listeners) {
				this.getLs(object, eventName);
				listeners = object[eventName + "listeners"];
			}
			for (var i = 0; setListener && i < listeners.length; i++) {
				if (listeners[i][0] == listener && listeners[i][1] == CropTool) {
					setListener = false;
				}
			}
			if (setListener) {
				listeners[listeners.length] = [listener, CropTool];
			}
			return false;
		},
		clLs: function (object, eventName) {
			object[eventName + "listeners"] = [];
			return false;
		},
		getLs: function (object, eventName) {
			var CropTool = this.editTools.Crop;

			object[eventName + "listeners"] = [];
			if (typeof object[eventName] === "function") {
				object[eventName + "listeners"][0] = [object[eventName], object];
			}
			object[eventName] = function (a) {
				var
					i,
					argumentsCopy = Array.prototype.slice.call(arguments),
					listeners = this[eventName + "listeners"],
					listenersCopy = [];

				if (arguments.length === 0 && this.editTools.win.event) {
					argumentsCopy[0] = CropTool.patchEvent(this.editTools.win.event, this);
				} else if (a && typeof a === "object" && a.toString().search(/event/i) !== -1) {
					argumentsCopy[0] = CropTool.patchEvent(a, this);
				}

				for (i = 0; i < listeners.length; i++) {
					listenersCopy[i] = listeners[i];
				}
				for (i = 0; i < listenersCopy.length; i++) {
					listenersCopy[i][0].apply(listenersCopy[i][1], argumentsCopy);
				}
			};
		},
		patchEvent: function (evt, currentTarget) {
			if (!evt.target) {
				evt.target = evt.srcElement;
			}
			if (!evt.currentTarget) {
				evt.currentTarget = currentTarget;
			}
			if (evt.layerX === undefined) {
				evt.layerX = evt.offsetX;
			}
			if (evt.layerY === undefined) {
				evt.layerY = evt.offsetY;
			}
			if (evt.clientX === undefined) {
				evt.clientX = evt.pageX;
			}
			if (evt.clientY === undefined) {
				evt.clientY = evt.pageY;
			}
			if (!evt.stopPropagation) {
				evt.stopPropagation = function () {
					this.cancelBubble = true;
				};
			}
			if (!evt.preventDefault) {
				evt.preventDefault = function () {
					this.returnValue = false;
				};
			}
			return evt;
		},
		handleEvent: function (e, sourceElement) {
			if (!e.target) {
				e.target = e.srcElement;
			}
			if (!e.sourceElement) {
				e.sourceElement = sourceElement;
			}
			if (e.layerX === undefined) {
				e.layerX = e.offsetX;
			}
			if (e.layerY === undefined) {
				e.layerY = e.offsetY;
			}
			if (e.clientX === undefined) {
				e.clientX = e.pageX;
			}
			if (e.clientY === undefined) {
				e.clientY = e.pageY;
			}
			if (!e.stopPropagation) {
				e.stopPropagation = function () {
					this.cancelBubble = true;
				};
			}
			if (!e.preventDefault) {
				e.preventDefault = function () {
					this.returnValue = false;
				};
			}
			return e;
		},
		handleMouseDragged: function (e) {
			if (e.altKey) {
				this.altKey = e.altKey;
			}
			if (e.shiftKey) {
				this.shiftKey = e.shiftKey;
			}
			if (e.ctrlKey) {
				this.ctrlKey = e.ctrlKey;
			}
			if (this.sel.track || this.sel.drag || this.sel.dragLeft || this.sel.dragTop || this.sel.dragRight || this.sel.dragBottom) {
				e.preventDefault();
				this.eventHandler.clientX = e.clientX;
				this.eventHandler.clientY = e.clientY;
				if (this.sel.track) {
					this.repaint(this.eventHandler);
				} else if (this.sel.drag) {
					this.recalculateDrop(this.eventHandler);
				} else if (this.sel.dragLeft) {
					this.dragLeft(this.eventHandler);
					this.sel.setCursor("e-resize");
				} else if (this.sel.dragTop) {
					this.dragTop(this.eventHandler);
					this.sel.setCursor("n-resize");
				} else if (this.sel.dragRight) {
					this.dragRight(this.eventHandler);
					this.sel.setCursor("e-resize");
				} else if (this.sel.dragBottom) {
					this.dragBottom(this.eventHandler);
					this.sel.setCursor("n-resize");
				}
			}
			this.prevScale.x = e.clientX;
			this.prevScale.y = e.clientY;
			return false;
		},
		handleMouseReleased: function (e) {
			e.preventDefault();
			e.stopPropagation();
			var cover = this.editTools.win.document.getElementById(this.coverID);
			if (e.altKey) {
				this.altKey = e.altKey;
			}
			if (e.shiftKey) {
				this.shiftKey = e.shiftKey;
			}
			if (e.ctrlKey) {
				this.ctrlKey = e.ctrlKey;
			}
			if (this.sel.track) {
				if (this.initScale.x == e.clientX && this.initScale.y == e.clientY) {
					this.resetCropTool();
				} else {
					this.repaint(e);
					cover.style.visibility = "visible";
					this.prevScale.x = e.clientX;
					this.prevScale.y = e.clientY;
				}
			} else if (e.target.id == this.coverID || e.target.id == this.imgID) {
				this.resetCropTool();
				this.editTools.win.document.getElementById("console").style.display = "none";
				WE().layout.button.switch_button_state(this.editTools.win.document, "save", "disabled");
			}
			this.sel.setCursor("e-resize", "n-resize", "e-resize", "n-resize", "move", "pointer");
			this.sel.track = false;
			this.sel.drag = false;
			this.sel.dragLeft = false;
			this.sel.dragTop = false;
			this.sel.dragRight = false;
			this.sel.dragBottom = false;
			cover = null;
			return false;
		},
		handleMousePressed: function (e) {
			e.preventDefault();
			e.stopPropagation();
			return false;
		},
		handleKeyPressed: function (e) {
			if (!this.altKey && e.altKey || !this.shiftKey && e.shiftKey || !this.ctrlKey && e.ctrlKey) {
				this.handleKeystroke(e);
			}
		},
		handleKeyReleased: function (e) {
			if (this.altKey && !e.altKey || this.shiftKey && !e.shiftKey || this.ctrlKey && !e.ctrlKey) {
				this.handleKeystroke(e);
			}
		},
		handleKeystroke: function (e) {
			this.altKey = e.altKey;
			this.shiftKey = e.shiftKey;
			this.ctrlKey = e.ctrlKey;
			if (this.sel.track || this.sel.drag) {
				e.preventDefault();
				this.eventHandler.preventDefault = function () {
				};
				this.eventHandler.clientX = this.prevScale.x;
				this.eventHandler.clientY = this.prevScale.y;
				this.eventHandler.altKey = e.altKey;
				this.eventHandler.shiftKey = e.shiftKey;
				this.eventHandler.ctrlKey = e.ctrlKey;
			}
			if (this.sel.track) {
				this.repaint(this.eventHandler);
			} else if (this.sel.drag) {
				this.handleMouseDragged(this.eventHandler);
			}
		},
		setSelectionEnabled: function (e) {
			e.preventDefault();
			e.stopPropagation();
			if (e.altKey) {
				this.altKey = e.altKey;
			}
			if (e.shiftKey) {
				this.shiftKey = e.shiftKey;
			}
			if (e.ctrlKey) {
				this.ctrlKey = e.ctrlKey;
			}
			if (this.sel.getVisibility() !== "visible") {
				this.scale = {x: e.layerX + 1, y: e.layerY + 1};
				this.initScale = {x: e.clientX, y: e.clientY};
				this.prevScale.x = e.clientX;
				this.prevScale.y = e.clientY;
				this.sel.draw(e.layerX, e.layerY, 1, 1);
				this.sel.track = true;
			}
			return false;
		},
		setDragEnabled: function (e) {
			e.preventDefault();
			if (e.altKey) {
				this.altKey = e.altKey;
			}
			if (e.shiftKey) {
				this.shiftKey = e.shiftKey;
			}
			if (e.ctrlKey) {
				this.ctrlKey = e.ctrlKey;
			}
			this.coords = {x: e.clientX, y: e.clientY};
			this.posX1 = this.sel.getLeft();
			this.posX2 = this.sel.getRight();
			this.posY1 = this.sel.getTop();
			this.posY2 = this.sel.getBottom();
			this.sel.drag = true;
			return false;
		},
		dragLeftEnabled: function (e) {
			e.preventDefault();
			this.coords = {x: e.clientX, y: e.clientY};
			this.posX1 = this.sel.getLeft();
			this.sel.dragLeft = true;
			return false;
		},
		dragLeft: function (e) {
			var
				CropTool = this.editTools.Crop,
				x1 = this.posX1 + e.clientX - this.coords.x,
				old_x1 = this.sel.getLeft(),
				old_w = this.sel.getWidth();
			var w = old_w - (x1 - old_x1);
			//this.sel.setLeft(x1);
			this.setCropCoordX(x1);
			CropTool.setCropWidth(w);
		},
		dragTopEnabled: function (e) {
			e.preventDefault();
			this.coords = {x: e.clientX, y: e.clientY};
			this.posY1 = this.sel.getTop();
			this.sel.dragTop = true;
			return false;
		},
		dragTop: function (e) {
			var
				CropTool = this.editTools.Crop,
				y1 = this.posY1 + e.clientY - this.coords.y,
				old_h = this.sel.getHeight(),
				old_y1 = this.sel.getTop();
			var h = old_h - (y1 - old_y1);
			//this.sel.setTop(y1);
			this.setCropCoordY(y1);
			CropTool.setCropHeight(h);
		},
		dragRightEnabled: function (e) {
			e.preventDefault();
			this.coords = {x: e.clientX, y: e.clientY};
			this.posX2 = this.sel.getRight();
			this.sel.dragRight = true;
			return false;
		},
		dragRight: function (e) {
			var CropTool = this.editTools.Crop,
				x1 = this.sel.getLeft();
			var new_x1 = x1 + e.clientX - this.coords.x;
			var new_x2 = this.sel.getRight();
			var orig_x2 = this.posX2;
			var orig_w = orig_x2 - x1;
			var old_w = this.sel.getWidth();
			var w = old_w - (new_x2 - new_x1) + orig_w;
			//var x2 = x1 + w;
			this.sel.setRight(new_x2);
			CropTool.setCropWidth(w);
		},
		dragBottomEnabled: function (e) {
			e.preventDefault();
			this.coords = {x: e.clientX, y: e.clientY};
			this.posY2 = this.sel.getBottom();
			this.sel.dragBottom = true;
			return false;
		},
		dragBottom: function (e) {
			var
				CropTool = this.editTools.Crop,
				y1 = this.sel.getTop(),
				new_y2 = this.sel.getBottom(),
				orig_y2 = this.posY2,
				old_h = this.sel.getHeight();
			var new_y1 = y1 + e.clientY - this.coords.y;
			var orig_h = orig_y2 - y1;
			var h = old_h - (new_y2 - new_y1) + orig_h;
			//var y2 = y1 + h;
			this.sel.setBottom(new_y2);
			CropTool.setCropHeight(h);
		},
		recalculateDrop: function (e) {
			var
				x1 = this.posX1 + e.clientX - this.coords.x,
				y1 = this.posY1 + e.clientY - this.coords.y,
				w = this.sel.getWidth(),
				h = this.sel.getHeight();
			var edge_x1 = x1;

			if (edge_x1 < 0 || edge_x1 <= this.snap && !this.ctrlKey) {
				x1 = 0;
			}
			var edge_y1 = y1;
			if (edge_y1 < 0 || edge_y1 <= this.snap && !this.ctrlKey) {
				y1 = 0;
			}
			var edge_x2 = this.imgW - w - x1;
			if (edge_x2 < 0 || edge_x2 <= this.snap && !this.ctrlKey) {
				x1 += edge_x2;
			}
			var edge_y2 = this.imgH - h - y1;
			if (edge_y2 < 0 || edge_y2 <= this.snap && !this.ctrlKey) {
				y1 += edge_y2;
			}
			this.sel.moveTo(x1, y1);
			this.setCropSize(x1, y1);
		},
		repaint: function (e) {
			if (this.altKey) {
				this.scaleX += e.clientX - this.prevScale.x;
				this.scaleY += e.clientY - this.prevScale.y;
			}
			var
				invertH = (e.clientX - (this.initScale.x + this.scaleX) < 0),
				invertV = (e.clientY - (this.initScale.y + this.scaleY) < 0),
				w = Math.abs(e.clientX - (this.initScale.x + this.scaleX)),
				h = Math.abs(e.clientY - (this.initScale.y + this.scaleY)),
				x1 = this.scale.x + this.scaleX,
				y1 = this.scale.y + this.scaleY;

			if (invertH) {
				x1 -= w;
			}
			if (invertV) {
				y1 -= h;
			}
			var edge_x1 = x1;
			if (edge_x1 < 0 || edge_x1 <= this.snap && !this.ctrlKey) {
				w += edge_x1;
				x1 = 0;
			}
			var edge_y1 = y1;
			if (edge_y1 < 0 || edge_y1 <= this.snap && !this.ctrlKey) {
				h += edge_y1;
				y1 = 0;
			}
			var edge_x2 = this.imgW - w - x1;
			if (edge_x2 < 0 || edge_x2 <= this.snap && !this.ctrlKey) {
				w += edge_x2;
			}
			var edge_y2 = this.imgH - h - y1;
			if (edge_y2 < 0 || edge_y2 <= this.snap && !this.ctrlKey) {
				h += edge_y2;
			}
			if (this.shiftKey) {
				if (w > h) {
					if (invertH) {
						x1 += w - h;
					}
					w = h;
				}
				if (h > w) {
					if (invertV) {
						y1 += h - w;
					}
					h = w;
				}
			}
			this.sel.draw(x1, y1, w, h);
			this.setCropSize(x1, y1, w, h);
			this.editTools.win.document.getElementById("console").style.display = "block";
			WE().layout.button.switch_button_state(this.editTools.win.document, "save", "enabled");
		},
		setCropSize: function (x1, y1, w, h) {
			this.editTools.win.document.forms.we_form.cropCoordX.value = x1;
			this.editTools.win.document.forms.we_form.cropCoordY.value = y1;
			if (arguments.length > 2) {
				this.setCropWidth(w);
				this.setCropHeight(h);
			}
		},
		setCropCoordX: function (x1) {
			x1 = Math.round(Number(x1));
			var
				y1 = this.sel.getTop(),
				w = this.sel.getWidth(),
				h = this.sel.getHeight(),
				cover = this.editTools.win.document.getElementById(this.coverID);

			if ((x1 - x1 !== 0) || (x1 < 0) || (x1 > this.imgW)) {
				if (typeof this.sel.getLeft() !== "number") {
					this.editTools.win.document.forms.we_form.cropCoordX.value = "";
				} else {
					this.editTools.win.document.forms.we_form.cropCoordX.value = this.sel.getLeft();
				}
				return;
			}
			this.sel.setLeft(x1);
			this.sel.setRight(x1 + w);
			this.editTools.win.document.forms.we_form.cropCoordX.value = x1;
			if (typeof w === "number" && x1 + w > this.imgW) {
				w = this.imgW - x1;
			}
			if (typeof w === "number" && typeof y1 === "number" && typeof h === "number") {
				this.sel.draw(x1, y1, w, h);
				cover.style.visibility = "visible";
			}
		},
		setCropCoordY: function (y1) {
			y1 = Math.round(Number(y1));
			var
				x1 = this.sel.getLeft(),
				w = this.sel.getWidth(),
				h = this.sel.getHeight(),
				cov = this.editTools.win.document.getElementById(this.coverID);

			if ((y1 - y1 !== 0) || (y1 < 0) || (y1 > this.imgH)) {
				if (typeof this.sel.getTop() !== "number") {
					this.editTools.win.document.forms.we_form.cropCoordY.value = "";
				} else {
					this.editTools.win.document.forms.we_form.cropCoordY.value = this.sel.getTop();
				}
				return;
			}
			this.sel.setTop(y1);
			this.editTools.win.document.forms.we_form.cropCoordY.value = y1;
			if (typeof h === "number" && y1 + h > this.imgH) {
				h = this.imgH - y1;
			}
			if (typeof x1 === "number" && typeof w === "number" && typeof h === "number") {
				this.sel.draw(x1, y1, w, h);
				cov.style.visibility = "visible";
			}
		},
		setCropWidth: function (w) {
			w = Math.round(Number(w));
			w = (w > 0) ? w : 1;
			var
				x1 = this.sel.getLeft(),
				y1 = this.sel.getTop(),
				h = this.sel.getHeight(),
				cov = this.editTools.win.document.getElementById(this.coverID);


			if ((w - w !== 0) || (w < 1) || (w > this.imgW)) {
				if (typeof this.sel.getWidth() !== "number") {
					this.editTools.win.document.forms.we_form.CropWidth.value = "";
				} else {
					this.editTools.win.document.forms.we_form.CropWidth.value = this.sel.getWidth();
				}
				return;
			}
			this.sel.setWidth(w);
			this.editTools.win.document.forms.we_form.CropWidth.value = w;
			if (typeof x1 === "number" && x1 + w > this.imgW) {
				x1 = this.imgW - w;
			}
			if (typeof x1 === "number" && typeof y1 === "number" && typeof h === "number") {
				this.sel.draw(x1, y1, w, h);
				cov.style.visibility = "visible";
			}
		},
		setCropHeight: function (h) {
			h = Math.round(Number(h));
			h = (h > 0) ? h : 1;
			var
				x1 = this.sel.getLeft(),
				y1 = this.sel.getTop(),
				w = this.sel.getWidth(),
				cov = this.editTools.win.document.getElementById(this.coverID);

			if ((h - h !== 0) || (h < 1) || (h > this.imgH)) {
				if (typeof this.sel.getHeight() !== "number") {
					this.editTools.win.document.forms.we_form.CropHeight.value = "";
				} else {
					this.editTools.win.document.forms.we_form.CropHeight.value = this.sel.getHeight();
				}
				return;
			}
			this.sel.setHeight(h);
			this.editTools.win.document.forms.we_form.CropHeight.value = h;
			if (typeof y1 === "number" && y1 + h > this.imgH) {
				y1 = this.imgH - h;
			}
			if (typeof x1 === "number" && typeof y1 === "number" && typeof w === "number") {
				this.sel.draw(x1, y1, w, h);
				cov.style.visibility = "visible";
			}
		},
		getElements: function () {
			var
				CropTool = this.editTools.Crop,
				xy = this.editTools.win.document.getElementById("weImagePanel");

			this.sel = {
				left: undefined,
				top: undefined,
				right: undefined,
				bottom: undefined,
				width: undefined,
				height: undefined,
				idLeft: "divSelectionLeft",
				idTop: "divSelectionTop",
				idRight: "divSelectionRight",
				idBottom: "divSelectionBottom",
				visible: false,
				win: this.editTools.win
			};

			var x1 = this.editTools.win.document.createElement("div");
			x1.id = this.sel.idLeft;
			x1.className = "clVerLine";
			xy.appendChild(x1);

			var y1 = this.editTools.win.document.createElement("div");
			y1.id = this.sel.idTop;
			y1.className = "clHorLine";
			xy.appendChild(y1);

			var x2 = this.editTools.win.document.createElement("div");
			x2.id = this.sel.idRight;
			x2.className = "clVerLine";
			xy.appendChild(x2);

			var y2 = this.editTools.win.document.createElement("div");
			y2.id = this.sel.idBottom;
			y2.className = "clHorLine";
			xy.appendChild(y2);

			var img = this.editTools.win.document.createElement("img");
			img.id = this.cropID;
			img.src = this.imgSrc;
			xy.appendChild(img);

			var cov = this.editTools.win.document.createElement("div");
			cov.id = this.coverID;
			cov.style.width = this.imgW + "px";
			cov.style.height = this.imgH + "px";
			xy.appendChild(cov);

			this.sel.setVisibility = function (v) {
				this.win.document.getElementById(this.idLeft).style.visibility = v;
				this.win.document.getElementById(this.idTop).style.visibility = v;
				this.win.document.getElementById(this.idRight).style.visibility = v;
				this.win.document.getElementById(this.idBottom).style.visibility = v;
				this.win.document.getElementById(CropTool.cropID).style.visibility = v;
				this.visibility = v;
			};

			this.sel.setCursor = function (x1, y1, x2, y2, cr, co) {
				if (arguments.length === 1) {
					y1 = x2 = y2 = cr = co = x1;
				}
				this.win.document.getElementById(this.idLeft).style.cursor = x1;
				this.win.document.getElementById(this.idTop).style.cursor = y1;
				this.win.document.getElementById(this.idRight).style.cursor = x2;
				this.win.document.getElementById(this.idBottom).style.cursor = y2;
				this.win.document.getElementById(CropTool.cropID).style.cursor = cr;
				this.win.document.getElementById(CropTool.coverID).style.cursor = co;
			};

			this.sel.setLeft = function (x1) {
				this.left = this.right = x1;
			};
			this.sel.setTop = function (y1) {
				this.top = y1;
			};
			this.sel.setRight = function (x2) {
				this.right = x2;
			};
			this.sel.setBottom = function (y2) {
				this.bottom = y2;
			};
			this.sel.setWidth = function (w) {
				this.width = w;
			};
			this.sel.setHeight = function (h) {
				this.height = h;
			};

			this.sel.getVisibility = function () {
				return this.visibility;
			};
			this.sel.getLeft = function () {
				return this.left;
			};
			this.sel.getTop = function () {
				return this.top;
			};
			this.sel.getRight = function () {
				return this.right;
			};
			this.sel.getBottom = function () {
				return this.bottom;
			};
			this.sel.getWidth = function () {
				return this.width;
			};
			this.sel.getHeight = function () {
				return this.height;
			};

			this.sel.draw = function (x1, y1, w, h) {
				this.left = x1;
				this.top = y1;
				this.width = w;
				this.height = h;
				this.right = x1 + w;
				this.bottom = y1 + h;

				var elCrop = this.win.document.getElementById(CropTool.cropID);
				var elLeft = this.win.document.getElementById(this.idLeft);
				var elTop = this.win.document.getElementById(this.idTop);
				var elRight = this.win.document.getElementById(this.idRight);
				var elBottom = this.win.document.getElementById(this.idBottom);

				elLeft.style.left = x1 + "px";
				elTop.style.left = x1 + "px";
				elRight.style.left = x1 + w - 1 + "px";
				elBottom.style.left = x1 + "px";

				elLeft.style.top = y1 + "px";
				elTop.style.top = y1 + "px";
				elRight.style.top = y1 + "px";
				elBottom.style.top = y1 + h - 1 + "px";

				elLeft.style.height = h + "px";
				elTop.style.width = w + "px";
				elRight.style.height = h + "px";
				elBottom.style.width = w + "px";

				elCrop.style.clip = "rect(" + y1 + "px, " + (x1 + w) + "px, " + (y1 + h) + "px, " + x1 + "px)";

				this.setVisibility("visible");
				elCrop = null;
				elLeft = null;
				elTop = null;
				elRight = null;
				elBottom = null;
			};

			this.sel.moveTo = function (x1, y1) {
				var w = this.getWidth();
				var h = this.getHeight();
				var elCrop = this.win.document.getElementById(CropTool.cropID);
				var elLeft = this.win.document.getElementById(this.idLeft);
				var elTop = this.win.document.getElementById(this.idTop);
				var elRight = this.win.document.getElementById(this.idRight);
				var elBottom = this.win.document.getElementById(this.idBottom);

				this.left = x1;
				this.right = x1 + w;
				this.top = y1;

				elLeft.style.left = x1 + "px";
				elTop.style.left = x1 + "px";
				elRight.style.left = x1 + w - 1 + "px";
				elBottom.style.left = x1 + "px";

				elLeft.style.top = y1 + "px";
				elTop.style.top = y1 + "px";
				elRight.style.top = y1 + "px";
				elBottom.style.top = y1 + h - 1 + "px";

				elCrop.style.clip = "rect(" + y1 + "px, " + (x1 + w) + "px, " + (y1 + h) + "px, " + x1 + "px)";

				elCrop = null;
				elLeft = null;
				elTop = null;
				elRight = null;
				elBottom = null;
			};

			this.sel.reset = function () {
				this.left = undefined;
				this.top = undefined;
				this.right = undefined;
				this.bottom = undefined;
				this.width = undefined;
				this.height = undefined;
				this.setVisibility("hidden");
				this.setCursor("crosshair");
				this.track = false;
				this.drag = false;
			};
			xy = null;
		},
		zoom: function (px) {
			var w = this.sel.getWidth();
			var h = this.sel.getHeight();
			if (w !== undefined || h !== undefined) {
				this.setCropWidth(w + px);
				this.setCropHeight(h + px);
			}
		},
		zoomH: function (px) {
			var w = this.sel.getWidth();
			if (w !== undefined) {
				this.setCropWidth(w + px);
			}
		},
		zoomV: function (px) {
			var h = this.sel.getHeight();
			if (h !== undefined) {
				this.setCropHeight(h + px);
			}
		},
		switch_button_state: function (element, state) {
			if (state === "enabled") {
				WE().layout.button.enable(this.editTools.win.document, element);
				return true;
			}
			if (state === "disabled") {
				WE().layout.button.disable(this.editTools.win.document, element);
				return false;
			}
			return false;
		},
		catchKeystroke: function (e, inp) {
			var keynum, keychar, numcheck,
				CropTool = this.editTools.Crop;
			if (this.editTools.win.event) {
				keynum = e.keyCode;
			} else if (e.which) {
				keynum = e.which;
			}
			switch (keynum) {
				case 13:
					CropTool["set" + inp.name](inp.value);
					return true;
				case 8:
					return true;
				default:
					keychar = String.fromCharCode(keynum);
					numcheck = /\d/;
					return numcheck.test(keychar);
			}
		},
		resetCropTool: function () {
			this.scale = null;
			this.initScale = null;
			this.setCropSize("", "", "", "");
			this.sel.reset();
			this.editTools.win.document.getElementById(this.coverID).style.visibility = "hidden";
			this.altKey = false;
			this.ctrlKey = false;
			this.shiftKey = false;
		},
		drop: function () {
			var elIdDiv = this.editTools.win.document.getElementById(this.imgDiv),
				elIdBorder = this.editTools.win.document.getElementById(this.imgBorder),
				elCropCtrl = this.editTools.win.document.getElementById(this.cropCtrl),
				elIdImage = this.editTools.win.document.getElementById(this.imgID),
				elIds = this.editTools.win.document.getElementById(this.imgPanel),
				elIdCrop = this.editTools.win.document.getElementById(this.cropID),
				elIdLeft = this.editTools.win.document.getElementById(this.sel.idLeft),
				elIdTop = this.editTools.win.document.getElementById(this.sel.idTop),
				elIdRight = this.editTools.win.document.getElementById(this.sel.idRight),
				elIdBottom = this.editTools.win.document.getElementById(this.sel.idBottom);

			this.editTools.activeTool = this.editTools.activeTool === 'crop' ? '' : this.editTools.activeTool;
			this.editTools.setBtnActive('crop', false);

			elIdImage.width = this.defW;
			elIdImage.height = this.defH;
			elIdDiv.style.border = "0px solid #CECECE";
			elIdBorder.style.border = "0px solid #808080";
			elCropCtrl.style.display = "none";
			elIdImage.style.cursor = "default";
			elIds.style.cursor = "default";

			this.clLs(elIds, "onmousedown");
			this.clLs(elIds, "onmouseup");
			this.clLs(this.editTools.win, "onmouseup");
			this.clLs(this.editTools.win, "onmousemove");
			this.clLs(elIdCrop, "onmousedown");
			this.clLs(elIds, "onclick");
			this.clLs(this.editTools.win, "onkeydown");
			this.clLs(this.editTools.win, "onkeyup");
			this.clLs(elIdLeft, "onmousedown");
			this.clLs(elIdTop, "onmousedown");
			this.clLs(elIdRight, "onmousedown");
			this.clLs(elIdBottom, "onmousedown");

			this.resetCropTool();
			elIdDiv = null;
			elIdBorder = null;
			elCropCtrl = null;
			elIdImage = null;
			elIds = null;
			elIdCrop = null;
			elIdLeft = null;
			elIdTop = null;
			elIdRight = null;
			elIdBottom = null;
		}
	};

	this.EditFull = {
		editTools: this,
		win: this.win,
		doc: null,
		metaData: null,
		uploader: null,
		imageEditor: null,
		intervalId: 0,

		start: function () {
			if (this.editTools.activeTool === 'editFull') {
				this.drop();
				return;
			}

			this.editTools.deactivateAll();
			this.uploader = this.win.weFileUpload_instance;
			this.editTools.setBtnActive('editFull', true);
			this.editTools.activeTool = 'editFull';
			this.doc = this.win.doc;
			this.win.document.getElementById('weImgEditor').style.display = "none";
			this.win.document.getElementById('tui-image-editor-top').style.display = "block";
			var el = this.win.document.getElementById('tui-image-editor-container');
			el.style.display = "block";

			this.imageEditor = new this.win.tui.ImageEditor('#tui-image-editor-container', {
				includeUI: {
					loadImage: {
						path: el.getAttribute('data-url'),
						name: 'weFile'
					},
					//theme: blackTheme, // or whiteTheme
					//initMenu: 'filter',
					menuBarPosition: 'bottom'
				},
				//cssMaxWidth: 700,
				//cssMaxHeight: 500,
				usageStatistics: false
			});
			this.win.onresize = function () {
				this.doc.imageEditor.ui.resizeEditor();
			};
			this.startCheckDocumentHot();
		},
		isActive: function () {
			return this.editTools.activeTool === 'editFull' && this.imageEditor;
		},
		isHot: function () {
			return this.isActive() && !this.imageEditor.isEmptyUndoStack();
		},
		unsetHot: function () {
			this.imageEditor.clearUndoStack();
			this.imageEditor.clearRedoStack();
			this.startCheckDocumentHot();
		},
		startCheckDocumentHot: function () {
			this.stopCheckDocumentHot();
			this.intervalId = this.win.setInterval(this.checkDocumentHot.bind(this), 1000);
		},
		stopCheckDocumentHot: function () {
			this.win.clearInterval(this.intervalId);
		},
		checkDocumentHot: function () {
			// after saving _EditorFrame is not immediately unset hot: so we cannot take this as condition!
			if (this.isHot()/* || this.win._EditorFrame.getEditorIsHot()*/) {
				this.win._EditorFrame.setEditorIsHot(true);
				this.stopCheckDocumentHot();
			}
		},
		upload: function (nextCmd, isAsked) {
			this.uploader = this.win.weFileUpload_instance;
			switch (nextCmd && nextCmd.length ? nextCmd[0] : '') {
				case 'we_save_document':
				case 'switch_edit_page':
				case 'switch_edit_tool':
					this.uploader.uploadFromUrlAndData({
						url: this.win.doc.docPath,
						name: this.win.doc.docText,
						transaction: this.win.doc.we_transaction,
						cmdUploadFinished: nextCmd,
						externallyEditedData: this.imageEditor.toDataURL({
							format: this.win.doc.gdType === 'jpg' ? 'jpeg' : (this.win.doc.gdType === 'png' ? 'png' : ''),
							quality: 1//0..1
						})
					});
					break;
			}
			this.unsetHot();
		},
		askForUpload: function (nextCmd) {
			switch (nextCmd && nextCmd.length ? nextCmd[0] : '') {
				case 'switch_edit_page':
				case 'switch_edit_tool':
					this.win.we_cmd('editFull_askForUpload', nextCmd.join(','));
					break;
			}
		},
		drop: function () {
			this.editTools.activeTool = this.editTools.activeTool === 'editFull' ? '' : this.editTools.activeTool;
			this.editTools.setBtnActive('editFull', false);
			this.stopCheckDocumentHot();
			this.imageEditor = null;
			this.win.document.getElementById('weImgEditor').style.display = 'block';
			this.win.document.getElementById('tui-image-editor-top').style.display = 'none';
		}
	};

	/*
	 this.convertPNG = {};
	 this.convertGIF = {};
	 */
};

