/* global WE, errorHandler,treeData,drawTree, top */

/**
 * webEdition CMS
 *
 * $Rev: 6329b796be62 $
 * $Author: Lukas Imhof $
 * $Date: 2020-11-10 16:09 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';
/**
 * show a confirm dialog
 * @param {window} win window calling
 * @param {string} title title of the dialog, if none "question" is set
 * @param {string} message the message to display
 * @param {array} yesCmd we-command on yes
 * @param {array} noCmd we-command on no
 * @param {string} yesText text for button yes, if other
 * @param {string} noText text for button no, if other
 * @returns {undefined}
 */
WE().util.showConfirm = function (win, title, message, yesCmd, noCmd, yesText, noText) {
	title = title ? title : WE().consts.g_l.message_reporting.question;
	var ab;

	if (win.top.top.$) {
		ab = win.top.top.$("#alertBox");
		if (!ab.length) {
			var alertDiv = win.top.top.document.createElement('dialog');
			alertDiv.id = "alertBox";
			win.top.top.document.body.appendChild(alertDiv);
			ab = win.top.top.$("#alertBox");
		}
		if (ab.html()) {//still a present dialog
			var el = ab.get(0);
			if (!el.stack) {
				el.stack = [];
			}
			el.stack.push(['showConfirm', Array.prototype.slice.call(arguments)]);
			return;
		}
		ab[0].data = {
			win: win,
			yesCmd: yesCmd,
			noCmd: noCmd
		};
		var
			yesBut = {
				text: (yesText ? yesText : WE().consts.g_l.message_reporting.yes),
				icons: {
					primary: "fa fa-check fa-ok"
				},
				click: function () {
					var ab = this.ownerDocument.defaultView.$("#alertBox");
					if (ab[0].data.yesCmd) {
						if (!ab[0].data.win.we_cmd) {
							WE().t_e('we_cmd not found', ab[0].data.yesCmd);
							//try top!
							ab[0].data.win.top.we_cmd.apply(ab[0].data.win, ab[0].data.yesCmd);
						} else {
							ab[0].data.win.we_cmd.apply(ab[0].data.win, ab[0].data.yesCmd);
						}
					}
					ab[0].data = null;
					ab.dialog("close");
				}
			},
			noBut = {
				text: (noText ? noText : WE().consts.g_l.message_reporting.no),
				icons: {
					primary: "fa fa-close fa-cancel"
				},
				click: function () {
					var ab = this.ownerDocument.defaultView.$("#alertBox");
					if (ab[0].data.noCmd) {
						ab[0].data.win.we_cmd.apply(ab[0].data.win, ab[0].data.noCmd);
					}
					ab[0].data = null;
					ab.dialog("close");
				}
			},
			cancelBut = {
				text: WE().consts.g_l.message_reporting.cancel,
				icons: {
					primary: "fa fa-close fa-ban"
				},
				click: function () {
					var ab = this.ownerDocument.defaultView.$("#alertBox");
					ab[0].data = null;
					ab.dialog("close");
				}
			};

		ab.html('<span class="alertIcon fa-stack fa-lg" style="color:#F2F200;"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i></span><div class="message">' + message.replace(/\\n|\\\\n/g, '<br/>') + "</div>");
		ab.dialog({
			dialogClass: "no-close",
			modal: true,
			title: title,
			height: "auto",
			maxHeight: 400,
			width: "auto",
			maxWidth: 400,
			closeOnEscape: true,
			buttons: (WE().session.isMac ?
				(noCmd ? [noBut, cancelBut, yesBut] : [noBut, yesBut]) :
				(noCmd ? [yesBut, noBut, cancelBut] : [yesBut, noBut])
				),
			open: function () {
				$(this).parent().find('.ui-dialog-buttonpane button:eq(0)').focus();
			},
			close: function () {
				this.innerHTML = "";
				if (this.stack && this.stack.length) {
					var call = this.stack.shift();
					WE().util[call[0]].apply(window, call[1]);
				}
			}
		});
		return;
	}
	//fallback
	message = (title ? title + ":\n" : "") + message;
	if (win.confirm(message)) {
		if (yesCmd) {
			win.we_cmd(yesCmd);
		}
	} else {
		if (noCmd) {
			win.we_cmd(noCmd);
		}
	}

};